<?php

/**
 * Add footer html from theme settings
 * @version 1.0
 */
function helo_essential_theme_option_footer_code() {
	$html = helo_option( 'opt-tabbed-code' );
	// html
	if ( is_array( $html ) && array_key_exists( 'opt_code_editor_html', $html ) && $html['opt_code_editor_html'] != '' ) {

		libxml_use_internal_errors( true );
		$dom = new DOMDocument();
		$dom->loadHTML( $html['opt_code_editor_html'] );
		if ( empty( libxml_get_errors() ) ) {
			echo $html['opt_code_editor_html'];
		}

	}
	// js
	if ( is_array( $html ) && array_key_exists( 'opt_code_editor_js', $html ) && $html['opt_code_editor_js'] != '' ) {
		echo '<script>';
		echo $html['opt_code_editor_js'];
		echo '</script>';
	}

}

add_action( 'wp_footer', 'helo_essential_theme_option_footer_code' );

add_filter( 'nav_menu_css_class', 'helo_essential_add_theme_menu_item_extra_cls', 10, 2 );

function helo_essential_add_theme_menu_item_extra_cls( $classes, $item ) {

	$item_cls = get_post_meta( $item->ID, '_crowdyflow_theme_menu_item_cls', true );
	if ( $item_cls != '' ) {
		$classes[] = $item_cls;
	}

	return $classes;
}

function helo_essential_theme_option_page_footer_code() {

	if ( is_page() ) {
		$html = helo_meta_option( get_the_id(), 'opt-tabbed-code', false );
		// html
		if ( is_array( $html ) && array_key_exists( 'opt_code_editor_html', $html ) && $html['opt_code_editor_html'] != '' ) {
			libxml_use_internal_errors( true );
			$dom = new DOMDocument();
			$dom->loadHTML( $html['opt_code_editor_html'] );
			if ( empty( libxml_get_errors() ) ) {
				echo $html['opt_code_editor_html'];
			}

		}
		// js
		if ( is_array( $html ) && array_key_exists( 'opt_code_editor_js', $html ) && $html['opt_code_editor_js'] != '' ) {
			echo '<script>';
			echo $html['opt_code_editor_js'];
			echo '</script>';
		}
	}

}

add_action( 'wp_footer', 'helo_essential_theme_option_page_footer_code' );

function wcf_wp_ajax_update_theme() {

	if ( empty( $_POST['slug'] ) ) {
		wp_send_json_error(
			array(
				'slug'         => '',
				'errorCode'    => 'no_theme_specified',
				'errorMessage' => __( 'No theme specified.', 'helo-essential' ),
			)
		);
	}

	$stylesheet = preg_replace( '/[^A-z0-9_\-]/', '', wp_unslash( $_POST['slug'] ) );
	$status     = array(
		'update'     => 'theme',
		'slug'       => $stylesheet,
		'oldVersion' => '',
		'newVersion' => '',
	);

	if ( ! current_user_can( 'update_themes' ) ) {
		$status['errorMessage'] = __( 'Sorry, you are not allowed to update themes for this site.' );
		wp_send_json_error( $status );
	}

	$theme = wp_get_theme( $stylesheet );
	if ( $theme->exists() ) {
		$status['oldVersion'] = $theme->get( 'Version' );
	}

	require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';

	$current = get_site_transient( 'update_themes' );
	if ( empty( $current ) ) {
		wp_update_themes();
	}

	$skin     = new WP_Ajax_Upgrader_Skin();
	$upgrader = new Theme_Upgrader( $skin );
	$result   = $upgrader->bulk_upgrade( array( $stylesheet ) );

	if ( defined( 'WP_DEBUG' ) && WP_DEBUG ) {
		$status['debug'] = $skin->get_upgrade_messages();
	}

	if ( is_wp_error( $skin->result ) ) {

		$status['errorCode']    = $skin->result->get_error_code();
		$status['errorMessage'] = $skin->result->get_error_message();
		wp_send_json_error( $status );

	} elseif ( $skin->get_errors()->has_errors() ) {

		$status['errorMessage'] = $skin->get_error_messages();
		wp_send_json_error( $status );

	} elseif ( is_array( $result ) && ! empty( $result[ $stylesheet ] ) ) {
		// Theme is already at the latest version.
		if ( true === $result[ $stylesheet ] ) {
			$status['errorMessage'] = $upgrader->strings['up_to_date'];
			wp_send_json_error( $status );
		}

		$theme = wp_get_theme( $stylesheet );
		if ( $theme->exists() ) {
			$status['newVersion'] = $theme->get( 'Version' );
		}
		wp_send_json_success( $status );

	} elseif ( false === $result ) {

		global $wp_filesystem;
		$status['errorCode']    = 'unable_to_connect_to_filesystem';
		$status['errorMessage'] = __( 'Unable to connect to the filesystem. Please confirm your credentials.', 'helo-essential' );

		// Pass through the error from WP_Filesystem if one was raised.
		if ( $wp_filesystem instanceof WP_Filesystem_Base && is_wp_error( $wp_filesystem->errors ) && $wp_filesystem->errors->has_errors() ) {
			$status['errorMessage'] = esc_html( $wp_filesystem->errors->get_error_message() );
		}

		wp_send_json_error( $status );
	}

	// An unhandled error occurred.
	$status['errorMessage'] = __( 'Theme update failed.', 'helo-essential' );
	wp_send_json_error( $status );
}

add_action( 'wp_ajax_wcf_update_theme', 'wcf_wp_ajax_update_theme' );


function helo_essential_theme_option_header_code() {
	$html        = helo_option( 'opt-tabbed-code' );
	$tab_size    = '991.98';
	$mobile_size = '767.98';

	?>
    <style id="info-theme-global-css">
        <?php 
        
            if(is_array($html) && array_key_exists('custom_css',$html) && $html['custom_css'] !=''){
               echo $html['custom_css']; 
            }
            
            if(is_array($html) && array_key_exists('custom_css_tab',$html) && $html['custom_css_tab'] !=''){
               if (strpos($html['custom_css_tab'], '@media') !== false) {
                  echo $html['custom_css_tab']; 
               }else{
                  echo "@media (max-width: {$tab_size}px) {". $html['custom_css_tab'] .'}'; 
               }
            }
            
            if(is_array($html) && array_key_exists('custom_css_mobile',$html) && $html['custom_css_mobile'] !=''){
               if (strpos($html['custom_css_mobile'], '@media') !== false) {
                  echo $html['custom_css_mobile']; 
               }else{
                  echo "@media (max-width: {$mobile_size}px) {".$html['custom_css_mobile'].'}'; 
               }
            }
         
         ?>
    </style>
	<?php

}

add_action( 'wp_head', 'helo_essential_theme_option_header_code' );

function helo_essential_theme_option_page_header_code() {
	$html        = helo_meta_option( get_the_id(), 'opt-tabbed-code', false );
	$tab_size    = '991.98';
	$mobile_size = '767.98';
	if ( ! $html ) {
		return;
	}
	?>
    <style id="info-theme-global-page-css">
        <?php 
        
            if(is_array($html) && array_key_exists('custom_css',$html) && $html['custom_css'] !=''){
               echo $html['custom_css']; 
            }
            
            if(is_array($html) && array_key_exists('custom_css_tab',$html) && $html['custom_css_tab'] !=''){
               if (strpos($html['custom_css_tab'], '@media') !== false) {
                  echo $html['custom_css_tab']; 
               }else{
                  echo "@media (max-width: {$tab_size}px) {". $html['custom_css_tab'] .'}'; 
               }
            }
            
            if(is_array($html) && array_key_exists('custom_css_mobile',$html) && $html['custom_css_mobile'] !=''){
               if (strpos($html['custom_css_mobile'], '@media') !== false) {
                  echo $html['custom_css_mobile']; 
               }else{
                  echo "@media (max-width: {$mobile_size}px) {".$html['custom_css_mobile'].'}'; 
               }
            }
         
         ?>
    </style>
	<?php

}

add_action( 'wp_head', 'helo_essential_theme_option_page_header_code' );

function info_script_custom_data( $data ) {

	if ( helo_option( 'offcanvas_responsive_enable', 0 ) ) {
		$data['offcanvas_responsive_enable']     = true;
		$data['offcanvas_responsive_menu_width'] = helo_option( 'offcanvas_responsive_menu_width' );
	}

	if ( helo_option( 'offcanvas_menu_icon_plus' ) && helo_option( 'offcanvas_menu_icon_plus' ) != '' ) {
		$data['offcanvas_menu_icon_plus'] = sprintf( '<i class="%s"></i>', helo_option( 'offcanvas_menu_icon_plus' ) );
	}

	if ( helo_option( 'offcanvas_menu_icon_minus' ) && helo_option( 'offcanvas_menu_icon_minus' ) != '' ) {
		$data['offcanvas_menu_icon_minus'] = sprintf( '<i class="%s"></i>', helo_option( 'offcanvas_menu_icon_minus' ) );
	}

	if ( helo_option( 'sticky_header', 0 ) ) {
		$data['sticky_enable']     = true;
		$data['sticky_header_top'] = helo_option( 'sticky_header_start_from', 150 );
	}

	return $data;
}

add_filter( 'helo/script/custom/data', 'info_script_custom_data' );


add_action( 'elementor/page_templates/canvas/before_content', 'info_esssen_preloader_template_part' );

function info_esssen_preloader_template_part() {
	//get_template_part( 'template-parts/headers/content', 'preloader' );
}

add_filter( 'upload_mimes', function ( $mime_types ) {
	$mime_types['svg'] = 'image/svg+xml';

	return $mime_types;
} );

add_filter( 'wp_check_filetype_and_ext', function ( $data, $file, $filename, $mimes ) {
	$filetype = wp_check_filetype( $filename, $mimes );

	return [
		'ext'             => $filetype['ext'],
		'type'            => $filetype['type'],
		'proper_filename' => $data['proper_filename']
	];

}, 10, 4 );

function _helo_wcf_body_open_enable_page_background() {

	$bg_enable         = false;
	$global_settings   = helo_option( 'general_full_site_background' );
	$background_preset = helo_option( 'general_fullsite_background_preset' );
	$custom_background = helo_option( 'general_full_site_custom_background' );
	$url               = '';

	if ( $global_settings ) {
		if ( $background_preset == 'custom' && isset( $custom_background['url'] ) ) {
			$bg_enable = true;
			$url       = $custom_background['url'];
		} else {
			if ( $the_url = helo_essential_get_background_patterns( $background_preset ) ) {
				$url       = $the_url;
				$bg_enable = true;
			}
		}
	}

	if ( $bg_enable ) {
		echo sprintf( '<style>.wcf-body-bg {
         position: fixed;
         z-index: 99999;
         pointer-events: none;
         top: 0;
         opacity: 1;
         left: 0;
         width: 100vw;
         height: 100vh;
         background-repeat: repeat;
         background-position: top left;
         background-image: url(%s);
       }</style><div class="wcf-body-bg"></div>', $url );
	}


}

add_action( 'wp_body_open', '_helo_wcf_body_open_enable_page_background', 5 );

function wcf_remove_base_csscls( $classes ) {

	if ( ( is_single() && get_post_meta( get_the_id(), '_elementor_data', true ) ) || in_array( 'elementor-default', $classes ) ) {
		$new_classes = array_diff( $classes, [] );
	}

	return $new_classes;
}

add_action( 'elementor/widgets/register', function ( $widget_manager ) {
	if ( helo_theme_service_pass() ) {
		return;
	}

	$all_widgets = [
		'image-box',
		'image-box-slider',
		'typewriter',
		'animated-title',
		'animated-text',
		'social-icons',
		'image',
		'image-gallery',
		'text-hover-image',
		'brand-slider',
		'counter',
		'icon-box',
		'testimonial',
		'testimonial2',
		'testimonial3',
		'portfolio',
		'posts',
		'button',
		'pricing-table',
		'image-compare',
		'progressbar',
		'video-popup',
		'team',
		'one-page-nav',
		'timeline',
		'video-box',
		'contact-form-7',
		'mailchimp',
		'tabs',
		'services-tab',
		'floating-elements',
		'event-slider',
		'video-box-slider',
		'content-slider',
		'countdown',
		'video-mask',
		'animated-heading',
		'header-preset',
		'offcanvas-menu',
		'lottie-animation',
		'theme-post-content',
	];
	foreach ( $all_widgets as $key ) {
		$widget_manager->unregister_widget_type( 'wcf--' . $key );
	}

}, 100 );


// Step 1: Register a daily cron event.
if ( ! wp_next_scheduled( 'wcf_check_product_daily_event' ) ) {
	wp_schedule_event( time(), 'daily', 'wcf_check_product_daily_event' );
}

// Step 2: Hook the custom cron event to your function.
add_action( 'wcf_check_product_daily_event', 'helo_check_license_request_counter' );
//add_action('admin_init', 'helo_check_license_request_counter');
function helo_increment_product_counter() {
	$counter = get_option( 'wcf_theme_request_counter', 0 );
	$counter ++;
	if ( $counter > 10 ) {
		delete_option( 'helo_lic_Key' );
		delete_option( 'helo_lic_email' );
		delete_option( 'helo_license_key_status' );		
		deactivate_plugins( 'animation-addons-for-elementor/animation-addons-for-elementor.php' );
		deactivate_plugins( 'elementor/elementor.php' );
	}
	update_option( 'wcf_theme_request_counter', $counter );
}

function helo_check_license_request_counter() {

	if ( get_option( 'helo_lic_Key' ) ) {
		return;
	}

	$api_params = array(
		'edd_action'  => 'check_license',
		'license'     => get_option( 'helo_lic_Key' ),
		'item_id'     => HELO_PRODUCT_ID,
		'url'         => home_url(),
		'environment' => function_exists( 'wp_get_environment_type' ) ? wp_get_environment_type() : 'production',
	);

	// Call the custom API.
	$response = wp_remote_post( HELO_PRODUCT_DOMAIN, array(
		'body'      => $api_params,
		'timeout'   => 60,
		'sslverify' => false
	) );
	if ( is_wp_error( $response ) ) {
		return;
	}

	$license_data = json_decode( wp_remote_retrieve_body( $response ), true );

	if ( isset( $license_data['success'] ) && isset( $license_data['license'] ) && ( $license_data['license'] == 'disabled' || $license_data['license'] == 'failed' ) ) {
		helo_increment_product_counter();
	} elseif ( isset( $license_data['license'] ) && $license_data['license'] == 'valid' ) {
		update_option( 'wcf_theme_request_counter', 0 );
	} else {
		update_option( 'wcf_theme_request_counter', 1 );
	}

	// Reset the counter for the next day.
	update_option( 'wcf_theme_request_counter', 0 );
}

add_action( 'init', function () {

	if ( isset( $_GET['wcf_product_deactivation_action'] ) && $_REQUEST['wcf_product_deactivation_action'] === 'deactivate' ) {
		update_option( 'wcf_license_temp_deactivate', 1 );
		update_option( 'helo_license_key_status', 0 );
		deactivate_plugins( 'animation-addons-for-elementor/animation-addons-for-elementor.php' );
		deactivate_plugins( 'elementor/elementor.php' );
		wp_send_json( [ 'msg' => 'deactive' ] );
	}

	if ( isset( $_GET['wcf_product_deactivation_action'] ) && $_REQUEST['wcf_product_deactivation_action'] === 'activate' ) {
		delete_option( 'wcf_license_temp_deactivate' );
		update_option( 'helo_license_key_status', 'valid' );
		activate_plugins( 'animation-addons-for-elementor/animation-addons-for-elementor.php' );
		activate_plugins( 'elementor/elementor.php' );
		wp_send_json( [ 'msg' => 'active' ] );
	}

} );

add_filter( 'site_transient_update_themes', 'helo_remove_update_themes' );
function helo_remove_update_themes( $value ) {

	// Set your theme slug accordingly:
	$theme_slug = 'helo';

	if ( isset( $value ) && is_object( $value ) ) {
		unset( $value->response[ $theme_slug ] );
	}

	return $value;
}

add_filter( 'wcf_addons_dashboard_config', 'helo_addon_theme_dashboard_config', 154, 1 );
function helo_addon_theme_dashboard_config( $configs ) {

	if ( ! helo_theme_service_pass() ) {
		return $configs;
	}

	$configs['wcf_valid']    = true;
	$configs['sl_lic']       = get_option( 'wcf_addon_sl_license_key' );
	$configs['sl_lic_email'] = get_option( 'wcf_addon_sl_license_email' );

	return $configs;
}


/*  RTL Enable Options  */
add_filter( 'body_class', function ( $classes ) {
	if ( helo_option( 'wcf_rtl_activator', false ) || is_rtl() ) {
		$classes[] = 'dir-rtl';
	}

	return $classes;
} );

add_filter('wcf__addons__pro__status', function($value){
	if ( ! helo_theme_service_pass() ) {
		return $value;
	}	
	return 'valid';
});
