<?php

namespace HeloEssentialApp\Widgets;

use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Icons_Manager;
use Elementor\Plugin;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use WP_Query;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Donation
 *
 * Elementor widget for Donation.
 *
 * @since 1.0.0
 */
class CWT_Donation extends Widget_Base {

	/**
	 * Retrieve the widget name.
	 *
	 * @return string Widget name.
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function get_name() {
		return 'cwt--donation';
	}

	/**
	 * Retrieve the widget title.
	 *
	 * @return string Widget title.
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function get_title() {
		return esc_html__( 'Helo Donation', 'helo-essential' );
	}

	/**
	 * Retrieve the widget icon.
	 *
	 * @return string Widget icon.
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function get_icon() {
		return 'wcf eicon-flip';
	}

	/**
	 * Retrieve the list of categories the widget belongs to.
	 *
	 * Used to determine where to display the widget in the editor.
	 *
	 * Note that currently Elementor supports only one category.
	 * When multiple categories passed, Elementor uses the first one.
	 *
	 * @return array Widget categories.
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function get_categories() {
		return [ 'weal-coder-addon' ];
	}

	public function get_style_depends() {
		wp_register_style( 'cwt-donation', HELO_ESSENTIAL_ASSETS_URL . 'css/cwt-donation.css' );

		return [ 'swiper', 'cwt-donation' ];
	}

	public function get_script_depends() {
		wp_register_script( 'cwt-donation', HELO_ESSENTIAL_ASSETS_URL . '/js/widgets/cwt-donation.js', [ 'jquery' ], false, true );

		return [ 'swiper', 'flip', 'cwt-donation' ];
	}

	/**
	 * Register the widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function register_controls() {
		$this->start_controls_section(
			'section_content',
			[
				'label' => esc_html__( 'Helo Donation', 'helo-essential' ),
			]
		);

		$this->add_control(
			'donation_layout',
			[
				'label'   => esc_html__( 'Layout', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'default' => 'grid',
				'options' => [
					'grid'       => esc_html__( 'Grid', 'helo-essential' ),
					'slider'     => esc_html__( 'Slider', 'helo-essential' ),
					'filterable' => esc_html__( 'Filterable', 'helo-essential' ),
				],
			]
		);

		$this->end_controls_section();

		// Query
		$this->start_controls_section(
			'query_settings',
			[
				'label' => esc_html__( 'Query', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_CONTENT,
			]
		);

		$this->add_control(
			'post_order_by',
			[
				'label'   => esc_html__( 'Order By', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'default' => 'date',
				'options' => [
					'date'          => esc_html__( 'Date', 'helo-essential' ),
					'title'         => esc_html__( 'Title', 'helo-essential' ),
					'menu_order'    => esc_html__( 'Menu Order', 'helo-essential' ),
					'modified'      => esc_html__( 'Last Modified', 'helo-essential' ),
					'comment_count' => esc_html__( 'Comment Count', 'helo-essential' ),
					'rand'          => esc_html__( 'Random', 'helo-essential' ),
				],
			]
		);

		$this->add_control(
			'post_order',
			[
				'label'   => esc_html__( 'Order', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'default' => 'desc',
				'options' => [
					'asc'  => esc_html__( 'ASC', 'helo-essential' ),
					'desc' => esc_html__( 'DESC', 'helo-essential' ),
				],
			]
		);

		$this->add_control(
			'posts_per_page',
			[
				'label'   => esc_html__( 'Posts Per Page', 'helo-essential' ),
				'type'    => Controls_Manager::NUMBER,
				'default' => 6,
			]
		);

		$this->end_controls_section();

		// Slider Control
		$this->slider_controls();


		// Layout Style
		$this->start_controls_section(
			'style_layout',
			[
				'label'     => esc_html__( 'Layout', 'helo-essential' ),
				'tab'       => Controls_Manager::TAB_STYLE,
				'condition' => [ 'donation_layout!' => 'slider' ],
			]
		);

		$this->add_responsive_control(
			'donation_grid',
			[
				'label'     => esc_html__( 'Columns', 'helo-essential' ),
				'type'      => Controls_Manager::SELECT,
				'default'   => '3',
				'options'   => [
					'1' => esc_html__( '1', 'helo-essential' ),
					'2' => esc_html__( '2', 'helo-essential' ),
					'3' => esc_html__( '3', 'helo-essential' ),
					'4' => esc_html__( '4', 'helo-essential' ),
					'5' => esc_html__( '5', 'helo-essential' ),
					'6' => esc_html__( '6', 'helo-essential' ),
				],
				'selectors' => [
					'{{WRAPPER}} .cwt--donation .d-items' => 'grid-template-columns: repeat({{VALUE}}, 1fr);',
				],
			]
		);

		$this->add_responsive_control(
			'col_gap',
			[
				'label'      => esc_html__( 'Column Gap', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 200,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => 30,
				],
				'selectors'  => [
					'{{WRAPPER}} .cwt--donation .d-items' => 'column-gap: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'row_gap',
			[
				'label'      => esc_html__( 'Row Gap', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 200,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => 50,
				],
				'selectors'  => [
					'{{WRAPPER}} .cwt--donation .d-items' => 'row-gap: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();


		// Filter Style
		$this->start_controls_section(
			'style_filter',
			[
				'label' => esc_html__( 'Filter', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
                'condition' => [ 'donation_layout' => 'filterable' ],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'filter_typo',
				'selector' => '{{WRAPPER}} .filter button',
			]
		);

		$this->add_responsive_control(
			'filter_b_radius',
			[
				'label'      => esc_html__( 'Border Radius', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .filter button' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'filter_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .filter button' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		// Normal/Hover
		$this->start_controls_tabs(
			'style_filter_tabs'
		);

		$this->start_controls_tab(
			'filter_normal_tab',
			[
				'label' => esc_html__( 'Normal', 'helo-essential' ),
			]
		);

		$this->add_control(
			'filter_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'default'   => '',
				'selectors' => [
					'{{WRAPPER}} .filter button' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name'     => 'filter_bg',
				'types'    => [ 'classic', 'gradient' ],
				'exclude'  => [ 'image' ],
				'selector' => '{{WRAPPER}} .filter button',
			]
		);

		$this->end_controls_tab();

		// On Hover
		$this->start_controls_tab(
			'filter_hover_tab',
			[
				'label' => esc_html__( 'Hover/Active', 'helo-essential' ),
			]
		);

		$this->add_control(
			'filter_h_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'default'   => '',
				'selectors' => [
					'{{WRAPPER}} .filter button:hover, {{WRAPPER}} .filter .active' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name'     => 'filter_h_bg',
				'types'    => [ 'classic', 'gradient' ],
				'exclude'  => [ 'image' ],
				'selector' => '{{WRAPPER}} .filter button:hover, {{WRAPPER}} .filter .active',
			]
		);

		$this->end_controls_tab();

		$this->end_controls_tabs();

		$this->add_responsive_control(
			'filter_gap',
			[
				'label'      => esc_html__( 'Gap', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 300,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .filter' => 'gap: {{SIZE}}{{UNIT}};',
				],
				'separator' => 'before',
			]
		);

		$this->add_responsive_control(
			'filter_align',
			[
				'label'     => esc_html__( 'Alignment', 'helo-essential' ),
				'type'      => Controls_Manager::CHOOSE,
				'options'   => [
					'start'         => [
						'title' => esc_html__( 'Left', 'helo-essential' ),
						'icon'  => 'eicon-h-align-left',
					],
					'center'        => [
						'title' => esc_html__( 'Center', 'helo-essential' ),
						'icon'  => 'eicon-h-align-center',
					],
					'end'           => [
						'title' => esc_html__( 'Right', 'helo-essential' ),
						'icon'  => 'eicon-h-align-right',
					],
					'space-between' => [
						'title' => esc_html__( 'Stretch', 'helo-essential' ),
						'icon'  => 'eicon-align-stretch-h',
					],
				],
				'toggle'    => true,
				'selectors' => [
					'{{WRAPPER}} .filter' => 'justify-content: {{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'filter_margin',
			[
				'label'      => esc_html__( 'Margin', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .filter' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();


		// Content Style
		$this->start_controls_section(
			'style_content',
			[
				'label' => esc_html__( 'Content', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);
		
		$this->add_group_control(
			\Elementor\Group_Control_Background::get_type(),
			[
				'name' => 'background',
				'types' => [ 'classic', 'gradient' ],
				'selector' => '{{WRAPPER}} .content',
			]
		);

		$this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name'     => 'content_box_shadow',
				'selector' => '{{WRAPPER}} .content',
			]
		);

		$this->add_responsive_control(
			'content_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .content' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		// Title
		$this->add_control(
			'title_heading',
			[
				'label'     => esc_html__( 'Title', 'helo-essential' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'title_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'default'   => '',
				'selectors' => [
					'{{WRAPPER}} .title' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'title_typo',
				'selector' => '{{WRAPPER}} .title',
			]
		);

		$this->add_responsive_control(
			'title_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .title' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		// Description
		$this->add_control(
			'desc_heading',
			[
				'label'     => esc_html__( 'Description', 'helo-essential' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'desc_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'default'   => '',
				'selectors' => [
					'{{WRAPPER}} .desc' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'desc_typo',
				'selector' => '{{WRAPPER}} .desc',
			]
		);

		$this->add_responsive_control(
			'desc_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .desc' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();


		// Image style.
		$this->start_controls_section(
			'style_image',
			[
				'label' => esc_html__( 'Image', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_responsive_control(
			'img_width',
			[
				'label'      => esc_html__( 'Width', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%', 'em', 'rem' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 1000,
					],
					'%'  => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .thumb img' => 'width: {{SIZE}}{{UNIT}}; flex-basis: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'img_height',
			[
				'label'      => esc_html__( 'Height', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%', 'em', 'rem' ],
				'range'      => [
					'px' => [
						'min' => 1,
						'max' => 700,
					],
					'%'  => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .thumb img' => 'height: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();

		// Progress
		$this->start_controls_section(
			'style_progress',
			[
				'label' => esc_html__( 'Goal Progress', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'goal_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'default'   => '',
				'selectors' => [
					'{{WRAPPER}} .goal-stats li' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'achieve_color',
			[
				'label'     => esc_html__( 'Achieved Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'default'   => '',
				'selectors' => [
					'{{WRAPPER}} .goal-stats li:last-child' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'goal_typo',
				'selector' => '{{WRAPPER}} .goal-stats li',
			]
		);

		// Progress Bar
		$this->add_control(
			'progress_heading',
			[
				'label'     => esc_html__( 'Progress Bar', 'helo-essential' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'progress_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'default'   => '',
				'selectors' => [
					'{{WRAPPER}} .percentage'  => 'color: {{VALUE}};',
					'{{WRAPPER}} .bar::before' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} .bar::after'  => 'background-color: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'bar_color',
			[
				'label'     => esc_html__( 'Bar Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'default'   => '',
				'selectors' => [
					'{{WRAPPER}} .bar' => 'background-color: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'progress_typo',
				'selector' => '{{WRAPPER}} .percentage',
			]
		);

		$this->add_responsive_control(
			'progress_margin',
			[
				'label'      => esc_html__( 'Margin', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .progress' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();


		// Category
		$this->start_controls_section(
			'style_category',
			[
				'label' => esc_html__( 'Category', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'cat_typo',
				'selector' => '{{WRAPPER}} .category a',
			]
		);

		$this->add_responsive_control(
			'cat_b_radius',
			[
				'label'      => esc_html__( 'Border Radius', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .category a' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'cat_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .category a' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		// Normal/Hover
		$this->start_controls_tabs(
			'style_cat_tabs'
		);

		$this->start_controls_tab(
			'cat_normal_tab',
			[
				'label' => esc_html__( 'Normal', 'helo-essential' ),
			]
		);

		$this->add_control(
			'cat_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'default'   => '',
				'selectors' => [
					'{{WRAPPER}} .category a' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name'     => 'cat_bg',
				'types'    => [ 'classic', 'gradient' ],
				'exclude'  => [ 'image' ],
				'selector' => '{{WRAPPER}} .category a',
			]
		);

		$this->end_controls_tab();

		// On Hover
		$this->start_controls_tab(
			'cat_hover_tab',
			[
				'label' => esc_html__( 'Hover', 'helo-essential' ),
			]
		);

		$this->add_control(
			'cat_h_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'default'   => '',
				'selectors' => [
					'{{WRAPPER}} .category a:hover' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name'     => 'cat_h_bg',
				'types'    => [ 'classic', 'gradient' ],
				'exclude'  => [ 'image' ],
				'selector' => '{{WRAPPER}} .category a:hover',
			]
		);

		$this->end_controls_tab();

		$this->end_controls_tabs();

		$this->end_controls_section();

		// Donate Button
		$this->start_controls_section(
			'style_donate',
			[
				'label' => esc_html__( 'Donate Button', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'btn_typo',
				'selector' => '{{WRAPPER}} .donate-btn',
			]
		);

		$this->add_responsive_control(
			'btn_b_radius',
			[
				'label'      => esc_html__( 'Border Radius', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .donate-btn' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'btn_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .donate-btn' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		// Normal/Hover
		$this->start_controls_tabs(
			'style_btn_tabs'
		);

		$this->start_controls_tab(
			'btn_normal_tab',
			[
				'label' => esc_html__( 'Normal', 'helo-essential' ),
			]
		);

		$this->add_control(
			'btn_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'default'   => '',
				'selectors' => [
					'{{WRAPPER}} .donate-btn' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name'     => 'btn_bg',
				'types'    => [ 'classic', 'gradient' ],
				'exclude'  => [ 'image' ],
				'selector' => '{{WRAPPER}} .donate-btn',
			]
		);

		$this->end_controls_tab();

		// On Hover
		$this->start_controls_tab(
			'btn_hover_tab',
			[
				'label' => esc_html__( 'Hover', 'helo-essential' ),
			]
		);

		$this->add_control(
			'btn_h_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'default'   => '',
				'selectors' => [
					'{{WRAPPER}} .donate-btn:hover' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name'     => 'btn_h_bg',
				'types'    => [ 'classic', 'gradient' ],
				'exclude'  => [ 'image' ],
				'selector' => '{{WRAPPER}} .donate-btn:hover',
			]
		);

		$this->end_controls_tab();

		$this->end_controls_tabs();

		$this->end_controls_section();

		// Navigation Style
		$this->slider_navigation_style();

		// Pagination Style
		$this->slider_pagination_style();
	}


	/**
	 * Register the slider controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 */
	private function slider_controls() {
		$this->start_controls_section(
			'slider_options',
			[
				'label'     => esc_html__( 'Slider Options', 'helo-essential' ),
				'condition' => [ 'donation_layout' => 'slider' ],
			]
		);

		$slides_to_show = range( 1, 10 );
		$slides_to_show = array_combine( $slides_to_show, $slides_to_show );

		$this->add_responsive_control(
			'slides_to_show',
			[
				'label'       => esc_html__( 'Slides to Show', 'helo-essential' ),
				'type'        => Controls_Manager::SELECT,
				'default'     => 'auto',
				'required'    => true,
				'options'     => [
					                 'auto' => esc_html__( 'Auto', 'helo-essential' ),
				                 ] + $slides_to_show,
				'render_type' => 'template',
				'selectors'   => [
					'{{WRAPPER}}' => '--slides-to-show: {{VALUE}}',
				],
			]
		);

		$this->add_control(
			'autoplay',
			[
				'label'   => esc_html__( 'Autoplay', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'default' => 'yes',
				'options' => [
					'yes' => esc_html__( 'Yes', 'helo-essential' ),
					'no'  => esc_html__( 'No', 'helo-essential' ),
				],
			]
		);

		$this->add_control(
			'autoplay_delay',
			[
				'label'     => esc_html__( 'Autoplay delay', 'helo-essential' ),
				'type'      => Controls_Manager::NUMBER,
				'default'   => 3000,
				'condition' => [
					'autoplay' => 'yes',
				],
			]
		);

		$this->add_control(
			'autoplay_interaction',
			[
				'label'     => esc_html__( 'Autoplay Interaction', 'helo-essential' ),
				'type'      => Controls_Manager::SELECT,
				'default'   => 'true',
				'options'   => [
					'true'  => esc_html__( 'Yes', 'helo-essential' ),
					'false' => esc_html__( 'No', 'helo-essential' ),
				],
				'condition' => [
					'autoplay' => 'yes',
				],
			]
		);

		$this->add_control(
			'allow_touch_move',
			[
				'label'     => esc_html__( 'Allow Touch Move', 'helo-essential' ),
				'type'      => Controls_Manager::SELECT,
				'separator' => 'before',
				'default'   => 'false',
				'options'   => [
					'true'  => esc_html__( 'Yes', 'helo-essential' ),
					'false' => esc_html__( 'No', 'helo-essential' ),
				],
			]
		);

		$this->add_control(
			'loop',
			[
				'label'   => esc_html__( 'Loop', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'default' => 'true',
				'options' => [
					'true'  => esc_html__( 'Yes', 'helo-essential' ),
					'false' => esc_html__( 'No', 'helo-essential' ),
				],
			]
		);

		$this->add_control(
			'slide_effect',
			[
				'label'   => esc_html__( 'Effect', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'default' => 'slide',
				'options' => [
					'slide'     => esc_html__( 'Slide', 'helo-essential' ),
					'flip'      => esc_html__( 'Flip', 'helo-essential' ),
					'cube'      => esc_html__( 'Cube', 'helo-essential' ),
					'coverflow' => esc_html__( 'CoderFlow', 'helo-essential' ),
				],
			]
		);

		$this->add_control(
			'speed',
			[
				'label'   => esc_html__( 'Animation Speed', 'helo-essential' ),
				'type'    => Controls_Manager::NUMBER,
				'default' => 500,
			]
		);

		$this->add_responsive_control(
			'space_between',
			[
				'label'       => esc_html__( 'Space Between', 'helo-essential' ),
				'type'        => Controls_Manager::NUMBER,
				'default'     => 20,
				'render_type' => 'template',
				'selectors'   => [
					'{{WRAPPER}}' => '--space-between: {{VALUE}}px;',
				],
			]
		);

		//slider navigation
		$this->add_control(
			'navigation',
			[
				'label'     => esc_html__( 'Slider Controls', 'helo-essential' ),
				'type'      => Controls_Manager::SELECT,
				'separator' => 'before',
				'default'   => 'both',
				'options'   => [
					'both'   => esc_html__( 'Both', 'helo-essential' ),
					'arrows' => esc_html__( 'Navigation', 'helo-essential' ),
					'dots'   => esc_html__( 'Pagination', 'helo-essential' ),
					'none'   => esc_html__( 'None', 'helo-essential' ),
				],
			]
		);

		$this->add_control(
			'pagination_type',
			[
				'label'     => esc_html__( 'Pagination Type', 'helo-essential' ),
				'type'      => Controls_Manager::SELECT,
				'separator' => 'before',
				'default'   => 'bullets',
				'options'   => [
					'bullets'     => esc_html__( 'Dots', 'helo-essential' ),
					'fraction'    => esc_html__( 'Fraction', 'helo-essential' ),
					'progressbar' => esc_html__( 'Progressbar', 'helo-essential' ),
				],
				'condition' => [
					'navigation' => [ 'both', 'dots' ],
				],
			]
		);

		$this->add_control(
			'navigation_previous_icon',
			[
				'label'            => esc_html__( 'Previous Arrow Icon', 'helo-essential' ),
				'type'             => Controls_Manager::ICONS,
				'fa4compatibility' => 'icon',
				'skin'             => 'inline',
				'label_block'      => false,
				'skin_settings'    => [
					'inline' => [
						'none' => [
							'label' => 'Default',
							'icon'  => 'eicon-chevron-left',
						],
						'icon' => [
							'icon' => 'eicon-star',
						],
					],
				],
				'recommended'      => [
					'fa-regular' => [
						'arrow-alt-circle-left',
						'caret-square-left',
					],
					'fa-solid'   => [
						'angle-double-left',
						'angle-left',
						'arrow-alt-circle-left',
						'arrow-circle-left',
						'arrow-left',
						'caret-left',
						'caret-square-left',
						'chevron-circle-left',
						'chevron-left',
						'long-arrow-alt-left',
					],
				],
				'conditions'       => [
					'relation' => 'or',
					'terms'    => [
						[
							'name'     => 'navigation',
							'operator' => '=',
							'value'    => 'both',
						],
						[
							'name'     => 'navigation',
							'operator' => '=',
							'value'    => 'arrows',
						],
					],
				],
			]
		);

		$this->add_control(
			'navigation_next_icon',
			[
				'label'            => esc_html__( 'Next Arrow Icon', 'helo-essential' ),
				'type'             => Controls_Manager::ICONS,
				'fa4compatibility' => 'icon',
				'skin'             => 'inline',
				'label_block'      => false,
				'skin_settings'    => [
					'inline' => [
						'none' => [
							'label' => 'Default',
							'icon'  => 'eicon-chevron-right',
						],
						'icon' => [
							'icon' => 'eicon-star',
						],
					],
				],
				'recommended'      => [
					'fa-regular' => [
						'arrow-alt-circle-right',
						'caret-square-right',
					],
					'fa-solid'   => [
						'angle-double-right',
						'angle-right',
						'arrow-alt-circle-right',
						'arrow-circle-right',
						'arrow-right',
						'caret-right',
						'caret-square-right',
						'chevron-circle-right',
						'chevron-right',
						'long-arrow-alt-right',
					],
				],
				'conditions'       => [
					'relation' => 'or',
					'terms'    => [
						[
							'name'     => 'navigation',
							'operator' => '=',
							'value'    => 'both',
						],
						[
							'name'     => 'navigation',
							'operator' => '=',
							'value'    => 'arrows',
						],
					],
				],
			]
		);

		$this->end_controls_section();
	}

	/**
	 * Register the slider navigation style controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 */
	private function slider_navigation_style() {
		$this->start_controls_section(
			'style_navigation',
			[
				'label'     => esc_html__( 'Navigation', 'helo-essential' ),
				'tab'       => Controls_Manager::TAB_STYLE,
				'condition' => [
					'navigation'      => [ 'arrows', 'both' ],
					'donation_layout' => [ 'slider' ],
				],
			]
		);

		$this->add_control(
			'arrows_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .cwt-arrow' => 'color: {{VALUE}}; fill: {{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'arrows_size',
			[
				'label'     => esc_html__( 'Size', 'helo-essential' ),
				'type'      => Controls_Manager::SLIDER,
				'range'     => [
					'px' => [
						'min' => 0,
						'max' => 60,
					],
				],
				'selectors' => [
					'{{WRAPPER}} .cwt-arrow' => 'font-size: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name'     => 'arrows_border',
				'selector' => '{{WRAPPER}} .cwt-arrow',
			]
		);

		$this->add_responsive_control(
			'arrows_b_radius',
			[
				'label'      => esc_html__( 'Border Radius', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .cwt-arrow' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'arrows_circle',
			[
				'label'      => esc_html__( 'Circle Size', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .cwt-arrow' => 'width: {{SIZE}}{{UNIT}}; height: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'arrows_gap',
			[
				'label'      => esc_html__( 'Gap', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'default'    => [
					'unit' => 'px',
					'size' => 10,
				],
				'selectors'  => [
					'{{WRAPPER}} .cwt--donation-navigation' => 'gap: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'arrows_position',
			[
				'label'     => esc_html__( 'Position', 'textdomain' ),
				'type'      => Controls_Manager::SELECT,
				'default'   => 'relative',
				'options'   => [
					'relative' => esc_html__( 'Default', 'textdomain' ),
					'absolute' => esc_html__( 'Absolute', 'textdomain' ),
				],
				'selectors' => [
					'{{WRAPPER}} .cwt--donation-navigation' => 'position: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'navigation_align',
			[
				'label'     => esc_html__( 'Alignment', 'helo-essential' ),
				'type'      => Controls_Manager::CHOOSE,
				'options'   => [
					'start'         => [
						'title' => esc_html__( 'Left', 'helo-essential' ),
						'icon'  => 'eicon-h-align-left',
					],
					'center'        => [
						'title' => esc_html__( 'Center', 'helo-essential' ),
						'icon'  => 'eicon-h-align-center',
					],
					'end'           => [
						'title' => esc_html__( 'Right', 'helo-essential' ),
						'icon'  => 'eicon-h-align-right',
					],
					'space-between' => [
						'title' => esc_html__( 'Stretch', 'helo-essential' ),
						'icon'  => 'eicon-align-stretch-h',
					],
				],
				'default'   => 'center',
				'toggle'    => true,
				'selectors' => [
					'{{WRAPPER}} .cwt--donation-navigation' => 'justify-content: {{VALUE}};',
				],
				'condition' => [ 'arrows_position' => 'relative' ]
			]
		);

		$this->add_control(
			'arrows_hr_position',
			[
				'label'        => esc_html__( 'Horizontal Orientation', 'helo-essential' ),
				'type'         => Controls_Manager::CHOOSE,
				'prefix_class' => 'cwt-donation-hrp-',
				'options'      => [
					'left'  => [
						'title' => esc_html__( 'Left', 'helo-essential' ),
						'icon'  => 'eicon-h-align-left',
					],
					'right' => [
						'title' => esc_html__( 'Right', 'helo-essential' ),
						'icon'  => 'eicon-h-align-right',
					],
				],
				'condition'    => [ 'arrows_position' => 'absolute' ],
			]
		);

		$this->add_responsive_control(
			'arrows_hr_offset',
			[
				'label'      => esc_html__( 'Offset', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'range'      => [
					'px' => [
						'min' => - 1000,
						'max' => 1000,
					],
					'%'  => [
						'min' => - 100,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .cwt--donation-navigation' => '--arrows-hr-offset: {{SIZE}}{{UNIT}};',
				],
				'condition'  => [ 'arrows_position' => 'absolute' ],
			]
		);

		$this->add_control(
			'arrows_vr_position',
			[
				'label'        => esc_html__( 'Vertical Orientation', 'helo-essential' ),
				'type'         => Controls_Manager::CHOOSE,
				'prefix_class' => 'cwt-donation-vrp-',
				'options'      => [
					'top'    => [
						'title' => esc_html__( 'Top', 'helo-essential' ),
						'icon'  => 'eicon-v-align-top',
					],
					'bottom' => [
						'title' => esc_html__( 'Bottom', 'helo-essential' ),
						'icon'  => 'eicon-v-align-bottom',
					],
				],
				'condition'    => [ 'arrows_position' => 'absolute' ],
			]
		);

		$this->add_responsive_control(
			'arrows_vr_offset',
			[
				'label'      => esc_html__( 'Offset', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'range'      => [
					'px' => [
						'min' => - 1000,
						'max' => 1000,
					],
					'%'  => [
						'min' => - 100,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .cwt--donation-navigation' => '--arrows-vr-offset: {{SIZE}}{{UNIT}};',
				],
				'condition'  => [ 'arrows_position' => 'absolute' ],
			]
		);

		$this->start_controls_tabs(
			'style_arrows_tabs'
		);

		$this->start_controls_tab(
			'arrows_normal_tab',
			[
				'label' => esc_html__( 'Normal', 'textdomain' ),
			]
		);

		$this->end_controls_tab();


		// On Hover
		$this->start_controls_tab(
			'arrows_hover_tab',
			[
				'label' => esc_html__( 'Hover', 'textdomain' ),
			]
		);

		$this->end_controls_tab();

		$this->end_controls_tabs();

		$this->end_controls_section();
	}

	// Pagination
	private function slider_pagination_style() {
		$this->start_controls_section(
			'style_pagination',
			[
				'label'     => esc_html__( 'Pagination', 'helo-essential' ),
				'tab'       => Controls_Manager::TAB_STYLE,
				'condition' => [
					'navigation' => [ 'dots', 'both' ],
				],
			]
		);

		$this->add_control(
			'pagination_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .swiper-pagination-bullet'      => 'background-color: {{VALUE}};',
					'{{WRAPPER}} .swiper-pagination-fraction'    => 'color: {{VALUE}}; fill: {{VALUE}};',
					'{{WRAPPER}} .swiper-pagination-progressbar' => 'background-color: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'      => 'fraction_typo',
				'selector'  => '{{WRAPPER}} .swiper-pagination-fraction',
				'condition' => [ 'pagination_type' => 'fraction' ],
			]
		);

		$this->add_responsive_control(
			'bullet_width',
			[
				'label'      => esc_html__( 'Width', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .swiper-pagination-bullet, {{WRAPPER}} .swiper-pagination-progressbar' => 'width: {{SIZE}}{{UNIT}};',
				],
				'condition'  => [ 'pagination_type' => [ 'bullets', 'progressbar' ] ],
			]
		);

		$this->add_responsive_control(
			'bullet_height',
			[
				'label'      => esc_html__( 'Height', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 20,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .swiper-pagination-bullet, {{WRAPPER}} .swiper-pagination-progressbar' => 'height: {{SIZE}}{{UNIT}};',
				],
				'condition'  => [ 'pagination_type' => [ 'bullets', 'progressbar' ] ],
			]
		);

		$this->add_responsive_control(
			'paginate_b_radius',
			[
				'label'      => esc_html__( 'Border Radius', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .swiper-pagination-bullet, {{WRAPPER}} .swiper-pagination-progressbar, {{WRAPPER}} .swiper-pagination-progressbar-fill' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition'  => [ 'pagination_type' => [ 'bullets', 'progressbar' ] ],
			]
		);

		$this->add_responsive_control(
			'paginate_gap',
			[
				'label'      => esc_html__( 'Gap', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .swiper-pagination' => 'gap: {{SIZE}}{{UNIT}};',
				],
				'condition'  => [
					'pagination_type' => [ 'bullets', 'fraction' ],
				],
			]
		);

		$this->add_control(
			'paginate_direction',
			[
				'label'     => esc_html__( 'Direction', 'helo-essential' ),
				'type'      => Controls_Manager::SELECT,
				'default'   => 'row',
				'options'   => [
					'row'    => esc_html__( 'Row', 'helo-essential' ),
					'column' => esc_html__( 'Column', 'helo-essential' ),
				],
				'selectors' => [
					'{{WRAPPER}} .swiper-pagination' => 'flex-direction: {{VALUE}};',
				],
				'condition' => [ 'pagination_type' => 'bullets' ],
			]
		);

		$this->add_control(
			'paginate_pos',
			[
				'label'     => esc_html__( 'Position', 'helo-essential' ),
				'type'      => Controls_Manager::SELECT,
				'default'   => 'relative',
				'options'   => [
					'relative' => esc_html__( 'Default', 'helo-essential' ),
					'absolute' => esc_html__( 'Absolute', 'helo-essential' ),
				],
				'selectors' => [
					'{{WRAPPER}} .swiper-pagination' => 'position: {{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'paginate_align',
			[
				'label'     => esc_html__( 'Alignment', 'helo-essential' ),
				'type'      => Controls_Manager::CHOOSE,
				'options'   => [
					'left'   => [
						'title' => esc_html__( 'Left', 'helo-essential' ),
						'icon'  => 'eicon-text-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'helo-essential' ),
						'icon'  => 'eicon-text-align-center',
					],
					'right'  => [
						'title' => esc_html__( 'Right', 'helo-essential' ),
						'icon'  => 'eicon-text-align-right',
					],
				],
				'toggle'    => true,
				'selectors' => [
					'{{WRAPPER}} .cwt--donation-pagination' => 'text-align: {{VALUE}};',
				],
				'condition' => [ 'paginate_pos' => 'relative' ],
			]
		);

		$this->add_control(
			'paginate_hr_pos',
			[
				'label'        => esc_html__( 'Horizontal Orientation', 'helo-essential' ),
				'type'         => Controls_Manager::CHOOSE,
				'prefix_class' => 'cwt-donation-pg-hr-',
				'options'      => [
					'left'  => [
						'title' => esc_html__( 'Left', 'helo-essential' ),
						'icon'  => 'eicon-h-align-left',
					],
					'right' => [
						'title' => esc_html__( 'Right', 'helo-essential' ),
						'icon'  => 'eicon-h-align-right',
					],
				],
				'condition'    => [ 'paginate_pos' => 'absolute' ],
			]
		);

		$this->add_responsive_control(
			'paginate_hr_offset',
			[
				'label'      => esc_html__( 'Offset', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'range'      => [
					'px' => [
						'min' => - 1000,
						'max' => 1000,
					],
					'%'  => [
						'min' => - 100,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .swiper-pagination' => '--pg-hr-offset: {{SIZE}}{{UNIT}};',
				],
				'condition'  => [ 'paginate_pos' => 'absolute' ],
			]
		);

		$this->add_control(
			'paginate_vr_pos',
			[
				'label'        => esc_html__( 'Vertical Orientation', 'helo-essential' ),
				'type'         => Controls_Manager::CHOOSE,
				'prefix_class' => 'cwt-donation-pg-vr-',
				'options'      => [
					'top'    => [
						'title' => esc_html__( 'Top', 'helo-essential' ),
						'icon'  => 'eicon-v-align-top',
					],
					'bottom' => [
						'title' => esc_html__( 'Bottom', 'helo-essential' ),
						'icon'  => 'eicon-v-align-bottom',
					],
				],
				'condition'    => [ 'paginate_pos' => 'absolute' ],
			]
		);

		$this->add_responsive_control(
			'paginate_vr_offset',
			[
				'label'      => esc_html__( 'Offset', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'range'      => [
					'px' => [
						'min' => - 1000,
						'max' => 1000,
					],
					'%'  => [
						'min' => - 100,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .swiper-pagination' => '--pg-vr-offset: {{SIZE}}{{UNIT}};',
				],
				'condition'  => [ 'paginate_pos' => 'absolute' ],
			]
		);

		$this->add_responsive_control(
			'paginate_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .swiper-pagination' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'separator'  => 'before',
			]
		);

		// Pagination Active
		$this->add_control(
			'paginate_a_heading',
			[
				'label'     => esc_html__( 'Active', 'helo-essential' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'paginate_a_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .swiper-pagination-bullet-active'              => 'background-color: {{VALUE}};',
					'{{WRAPPER}} .swiper-pagination-current, {{WRAPPER}} .dash' => 'color: {{VALUE}}; fill: {{VALUE}};',
					'{{WRAPPER}} .swiper-pagination-progressbar-fill'           => 'background: {{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'bullet_a_width',
			[
				'label'      => esc_html__( 'Width', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .swiper-pagination-bullet-active' => 'width: {{SIZE}}{{UNIT}};',
				],
				'condition'  => [ 'pagination_type' => 'bullets' ],
			]
		);

		$this->add_responsive_control(
			'bullet_a_height',
			[
				'label'      => esc_html__( 'Height', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 20,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .swiper-pagination-bullet-active' => 'height: {{SIZE}}{{UNIT}};',
				],
				'condition'  => [ 'pagination_type' => 'bullets' ],
			]
		);

		$this->end_controls_section();
	}

	/**
	 * Render the widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function render() {
		if ( ! is_plugin_active( 'give/give.php' ) ) { ?>
            <div class="notice notice-error"
                 style="padding: 10px; border: 1px solid #f44336; background: #ffe8e8; color: #d32f2f;">
				<?php echo esc_html__( 'Please install and activate the GiveWP plugin first.', 'helo-essential' ); ?>
                <a href="https://wordpress.org/plugins/give/"
                   target="_blank"><?php echo esc_html__( 'Link', 'helo-essential' ); ?></a>
				<?php return; ?>
            </div>
			<?php
		}

		$settings = $this->get_settings_for_display();

		$args = [
			'post_type'      => 'give_forms',
			'post_status'    => 'publish',
			'posts_per_page' => $settings['posts_per_page'],
			'order'          => $settings['post_order'],
			'orderby'        => $settings['post_order_by'],
		];

		$query = new WP_Query( $args );

		// Slider Settings
		$show_dots   = ( in_array( $settings['navigation'], [ 'dots', 'both' ] ) );
		$show_arrows = ( in_array( $settings['navigation'], [ 'arrows', 'both' ] ) );

		$slider_settings = $this->get_slider_settings( $settings );

		$this->add_render_attribute(
			'wrapper',
			[
				'class'         => [ 'cwt_donation_slider_wrapper' ],
				'data-settings' => json_encode( $slider_settings ), //phpcs:ignore
			]
		);


		$this->add_render_attribute(
			'carousel-wrapper',
			[
				'class' => 'cwt_donation_slider swiper',
				'style' => 'position: static',
				'dir'   => 'ltr',
			]
		);
		?>

        <div class="cwt--donation cwt--donation-<?php echo $settings['donation_layout']; ?>">
			<?php if ( 'filterable' === $settings['donation_layout'] ) {
				$this->render_donation_filterable( $query );
			} else if ( 'slider' === $settings['donation_layout'] ) {
				$this->render_donation_slider( $query, $show_arrows, $show_dots );
			} else {
				$this->render_donation_grid( $query );
			} ?>
        </div>
		<?php
	}

	private function get_slider_settings( $settings ) {

		$show_dots   = ( in_array( $settings['navigation'], [ 'dots', 'both' ] ) );
		$show_arrows = ( in_array( $settings['navigation'], [ 'arrows', 'both' ] ) );

		$slider_settings = [
			'loop'           => 'true' === $settings['loop'],
			'speed'          => $settings['speed'],
			'allowTouchMove' => $settings['allow_touch_move'],
			'slidesPerView'  => $settings['slides_to_show'],
			'spaceBetween'   => $settings['space_between'],
			'effect'         => $settings['slide_effect'],
		];

		if ( 'yes' === $settings['autoplay'] ) {
			$slider_settings['autoplay'] = [
				'delay'                => $settings['autoplay_delay'],
				'disableOnInteraction' => $settings['autoplay_interaction'],
			];
		}

		if ( $show_arrows ) {
			$slider_settings['navigation'] = [
				'nextEl' => '.elementor-element-' . $this->get_id() . ' .cwt-arrow-next',
				'prevEl' => '.elementor-element-' . $this->get_id() . ' .cwt-arrow-prev',
			];
		}

		if ( $show_dots ) {
			$slider_settings['pagination'] = [
				'el'   => '.elementor-element-' . $this->get_id() . ' .swiper-pagination',
				'type' => $settings['pagination_type'],
			];
		}

		//slider breakpoints
		$active_breakpoints = Plugin::$instance->breakpoints->get_active_breakpoints();

		foreach ( $active_breakpoints as $breakpoint_name => $breakpoint ) {
			$slides_to_show = ! empty( $settings[ 'slides_to_show_' . $breakpoint_name ] ) ? $settings[ 'slides_to_show_' . $breakpoint_name ] : $settings['slides_to_show'];
			$space_between  = ! empty( $settings[ 'space_between_' . $breakpoint_name ] ) ? $settings[ 'space_between_' . $breakpoint_name ] : $settings['space_between'];

			$slider_settings['breakpoints'][ $breakpoint->get_value() ]['slidesPerView'] = $slides_to_show;
			$slider_settings['breakpoints'][ $breakpoint->get_value() ]['spaceBetween']  = $space_between;
		}

		return $slider_settings;
	}

	/**
	 * Render swiper button.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 */
	private function render_swiper_button( $type ) {
		$direction     = 'next' === $type ? 'right' : 'left';
		$icon_settings = $this->get_settings_for_display( 'navigation_' . $type . '_icon' );

		if ( empty( $icon_settings['value'] ) ) {
			$icon_settings = [
				'library' => 'eicons',
				'value'   => 'eicon-chevron-' . $direction,
			];
		}

		Icons_Manager::render_icon( $icon_settings, [ 'aria-hidden' => 'true' ] );
	}

	protected function render_donation_grid( $query ) { ?>
        <div class="d-items">
			<?php
			if ( $query->have_posts() ) {
				while ( $query->have_posts() ) : $query->the_post();
					$id                = get_the_ID();
					$goal_stats        = give_goal_progress_stats( $id );
					$goal_progress_val = ! empty( $goal_stats['progress'] ) ? $goal_stats['progress'] : 0;
					$goal              = ! empty( $goal_stats['goal'] ) ? $goal_stats['goal'] : 0;
					$achieve           = ! empty( $goal_stats['actual'] ) ? $goal_stats['actual'] : 0;
					$donation_settings = get_post_meta( $id, 'formBuilderSettings', true );
					$d_settings_array  = json_decode( $donation_settings, true );

					if ( taxonomy_exists( 'give_forms_category' ) ) {
						$categories = get_the_term_list( $id, 'give_forms_category', '', '', '' );
					} else {
						$categories = '';
					}
					?>
                    <article class="cwt--donation-item">
                        <div class="thumb">
							<?php if ( isset( $d_settings_array['designSettingsImageUrl'] ) ) { ?>
                                <img src="<?php echo esc_url( $d_settings_array['designSettingsImageUrl'] ); ?>"
                                     alt="<?php echo esc_html__( 'Donation Image', 'helo-essential' ); ?>">
							<?php } ?>
                        </div>
                        <div class="content">
							<?php if ( $categories ) { ?>
                                <div class="category"><?php echo $categories; ?></div>
							<?php } ?>
                            <h2 class="title"><a href="<?php the_permalink(); ?>"><?php the_title(); ?></a></h2>
							<?php if ( $excerpt = $this->getExcerpt() ) { ?>
                                <div class="desc"><?php echo $excerpt; ?></div>
							<?php } ?>
                            <div class="goal-progress">
                                <div class="progress" data-progress="<?php echo $goal_progress_val; ?>%">
                                    <div class="percentage">
										<?php echo $goal_progress_val . esc_html__( '%', 'helo-essential' ); ?>
                                    </div>
                                    <div class="bar"></div>
                                </div>
                                <ul class="goal-stats">
                                    <li>
										<?php echo esc_html__( 'Goal:', 'helo-essential' ); ?><br>
										<?php echo $goal; ?>
                                    </li>
                                    <li>
										<?php echo esc_html__( 'Achieved:', 'helo-essential' ); ?> <br>
										<?php echo $achieve; ?>
                                    </li>
                                </ul>
                            </div>
                        </div>
                        <a href="<?php the_permalink(); ?>" class="donate-btn">
							<?php echo esc_html( $d_settings_array['donateButtonCaption'] ); ?>
                        </a>
                    </article>
				<?php
				endwhile;
				wp_reset_postdata();
			}
			?>
        </div>
		<?php
	}

	protected function render_donation_slider( $query, $show_arrows, $show_dots ) { ?>
    <div <?php $this->print_render_attribute_string( 'wrapper' ); ?>>
    <div <?php $this->print_render_attribute_string( 'carousel-wrapper' ); ?>>
        <div class="swiper-wrapper">
		<?php
		if ( $query->have_posts() ) {
			while ( $query->have_posts() ) : $query->the_post();
				$id                = get_the_ID();
				$goal_stats        = give_goal_progress_stats( $id );
				$goal_progress_val = ! empty( $goal_stats['progress'] ) ? $goal_stats['progress'] : 0;
				$goal              = ! empty( $goal_stats['goal'] ) ? $goal_stats['goal'] : 0;
				$achieve           = ! empty( $goal_stats['actual'] ) ? $goal_stats['actual'] : 0;
				$donation_settings = get_post_meta( $id, 'formBuilderSettings', true );
				$d_settings_array  = json_decode( $donation_settings, true );

				if ( taxonomy_exists( 'give_forms_category' ) ) {
					$categories = get_the_term_list( $id, 'give_forms_category', '', '', '' );
				} else {
					$categories = '';
				}
				?>
                <div class="swiper-slide">
                    <article class="cwt--donation-item">
                        <div class="thumb">
							<?php if ( isset( $d_settings_array['designSettingsImageUrl'] ) ) { ?>
                                <img src="<?php echo esc_url( $d_settings_array['designSettingsImageUrl'] ); ?>"
                                     alt="<?php echo esc_html__( 'Donation Image', 'helo-essential' ); ?>">
							<?php } ?>
                        </div>
                        <div class="content">
							<?php if ( $categories ) { ?>
                                <div class="category"><?php echo $categories; ?></div>
							<?php } ?>
                            <h2 class="title"><a href="<?php the_permalink(); ?>"><?php the_title(); ?></a></h2>
							<?php if ( $excerpt = $this->getExcerpt() ) { ?>
                                <div class="desc"><?php echo $excerpt; ?></div>
							<?php } ?>
                            <div class="goal-progress">
                                <div class="progress" data-progress="<?php echo $goal_progress_val; ?>%">
                                    <div class="percentage">
										<?php echo $goal_progress_val . esc_html__( '%', 'helo-essential' ); ?>
                                    </div>
                                    <div class="bar"></div>
                                </div>

                                <ul class="goal-stats">
                                    <li>
										<?php echo esc_html__( 'Goal:', 'helo-essential' ); ?>
										<?php echo $goal; ?>
                                    </li>
                                    <li>
										<?php echo esc_html__( 'Achieved:', 'helo-essential' ); ?>
										<?php echo $achieve; ?>
                                    </li>
                                </ul>
                            </div>
                        </div>
                        <a href="<?php the_permalink(); ?>" class="donate-btn">
							<?php echo esc_html( $d_settings_array['donateButtonCaption'] ); ?>
                        </a>
                    </article>
                </div>
			<?php
			endwhile;
			wp_reset_postdata();
			?>
            </div>
            </div>

            <!-- navigation and pagination -->
			<?php if ( true ) : ?>
                <div class="controls-wrap">
					<?php if ( $show_arrows ) : ?>
                        <div class="cwt--donation-navigation">
                            <div class="cwt-arrow cwt-arrow-prev" role="button" tabindex="0">
								<?php $this->render_swiper_button( 'previous' ); ?>
                            </div>
                            <div class="cwt-arrow cwt-arrow-next" role="button" tabindex="0">
								<?php $this->render_swiper_button( 'next' ); ?>
                            </div>
                        </div>
					<?php endif; ?>

					<?php if ( $show_dots ) : ?>
                        <div class="cwt--donation-pagination">
                            <div class="swiper-pagination"></div>
                        </div>
					<?php endif; ?>
                </div>
			<?php endif; ?>
            </div>
			<?php
		}
	}

	protected function render_donation_filterable( $query ) {
		if ( taxonomy_exists( 'give_forms_category' ) ) {
			$terms = get_terms( [
				'taxonomy'   => 'give_forms_category',
				'hide_empty' => false,
			] );
			?>
            <div class="filter">
                <button data-filter="all" class="active">
					<?php echo esc_html__( 'All', 'helo-essential' ); ?>
                </button>
				<?php foreach ( $terms as $term ) { ?>
                    <button data-filter="<?php echo esc_attr( $term->slug ); ?>">
						<?php echo esc_html( $term->name ); ?>
                    </button>
				<?php } ?>
            </div>
			<?php
		} else {
			echo esc_html__( 'No Category Founded!', 'helo-essential' );
		}
		?>
        <div class="d-items">
			<?php
			if ( $query->have_posts() ) {
				while ( $query->have_posts() ) : $query->the_post();
					$id                = get_the_ID();
					$goal_stats        = give_goal_progress_stats( $id );
					$goal_progress_val = ! empty( $goal_stats['progress'] ) ? $goal_stats['progress'] : 0;
					$goal              = ! empty( $goal_stats['goal'] ) ? $goal_stats['goal'] : 0;
					$achieve           = ! empty( $goal_stats['actual'] ) ? $goal_stats['actual'] : 0;
					$donation_settings = get_post_meta( $id, 'formBuilderSettings', true );
					$d_settings_array  = json_decode( $donation_settings, true );

					if ( taxonomy_exists( 'give_forms_category' ) ) {
						$categories = get_the_term_list( $id, 'give_forms_category', '', '', '' );
					} else {
						$categories = '';
					}

					$cats        = get_the_terms( $id, 'give_forms_category' );
					$cat_classes = [];

					if ( ! empty( $cats ) && ! is_wp_error( $cats ) ) {
						foreach ( $cats as $cat ) {
							$cat_classes[] = $cat->slug;
						}
					}
					$class_string = implode( ' ', $cat_classes );

					?>
                    <article class="cwt--donation-item <?php echo esc_attr( $class_string ); ?>">
                        <div class="thumb">
							<?php if ( isset( $d_settings_array['designSettingsImageUrl'] ) ) { ?>
                                <img src="<?php echo esc_url( $d_settings_array['designSettingsImageUrl'] ); ?>"
                                     alt="<?php echo esc_html__( 'Donation Image', 'helo-essential' ); ?>">
							<?php } ?>
                        </div>
                        <div class="content">
							<?php if ( $categories ) { ?>
                                <div class="category"><?php echo $categories; ?></div>
							<?php } ?>
                            <h2 class="title"><a href="<?php the_permalink(); ?>"><?php the_title(); ?></a></h2>
							<?php if ( $excerpt = $this->getExcerpt() ) { ?>
                                <div class="desc"><?php echo $excerpt; ?></div>
							<?php } ?>
                            <div class="goal-progress">
                                <div class="progress" data-progress="<?php echo $goal_progress_val; ?>%">
                                    <div class="percentage">
										<?php echo $goal_progress_val . esc_html__( '%', 'helo-essential' ); ?>
                                    </div>
                                    <div class="bar"></div>
                                </div>
                                <ul class="goal-stats">
                                    <li>
										<?php echo esc_html__( 'Goal:', 'helo-essential' ); ?><br>
										<?php echo $goal; ?>
                                    </li>
                                    <li>
										<?php echo esc_html__( 'Achieved:', 'helo-essential' ); ?> <br>
										<?php echo $achieve; ?>
                                    </li>
                                </ul>
                            </div>
                        </div>
                        <a href="<?php the_permalink(); ?>" class="donate-btn">
							<?php echo esc_html( $d_settings_array['donateButtonCaption'] ); ?>
                        </a>
                    </article>
				<?php
				endwhile;
				wp_reset_postdata();
			}
			?>
        </div>
		<?php
	}

	public function getExcerpt() {
		$excerpt = get_the_excerpt();
		if ( $excerpt == '[]' ) {
			return false;
		}
		if ( $excerpt == '' ) {
			return false;
		}

		return $excerpt;
	}
}
