<?php

namespace HeloEssentialApp\Widgets;

use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Typography;
use Elementor\Icons_Manager;
use Elementor\Repeater;
use Elementor\Utils;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Service Accordion
 *
 * Elementor widget for Service Accordion.
 *
 * @since 1.0.0
 */
class Helo_Accordion extends Widget_Base {

	/**
	 * Retrieve the widget name.
	 *
	 * @return string Widget name.
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function get_name() {
		return 'helo--accordion';
	}

	/**
	 * Retrieve the widget title.
	 *
	 * @return string Widget title.
	 * @since 1.0.0
	 *
	 * @access public
	 */

	public function get_title() {
		return esc_html__( 'Helo Accordion', 'helo-essential' );
	}

	/**
	 * Retrieve the widget icon.
	 *
	 * @return string Widget icon.
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function get_icon() {
		return 'eicon-posts-grid';
	}

	/**
	 * Retrieve the list of categories the widget belongs to.
	 *
	 * Used to determine where to display the widget in the editor.
	 *
	 * Note that currently Elementor supports only one category.
	 * When multiple categories passed, Elementor uses the first one.
	 *
	 * @return array Widget categories.
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function get_categories() {
		return [ 'weal-coder-addon' ];
	}

	public function get_style_depends() {
		wp_register_style( 'helo-accordion', HELO_ESSENTIAL_ASSETS_URL . 'css/helo-accordion.css' );

		return [ 'helo-accordion' ];
	}

	public function get_script_depends() {
		wp_register_script( 'helo-accordion', HELO_ESSENTIAL_ASSETS_URL . '/js/widgets/helo-accordion.js', [ 'jquery' ], false, true );

		return [ 'helo-accordion' ];
	}

	/**
	 * Register the widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function register_controls() {

		$this->start_controls_section(
			'sec_content',
			[
				'label' => esc_html__( 'Accordion', 'helo-essential' ),
			]
		);

		$this->add_control(
			'accordion_style',
			[
				'label'   => esc_html__( 'Accordion Style', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'default' => '1',
				'options' => [
					'1' => esc_html__( 'One', 'helo-essential' ),
					'2' => esc_html__( 'Two', 'helo-essential' ),
					'3' => esc_html__( 'Three', 'helo-essential' ),
				],
			]
		);

		$repeater = new Repeater();

		$repeater->add_control(
			'title',
			[
				'label'       => esc_html__( 'Title', 'helo-essential' ),
				'type'        => Controls_Manager::TEXT,
				'label_block' => true,
				'default'     => esc_html__( 'Project Estimate & deal', 'helo-essential' ),
			]
		);

		$repeater->add_control(
			'subtitle',
			[
				'label'       => esc_html__( 'Sub Title', 'helo-essential' ),
				'type'        => Controls_Manager::TEXT,
				'label_block' => true,
				'default'     => esc_html__( 'FIRST STEP', 'helo-essential' ),
			]
		);

		$repeater->add_control(
			'desc',
			[
				'label'   => esc_html__( 'Description', 'helo-essential' ),
				'type'    => Controls_Manager::TEXTAREA,
				'default' => esc_html__( 'Its important to have a clear budget and discuss with your all potential expenses.', 'helo-essential' ),
			]
		);

		$this->add_control(
			'accordion_list',
			[
				'label'     => esc_html__( 'Accordions', 'helo-essential' ),
				'type'      => Controls_Manager::REPEATER,
				'separator' => 'before',
				'fields'    => $repeater->get_controls(),
				'default'   => [ [], [], [] ],
			]
		);

		$this->add_control(
			'selected_icon',
			[
				'label'       => esc_html__( 'Icon', 'helo-essential' ),
				'type'        => Controls_Manager::ICONS,
				'separator'   => 'before',
				'default'     => [
					'value'   => 'fas fa-plus',
					'library' => 'fa-solid',
				],
				'recommended' => [
					'fa-solid'   => [
						'chevron-down',
						'angle-down',
						'angle-double-down',
						'caret-down',
						'caret-square-down',
					],
					'fa-regular' => [
						'caret-square-down',
					],
				],
				'skin'        => 'inline',
				'label_block' => false,
			]
		);

		$this->add_control(
			'selected_active_icon',
			[
				'label'       => esc_html__( 'Active Icon', 'helo-essential' ),
				'type'        => Controls_Manager::ICONS,
				'default'     => [
					'value'   => 'fas fa-minus',
					'library' => 'fa-solid',
				],
				'recommended' => [
					'fa-solid'   => [
						'chevron-up',
						'angle-up',
						'angle-double-up',
						'caret-up',
						'caret-square-up',
					],
					'fa-regular' => [
						'caret-square-up',
					],
				],
				'skin'        => 'inline',
				'label_block' => false,
				'condition'   => [
					'selected_icon[value]!' => '',
				],
			]
		);

		$this->add_control(
			'title_tag',
			[
				'label'   => esc_html__( 'Title HTML Tag', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'options' => [
					'h1'   => 'H1',
					'h2'   => 'H2',
					'h3'   => 'H3',
					'h4'   => 'H4',
					'h5'   => 'H5',
					'h6'   => 'H6',
					'div'  => 'div',
					'span' => 'span',
					'p'    => 'p',
				],
				'default' => 'h3',
			]
		);

		$this->add_control(
			'subtitle_tag',
			[
				'label'   => esc_html__( 'Sub Title HTML Tag', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'options' => [
					'h1'   => 'H1',
					'h2'   => 'H2',
					'h3'   => 'H3',
					'h4'   => 'H4',
					'h5'   => 'H5',
					'h6'   => 'H6',
					'div'  => 'div',
					'span' => 'span',
					'p'    => 'p',
				],
				'default' => 'h4',
			]
		);

		$this->end_controls_section();


		// Style
		$this->register_body_style_controls();
	}

	protected function register_body_style_controls() {
		$this->start_controls_section(
			'style_accordion',
			[
				'label' => esc_html__( 'Accordion', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_responsive_control(
			'item_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .accordion-item' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();

		// Title
		$this->start_controls_section(
			'style_title',
			[
				'label' => esc_html__( 'Title', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'title_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .title' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'title_typo',
				'selector' => '{{WRAPPER}} .title',
			]
		);

		$this->add_responsive_control(
			'title_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .title' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();

		// Sub Title
		$this->start_controls_section(
			'style_subtitle',
			[
				'label' => esc_html__( 'Sub Title', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'subtitle_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .sub-title' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'subtitle_typo',
				'selector' => '{{WRAPPER}} .sub-title',
			]
		);

		$this->end_controls_section();

		// Number
		$this->start_controls_section(
			'style_number',
			[
				'label' => esc_html__( 'Number', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'num_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .number' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'num_typo',
				'selector' => '{{WRAPPER}} .number',
			]
		);

		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name' => 'num_bg',
				'types' => [ 'classic', 'gradient' ],
				'selector' => '{{WRAPPER}} .number',
				'condition' => ['accordion_style' => '2'],
			]
		);

		$this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'num_border',
				'selector' => '{{WRAPPER}} .number',
				'condition' => ['accordion_style' => '2'],
			]
		);

		$this->add_responsive_control(
			'num_radius',
			[
				'label'      => esc_html__( 'Border Radius', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .number' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
                'condition' => ['accordion_style' => '2'],
			]
		);

		$this->add_responsive_control(
			'num_margin',
			[
				'label'      => esc_html__( 'Margin', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .number' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'num_a_color',
			[
				'label'     => esc_html__( 'Active Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .active .number' => 'color: {{VALUE}}',
				],
				'condition' => ['accordion_style' => '2'],
			]
		);

		$this->add_control(
			'num_a_b_color',
			[
				'label'     => esc_html__( 'Active Border Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .active .number' => 'border-color: {{VALUE}}',
				],
				'condition' => ['accordion_style' => '2'],
			]
		);

		$this->end_controls_section();

		// Description
		$this->start_controls_section(
			'style_desc',
			[
				'label' => esc_html__( 'Description', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'desc_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}}  .desc' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'desc_typo',
				'selector' => '{{WRAPPER}} .desc',
			]
		);

		$this->add_responsive_control(
			'desc_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .ct-accordion-content' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();

		// Border Line
		$this->start_controls_section(
			'style_border',
			[
				'label' => esc_html__( 'Border Line', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'line_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .builder--accordion::before' => 'background-color: {{VALUE}}',
				],
			]
		);

		$this->add_control(
			'line_active_color',
			[
				'label'     => esc_html__( 'Active Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .accordion-item.active::before' => 'background-color: {{VALUE}}',
				],
			]
		);

		$this->add_responsive_control(
			'line_left',
			[
				'label'      => esc_html__( 'Position Left', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => - 100,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .builder--accordion::before, {{WRAPPER}} .accordion-item.active::before' => 'left: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();

	}

	/**
	 * Render the widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function render() {
		$settings = $this->get_settings_for_display();
		?>
        <div class="helo--accordion style-<?php echo $settings['accordion_style']; ?>">
			<?php
			foreach ( $settings['accordion_list'] as $key => $item ) {
				$step = $key + 1;
				if ( $step < 10 ) {
					$step = 0 . $step;
				}

				if ( '1' === $settings['accordion_style'] ) {
					$this->render_accordion_one( $settings, $step, $item );
				} elseif( '2' === $settings['accordion_style'] ) {
					$this->render_accordion_two( $settings, $step, $item );
				}else {
					$this->render_accordion_three( $settings, $step, $item );
				}
			}
			?>
        </div>
		<?php
	}

	protected function render_accordion_one( $settings, $step, $item ) { ?>
        <div class="accordion-item">
        <div class="number"> <?php echo esc_html( $step ); ?> </div>

		<?php if ( $item['subtitle'] ): ?>
            <<?php Utils::print_validated_html_tag( $settings['subtitle_tag'] ); ?> class="sub-title">
			<?php echo esc_html( $item['subtitle'] ); ?>
            </<?php Utils::print_validated_html_tag( $settings['subtitle_tag'] ); ?>>
		<?php endif; ?>

		<?php if ( $item['title'] ): ?>
            <<?php Utils::print_validated_html_tag( $settings['title_tag'] ); ?> class="title">
			<?php echo esc_html( $item['title'] ); ?>
            </<?php Utils::print_validated_html_tag( $settings['title_tag'] ); ?>>
		<?php endif; ?>

        <div class="ct-accordion-content">
            <p class="desc"><?php echo $item['desc']; ?></p>
        </div>
        </div>
		<?php
	}

	protected function render_accordion_two( $settings, $step, $item ) { ?>
        <div class="accordion-item">
        <div class="number"> <?php echo esc_html( $step ); ?> </div>

        <div class="content-wrap">
		<?php if ( $item['subtitle'] ): ?>
        <<?php Utils::print_validated_html_tag( $settings['subtitle_tag'] ); ?> class="sub-title">
		<?php echo esc_html( $item['subtitle'] ); ?>
        </<?php Utils::print_validated_html_tag( $settings['subtitle_tag'] ); ?>>
	<?php endif; ?>

		<?php if ( $item['title'] ): ?>
            <<?php Utils::print_validated_html_tag( $settings['title_tag'] ); ?> class="title">
			<?php echo esc_html( $item['title'] ); ?>
            </<?php Utils::print_validated_html_tag( $settings['title_tag'] ); ?>>
		<?php endif; ?>

        <div class="ct-accordion-content">
            <p class="desc"><?php echo $item['desc']; ?></p>
        </div>
        </div>
        </div>
		<?php
	}


	protected function render_accordion_three( $settings, $step, $item ) { ?>
        <div class="accordion-item">
        <div class="number"> <?php echo esc_html( $step ); ?> </div>
		<div class="content_area">
			<?php if ( $item['title'] ): ?>
				<<?php Utils::print_validated_html_tag( $settings['title_tag'] ); ?> class="title">
				<?php echo $item['title']; ?>
				</<?php Utils::print_validated_html_tag( $settings['title_tag'] ); ?>>
			<?php endif; ?>

			<div class="ct-accordion-content">
				<p class="desc"><?php echo $item['desc']; ?></p>
			</div>
		</div>
		<div class="icon_area">
			<span class="accordion-icon-closed accordion-icon">
				<?php Icons_Manager::render_icon( $settings['selected_icon'], [ 'aria-hidden' => 'true' ] ); ?>
			</span>
			<span class="accordion-icon-opened accordion-icon">
				<?php Icons_Manager::render_icon( $settings['selected_active_icon'], [ 'aria-hidden' => 'true' ] ); ?>
			</span>
		</div>

        </div>
		<?php
	}

}
