<?php

namespace HeloEssentialApp\Widgets;

use Elementor\Control_Media;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Image_Size;
use Elementor\Group_Control_Typography;
use Elementor\Icons_Manager;
use Elementor\Plugin;
use Elementor\Repeater;
use Elementor\Utils;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Testimonial
 *
 * Elementor widget for testimonial.
 *
 * @since 1.0.0
 */
class Helo_Hero_Slider extends Widget_Base {

	/**
	 * Retrieve the widget name.
	 *
	 * @return string Widget name.
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function get_name() {
		return 'helo--hero-slider';
	}

	/**
	 * Retrieve the widget title.
	 *
	 * @return string Widget title.
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function get_title() {
		return esc_html__( 'Helo Hero Slider', 'helo-essential' );
	}

	/**
	 * Retrieve the widget icon.
	 *
	 * @return string Widget icon.
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function get_icon() {
		return 'wcf eicon-post-slider';
	}

	/**
	 * Retrieve the list of categories the widget belongs to.
	 *
	 * Used to determine where to display the widget in the editor.
	 *
	 * Note that currently Elementor supports only one category.
	 * When multiple categories passed, Elementor uses the first one.
	 *
	 * @return array Widget categories.
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function get_categories() {
		return [ 'weal-coder-addon' ];
	}

	public function get_style_depends() {
		wp_register_style( 'helo-hero-slider', HELO_ESSENTIAL_ASSETS_URL . 'css/hero-slider.css' );

		return [ 'swiper', 'helo-hero-slider' ];
	}

	public function get_script_depends() {
		wp_register_script( 'helo-hero-slider', HELO_ESSENTIAL_ASSETS_URL . '/js/widgets/hero-slider.js', [ 'jquery' ], false, true );

		return [ 'swiper', 'helo-hero-slider' ];
	}

	/**
	 * Register the widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function register_controls() {
		$this->start_controls_section(
			'section_content',
			[
				'label' => esc_html__( 'Hero Slider', 'helo-essential' ),
			]
		);

		$this->add_control(
			'element_list',
			[
				'label'   => esc_html__( 'Style', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'default' => '1',
				'options' => [
					'1' => esc_html__( 'One', 'helo-essential' ),
					'2' => esc_html__( 'Two', 'helo-essential' ),
				],
			]
		);

		$repeater = new Repeater();

		$repeater->add_control(
			'title',
			[
				'label'       => esc_html__( 'Title', 'helo-essential' ),
				'type'        => Controls_Manager::TEXT,
				'label_block' => true,
				'default'     => esc_html__( 'Business Skyrocketing With Digital Marketing', 'helo-essential' ),
				'description' => 'For Highlight, keep text in [ ]. Ex. [ Text ]',
			]
		);

		$repeater->add_control(
			'image',
			[
				'label'   => esc_html__( 'Image', 'helo-essential' ),
				'type'    => Controls_Manager::MEDIA,
				'dynamic' => [
					'active' => true,
				],
				'default' => [
					'url' => Utils::get_placeholder_image_src(),
				],
			]
		);

		$repeater->add_control(
			'btn_text',
			[
				'label'   => esc_html__( 'Button Text', 'helo-essential' ),
				'type'    => Controls_Manager::TEXT,
				'default' => esc_html__( 'Discover More', 'helo-essential' ),
			]
		);

		$repeater->add_control(
			'btn_icon',
			[
				'label'       => esc_html__( 'Button Icon', 'helo-essential' ),
				'type'        => Controls_Manager::ICONS,
				'skin'        => 'inline',
				'label_block' => false,
				'default'     => [
					'value'   => 'fas fa-arrow-right',
					'library' => 'fa-solid',
				],
			]
		);

		$repeater->add_control(
			'btn_link',
			[
				'label'       => esc_html__( 'Link', 'helo-essential' ),
				'type'        => Controls_Manager::URL,
				'options'     => [ 'url', 'is_external', 'nofollow' ],
				'default'     => [
					'url'         => '#',
					'is_external' => false,
					'nofollow'    => true,
				],
				'label_block' => true,
			]
		);

		$this->add_control(
			'hero_slides',
			[
				'label'   => esc_html__( 'Hero Slides', 'helo-essential' ),
				'type'    => Controls_Manager::REPEATER,
				'fields'  => $repeater->get_controls(),
				'default' => [ [], [], [] ],
			]
		);

		$this->add_control(
			'title_tag',
			[
				'label'   => esc_html__( 'Title HTML Tag', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'options' => [
					'h1'   => 'H1',
					'h2'   => 'H2',
					'h3'   => 'H3',
					'h4'   => 'H4',
					'h5'   => 'H5',
					'h6'   => 'H6',
					'div'  => 'div',
					'span' => 'span',
					'p'    => 'p',
				],
				'default' => 'h2',
			]
		);

		$this->add_control(
			'btn_style',
			[
				'label'   => esc_html__( 'Button Style', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'default' => '1',
				'options' => [
					'1' => esc_html__( '1', 'helo-essential' ),
					'2' => esc_html__( '2', 'helo-essential' ),
				],
			]
		);

		$this->add_control(
			'shape_1',
			[
				'label'   => esc_html__( 'Shape One', 'helo-essential' ),
				'type'    => Controls_Manager::MEDIA,
				'default' => [
					'url' => Utils::get_placeholder_image_src(),
				],
			]
		);

		$this->add_control(
			'shape_2',
			[
				'label'   => esc_html__( 'Shape Two', 'helo-essential' ),
				'type'    => Controls_Manager::MEDIA,
				'default' => [
					'url' => Utils::get_placeholder_image_src(),
				],
			]
		);

		$this->add_responsive_control(
			'slider_align',
			[
				'label'     => esc_html__( 'Alignment', 'helo-essential' ),
				'type'      => Controls_Manager::CHOOSE,
				'options'   => [
					'start'  => [
						'title' => esc_html__( 'Left', 'helo-essential' ),
						'icon'  => 'eicon-text-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'helo-essential' ),
						'icon'  => 'eicon-text-align-center',
					],
					'end'    => [
						'title' => esc_html__( 'Right', 'helo-essential' ),
						'icon'  => 'eicon-text-align-right',
					],
				],
				'toggle'    => true,
				'selectors' => [
					'{{WRAPPER}} .swiper-slide'       => 'text-align: {{VALUE}};',
					'{{WRAPPER}} .swiper-slide .wrap' => 'justify-content: {{VALUE}};',
				],
			]
		);

		$this->end_controls_section();


		// Slider control
		$this->slider_controls();

		//layout style
		$this->start_controls_section(
			'section_slide_style',
			[
				'label' => esc_html__( 'Slide', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_responsive_control(
			'slide_direction',
			[
				'label' => esc_html__( 'Direction', 'helo-essential' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'row',
				'options' => [
					'row' => esc_html__( 'Row', 'helo-essential' ),
					'row-reverse' => esc_html__( 'Row Reverse', 'helo-essential' ),
					'column' => esc_html__( 'Column', 'helo-essential' ),
					'column-reverse' => esc_html__( 'Column Reverse', 'helo-essential' ),
				],
				'selectors' => [
					'{{WRAPPER}} .slide' => 'flex-direction: {{VALUE}};',
				],
                'condition' => [ 'element_list' => '2' ],
			]
		);

		$this->add_responsive_control(
			'content_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .content' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();

		// Title
		$this->start_controls_section(
			'style_title',
			[
				'label' => esc_html__( 'Title', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'title_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'default'   => '',
				'selectors' => [
					'{{WRAPPER}} .title' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'title_typo',
				'selector' => '{{WRAPPER}} .title',
			]
		);

		$this->add_responsive_control(
			'title_width',
			[
				'label'      => esc_html__( 'Max Width', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%', 'em', 'rem' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 1000,
					],
					'%'  => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .title' => 'max-width: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'title_margin',
			[
				'label'      => esc_html__( 'Margin', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .title' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		// Highlight
		$this->add_control(
			'highlight_heading',
			[
				'label'     => esc_html__( 'Highlight', 'helo-essential' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'highlight_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'default'   => '',
				'selectors' => [
					'{{WRAPPER}} .highlight' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'highlight_typo',
				'selector' => '{{WRAPPER}} .highlight',
			]
		);

		$this->end_controls_section();

		// Image style.
		$this->start_controls_section(
			'style_image',
			[
				'label'     => esc_html__( 'Image', 'helo-essential' ),
				'tab'       => Controls_Manager::TAB_STYLE,
				'condition' => [ 'element_list' => '2' ],
			]
		);

		$this->add_responsive_control(
			'img_width',
			[
				'label'      => esc_html__( 'Width', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%', 'em', 'rem' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 1000,
					],
					'%'  => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .thumb img' => 'width: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'img_height',
			[
				'label'      => esc_html__( 'Height', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%', 'em', 'rem' ],
				'range'      => [
					'px' => [
						'min'  => 1,
						'max'  => 700,
						'step' => 1,
					],
					'%'  => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .thumb img' => 'height: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'img_wr_width',
			[
				'label'      => esc_html__( 'Wrapper Width', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%', 'em', 'rem' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 1000,
					],
					'%'  => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .thumb' => 'flex-basis: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();

		// Button Style
		$this->button_style_controls();

		// Shape style.
		$this->start_controls_section(
			'style_shape',
			[
				'label'     => esc_html__( 'Shape', 'helo-essential' ),
				'tab'       => Controls_Manager::TAB_STYLE,
				'condition' => [ 'element_list' => '1' ],
			]
		);
		// Shape One
		$this->add_control(
			'shape1_heading',
			[
				'label'     => esc_html__( 'Shape One', 'helo-essential' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			's1_blend_mode',
			[
				'label'     => esc_html__( 'Blend Mode', 'helo-essential' ),
				'type'      => Controls_Manager::SELECT,
				'default'   => 'hard-light',
				'options'   => [
					''            => esc_html__( 'Normal', 'helo-essential' ),
					'multiply'    => 'Multiply',
					'hard-light'  => 'Hard Light',
					'screen'      => 'Screen',
					'overlay'     => 'Overlay',
					'darken'      => 'Darken',
					'lighten'     => 'Lighten',
					'color-dodge' => 'Color Dodge',
					'saturation'  => 'Saturation',
					'color'       => 'Color',
					'difference'  => 'Difference',
					'exclusion'   => 'Exclusion',
					'hue'         => 'Hue',
					'luminosity'  => 'Luminosity',
				],
				'selectors' => [
					'{{WRAPPER}} .shape-1' => 'mix-blend-mode: {{VALUE}}',
				],
				'separator' => 'none',
			]
		);

		$this->add_responsive_control(
			'shape1_size',
			[
				'label'      => esc_html__( 'Width', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 1500,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .shape-1' => 'width: {{SIZE}}{{UNIT}};',
				],
			]
		);

		// Shape Two
		$this->add_control(
			'shape2_heading',
			[
				'label'     => esc_html__( 'Shape Two', 'helo-essential' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			's2_blend_mode',
			[
				'label'     => esc_html__( 'Blend Mode', 'helo-essential' ),
				'type'      => Controls_Manager::SELECT,
				'default'   => 'multiply',
				'options'   => [
					''            => esc_html__( 'Normal', 'helo-essential' ),
					'multiply'    => 'Multiply',
					'hard-light'  => 'Hard Light',
					'screen'      => 'Screen',
					'overlay'     => 'Overlay',
					'darken'      => 'Darken',
					'lighten'     => 'Lighten',
					'color-dodge' => 'Color Dodge',
					'saturation'  => 'Saturation',
					'color'       => 'Color',
					'difference'  => 'Difference',
					'exclusion'   => 'Exclusion',
					'hue'         => 'Hue',
					'luminosity'  => 'Luminosity',
				],
				'selectors' => [
					'{{WRAPPER}} .shape-2' => 'mix-blend-mode: {{VALUE}}',
				],
			]
		);

		$this->add_responsive_control(
			'shape2_size',
			[
				'label'      => esc_html__( 'Width', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 1500,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .shape-2' => 'width: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();

		// Slider navigation style
		$this->slider_navigation_style();

		// Slider pagination style
		$this->slider_pagination_style();
	}

	// Button Style
	public function button_style_controls() {

		$this->start_controls_section(
			'style_button',
			[
				'label' => esc_html__( 'Button', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'btn_typo',
				'selector' => '{{WRAPPER}} .helo--btn',
			]
		);

		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name'      => 'btn_bg',
				'types'     => [ 'classic', 'gradient' ],
				'exclude'   => [ 'image' ],
				'selector'  => '{{WRAPPER}} .helo--btn',
			]
		);

		$this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name'      => 'btn_border',
				'selector'  => '{{WRAPPER}} .helo--btn',
                'condition' => [ 'btn_style' => '1'],
			]
		);

		$this->add_responsive_control(
			'btn_border_radius',
			[
				'label'      => esc_html__( 'Border Radius', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%' ],
				'selectors'  => [
					'{{WRAPPER}} .helo--btn' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [ 'btn_style' => '1'],
			]
		);

		$this->add_responsive_control(
			'btn_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%' ],
				'selectors'  => [
					'{{WRAPPER}} .helo--btn' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		// Icon Style
		$this->add_control(
			'btn_icon_heading',
			[
				'label'     => esc_html__( 'Icon', 'helo-essential' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_responsive_control(
			'btn_icon_size',
			[
				'label'      => esc_html__( 'Icon Size', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 200,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .helo--btn .icon' => 'font-size: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'btn_gap',
			[
				'label'      => esc_html__( 'Gap', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .helo--btn' => 'gap: {{SIZE}}{{UNIT}};',
				],
			]
		);

		// Tabs
		$this->start_controls_tabs(
			'btn_style_tabs'
		);

		$this->start_controls_tab(
			'btn_normal_tab',
			[
				'label' => esc_html__( 'Normal', 'helo-essential' ),
			]
		);

		$this->add_control(
			'btn_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .helo--btn' => 'color: {{VALUE}}; fill: {{VALUE}}',
				],
			]
		);

		$this->end_controls_tab();

		$this->start_controls_tab(
			'btn_hover_tab',
			[
				'label' => esc_html__( 'Hover', 'helo-essential' ),
			]
		);

		$this->add_control(
			'btn_h_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .helo--btn:hover' => 'color: {{VALUE}}; fill: {{VALUE}}',
				],
			]
		);

		$this->add_control(
			'btn_h_border',
			[
				'label'     => esc_html__( 'Border Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .helo--btn:hover' => 'border-color: {{VALUE}}',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name'      => 'btn_h_bg',
				'types'     => [ 'classic', 'gradient' ],
				'exclude'   => [ 'image' ],
				'selector'  => '{{WRAPPER}} .helo--btn:hover',
			]
		);

		$this->end_controls_tab();

		$this->end_controls_tabs();

		$this->end_controls_section();
	}

	/**
	 * Register the slider controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 */
	private function slider_controls() {
		$this->start_controls_section(
			'section_slider_options',
			[
				'label' => esc_html__( 'Slider Options', 'helo-essential' ),
			]
		);

		$slides_to_show = range( 1, 10 );
		$slides_to_show = array_combine( $slides_to_show, $slides_to_show );

		$this->add_responsive_control(
			'slides_to_show',
			[
				'label'       => esc_html__( 'Slides to Show', 'helo-essential' ),
				'type'        => Controls_Manager::SELECT,
				'default'     => 'auto',
				'required'    => true,
				'options'     => [
					                 'auto' => esc_html__( 'Auto', 'helo-essential' ),
				                 ] + $slides_to_show,
				'render_type' => 'template',
				'selectors'   => [
					'{{WRAPPER}}' => '--slides-to-show: {{VALUE}}',
				],
			]
		);

		$this->add_control(
			'autoplay',
			[
				'label'   => esc_html__( 'Autoplay', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'default' => 'yes',
				'options' => [
					'yes' => esc_html__( 'Yes', 'helo-essential' ),
					'no'  => esc_html__( 'No', 'helo-essential' ),
				],
			]
		);

		$this->add_control(
			'autoplay_delay',
			[
				'label'     => esc_html__( 'Autoplay delay', 'helo-essential' ),
				'type'      => Controls_Manager::NUMBER,
				'default'   => 3000,
				'condition' => [
					'autoplay' => 'yes',
				],
			]
		);

		$this->add_control(
			'autoplay_interaction',
			[
				'label'     => esc_html__( 'Autoplay Interaction', 'helo-essential' ),
				'type'      => Controls_Manager::SELECT,
				'default'   => 'true',
				'options'   => [
					'true'  => esc_html__( 'Yes', 'helo-essential' ),
					'false' => esc_html__( 'No', 'helo-essential' ),
				],
				'condition' => [
					'autoplay' => 'yes',
				],
			]
		);

		$this->add_control(
			'allow_touch_move',
			[
				'label'     => esc_html__( 'Allow Touch Move', 'helo-essential' ),
				'type'      => Controls_Manager::SELECT,
				'separator' => 'before',
				'default'   => 'false',
				'options'   => [
					'true'  => esc_html__( 'Yes', 'helo-essential' ),
					'false' => esc_html__( 'No', 'helo-essential' ),
				],
			]
		);

		// Loop requires a re-render so no 'render_type = none'
		$this->add_control(
			'loop',
			[
				'label'   => esc_html__( 'Loop', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'default' => 'true',
				'options' => [
					'true'  => esc_html__( 'Yes', 'helo-essential' ),
					'false' => esc_html__( 'No', 'helo-essential' ),
				],
			]
		);

		$this->add_control(
			'slide_effect',
			[
				'label'   => esc_html__( 'Effect', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'default' => 'slide',
				'options' => [
					'slide'     => esc_html__( 'Slide', 'helo-essential' ),
					'fade'      => esc_html__( 'Fade', 'helo-essential' ),
					'flip'      => esc_html__( 'Flip', 'helo-essential' ),
					'cube'      => esc_html__( 'Cube', 'helo-essential' ),
					'coverflow' => esc_html__( 'CoderFlow', 'helo-essential' ),
				],
			]
		);

		$this->add_control(
			'speed',
			[
				'label'   => esc_html__( 'Animation Speed', 'helo-essential' ),
				'type'    => Controls_Manager::NUMBER,
				'default' => 500,
			]
		);

		$this->add_control(
			'space_between',
			[
				'label'       => esc_html__( 'Space Between', 'helo-essential' ),
				'type'        => Controls_Manager::NUMBER,
				'default'     => 20,
				'render_type' => 'template',
				'selectors'   => [
					'{{WRAPPER}}' => '--space-between: {{VALUE}}px;',
				],
			]
		);

		//slider navigation
		$this->add_control(
			'navigation',
			[
				'label'     => esc_html__( 'Controls', 'helo-essential' ),
				'type'      => Controls_Manager::SELECT,
				'separator' => 'before',
				'default'   => 'both',
				'options'   => [
					'both'   => esc_html__( 'Both', 'helo-essential' ),
					'arrows' => esc_html__( 'Navigation', 'helo-essential' ),
					'dots'   => esc_html__( 'Pagination', 'helo-essential' ),
					'none'   => esc_html__( 'None', 'helo-essential' ),
				],
			]
		);

		$this->add_control(
			'navigation_previous_icon',
			[
				'label'            => esc_html__( 'Previous Arrow Icon', 'helo-essential' ),
				'type'             => Controls_Manager::ICONS,
				'fa4compatibility' => 'icon',
				'skin'             => 'inline',
				'label_block'      => false,
				'skin_settings'    => [
					'inline' => [
						'none' => [
							'label' => 'Default',
							'icon'  => 'eicon-chevron-left',
						],
						'icon' => [
							'icon' => 'eicon-star',
						],
					],
				],
				'recommended'      => [
					'fa-regular' => [
						'arrow-alt-circle-left',
						'caret-square-left',
					],
					'fa-solid'   => [
						'angle-double-left',
						'angle-left',
						'arrow-alt-circle-left',
						'arrow-circle-left',
						'arrow-left',
						'caret-left',
						'caret-square-left',
						'chevron-circle-left',
						'chevron-left',
						'long-arrow-alt-left',
					],
				],
				'conditions'       => [
					'relation' => 'or',
					'terms'    => [
						[
							'name'     => 'navigation',
							'operator' => '=',
							'value'    => 'both',
						],
						[
							'name'     => 'navigation',
							'operator' => '=',
							'value'    => 'arrows',
						],
					],
				],
			]
		);

		$this->add_control(
			'navigation_next_icon',
			[
				'label'            => esc_html__( 'Next Arrow Icon', 'helo-essential' ),
				'type'             => Controls_Manager::ICONS,
				'fa4compatibility' => 'icon',
				'skin'             => 'inline',
				'label_block'      => false,
				'skin_settings'    => [
					'inline' => [
						'none' => [
							'label' => 'Default',
							'icon'  => 'eicon-chevron-right',
						],
						'icon' => [
							'icon' => 'eicon-star',
						],
					],
				],
				'recommended'      => [
					'fa-regular' => [
						'arrow-alt-circle-right',
						'caret-square-right',
					],
					'fa-solid'   => [
						'angle-double-right',
						'angle-right',
						'arrow-alt-circle-right',
						'arrow-circle-right',
						'arrow-right',
						'caret-right',
						'caret-square-right',
						'chevron-circle-right',
						'chevron-right',
						'long-arrow-alt-right',
					],
				],
				'conditions'       => [
					'relation' => 'or',
					'terms'    => [
						[
							'name'     => 'navigation',
							'operator' => '=',
							'value'    => 'both',
						],
						[
							'name'     => 'navigation',
							'operator' => '=',
							'value'    => 'arrows',
						],
					],
				],
				'separator' => 'after',
			]
		);

		$this->add_control(
			'pagination_type',
			[
				'label'     => esc_html__( 'Pagination Type', 'helo-essential' ),
				'type'      => Controls_Manager::SELECT,
				'default'   => 'bullets',
				'options'   => [
					'bullets'   => esc_html__( 'Bullets', 'helo-essential' ),
					'fraction' => esc_html__( 'Fraction', 'helo-essential' ),
				],
				'conditions'       => [
					'relation' => 'or',
					'terms'    => [
						[
							'name'     => 'navigation',
							'operator' => '=',
							'value'    => 'both',
						],
						[
							'name'     => 'navigation',
							'operator' => '=',
							'value'    => 'dots',
						],
					],
				],
			]
		);


		$this->end_controls_section();
	}

	/**
	 * Register the slider navigation style controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 */
	private function slider_navigation_style() {
		$this->start_controls_section(
			'style_navigation',
			[
				'label'     => esc_html__( 'Navigation', 'helo-essential' ),
				'tab'       => Controls_Manager::TAB_STYLE,
				'condition' => [
					'navigation' => [ 'arrows', 'both' ],
				],
			]
		);

		$this->add_control(
			'navi_color',
			[
				'label' => esc_html__( 'Color', 'helo-essential' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .wcf-arrow' => 'fill: {{VALUE}}; color: {{VALUE}}',
				],
			]
		);

		$this->add_responsive_control(
			'nav_icon_size',
			[
				'label'      => esc_html__( 'Icon Size', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 1000,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .wcf-arrow' => 'font-size: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'navi_pos',
			[
				'label' => esc_html__( 'Position', 'helo-essential' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'relative',
				'options' => [
					'relative' => esc_html__( 'Default', 'helo-essential' ),
					'absolute' => esc_html__( 'Absolute', 'helo-essential' ),
				],
				'selectors' => [
					'{{WRAPPER}} .hero-navigation' => 'position: {{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'navi_align',
			[
				'label' => esc_html__( 'Alignment', 'helo-essential' ),
				'type' => Controls_Manager::CHOOSE,
				'options' => [
					'start' => [
						'title' => esc_html__( 'Left', 'helo-essential' ),
						'icon' => 'eicon-text-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'helo-essential' ),
						'icon' => 'eicon-text-align-center',
					],
					'end' => [
						'title' => esc_html__( 'Right', 'helo-essential' ),
						'icon' => 'eicon-text-align-right',
					],
					'space-between' => [
						'title' => esc_html__( 'Stretch', 'helo-essential' ),
						'icon' => 'eicon-align-stretch-h',
					],
				],
				'toggle' => true,
				'selectors' => [
					'{{WRAPPER}} .hero-navigation' => 'justify-content: {{VALUE}};',
				],
                'condition' => [ 'navi_pos' => 'relative' ],
			]
		);

		$this->add_control(
			'navi_horizontal_pos',
			[
				'label' => esc_html__( 'Horizontal Orientation', 'helo-essential' ),
				'type' => Controls_Manager::CHOOSE,
				'prefix_class' => 'helo-hero-slider-hrp-',
				'options' => [
					'left' => [
						'title' => esc_html__( 'Left', 'helo-essential' ),
						'icon' => 'eicon-h-align-left',
					],
					'right' => [
						'title' => esc_html__( 'Right', 'helo-essential' ),
						'icon' => 'eicon-h-align-right',
					],
				],
				'condition' => [ 'navi_pos' => 'absolute' ],
			]
		);

		$this->add_responsive_control(
			'navi_hr_offset',
			[
				'label'      => esc_html__( 'Offset', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'range'      => [
					'px' => [
						'min' => -1000,
						'max' => 1000,
					],
					'%'  => [
						'min' => -100,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .hero-navigation' => '--hr-offset: {{SIZE}}{{UNIT}};',
				],
				'condition' => [ 'navi_pos' => 'absolute' ],
			]
		);

		$this->add_control(
			'navi_vertical_pos',
			[
				'label' => esc_html__( 'Vertical Orientation', 'helo-essential' ),
				'type' => Controls_Manager::CHOOSE,
				'prefix_class' => 'helo-hero-slider-vrp-',
				'options' => [
					'top' => [
						'title' => esc_html__( 'Top', 'helo-essential' ),
						'icon' => 'eicon-v-align-top',
					],
					'bottom' => [
						'title' => esc_html__( 'Bottom', 'helo-essential' ),
						'icon' => 'eicon-v-align-bottom',
					],
				],
				'condition' => [ 'navi_pos' => 'absolute' ],
			]
		);

		$this->add_responsive_control(
			'navi_vr_offset',
			[
				'label'      => esc_html__( 'Offset', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'range'      => [
					'px' => [
						'min' => -1000,
						'max' => 1000,
					],
					'%'  => [
						'min' => -100,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .hero-navigation' => '--vr-offset: {{SIZE}}{{UNIT}};',
				],
				'condition' => [ 'navi_pos' => 'absolute' ],
			]
		);


		// Pagination Active
		$this->add_control(
			'navi_hover_heading',
			[
				'label'     => esc_html__( 'Hover', 'helo-essential' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'navi_h_color',
			[
				'label' => esc_html__( 'Color', 'helo-essential' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .wcf-arrow:hover' => 'fill: {{VALUE}}; color: {{VALUE}}',
				],
			]
		);

		$this->end_controls_section();
	}


	// Pagination Style
	private function slider_pagination_style() {
		$this->start_controls_section(
			'style_pagination',
			[
				'label'     => esc_html__( 'Pagination', 'helo-essential' ),
				'tab'       => Controls_Manager::TAB_STYLE,
				'condition' => [
					'navigation' => [ 'dots', 'both' ],
				],
			]
		);

		$this->add_control(
			'pagination_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .swiper-pagination-bullet' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} .swiper-pagination-fraction' => 'color: {{VALUE}}; fill: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'fraction_typo',
				'selector' => '{{WRAPPER}} .swiper-pagination-fraction',
				'condition' => [ 'pagination_type' => 'fraction' ],
			]
		);

		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name'     => 'pagination_bg',
				'types'    => [ 'classic', 'gradient' ],
				'selector' => '{{WRAPPER}} .swiper-pagination-fraction',
				'condition' => [ 'pagination_type' => 'fraction' ],
			]
		);

		$this->add_responsive_control(
			'bullet_width',
			[
				'label'      => esc_html__( 'Width', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .swiper-pagination-bullet' => 'width: {{SIZE}}{{UNIT}};',
				],
				'condition' => [ 'pagination_type' => 'bullets' ],
			]
		);

		$this->add_responsive_control(
			'bullet_height',
			[
				'label'      => esc_html__( 'Height', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 20,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .swiper-pagination-bullet' => 'height: {{SIZE}}{{UNIT}};',
				],
				'condition' => [ 'pagination_type' => 'bullets' ],
			]
		);

		$this->add_responsive_control(
			'pagi_b_radius',
			[
				'label'      => esc_html__( 'Border Radius', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .swiper-pagination-bullet' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
				'condition' => [ 'pagination_type' => 'bullets' ],
			]
		);

		$this->add_responsive_control(
			'pagi_gap',
			[
				'label'      => esc_html__( 'Gap', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .swiper-pagination' => 'gap: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'pagi_direction',
			[
				'label' => esc_html__( 'Direction', 'helo-essential' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'row',
				'options' => [
					'row' => esc_html__( 'Row', 'helo-essential' ),
					'column' => esc_html__( 'Column', 'helo-essential' ),
				],
				'selectors' => [
					'{{WRAPPER}} .swiper-pagination' => 'flex-direction: {{VALUE}};',
				],
				'condition' => [ 'pagination_type' => 'bullets' ],
			]
		);

		$this->add_control(
			'pagi_pos',
			[
				'label' => esc_html__( 'Position', 'helo-essential' ),
				'type' => Controls_Manager::SELECT,
				'default' => 'relative',
				'options' => [
					'relative' => esc_html__( 'Default', 'helo-essential' ),
					'absolute' => esc_html__( 'Absolute', 'helo-essential' ),
				],
				'selectors' => [
					'{{WRAPPER}} .swiper-pagination' => 'position: {{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'pagi_align',
			[
				'label' => esc_html__( 'Alignment', 'helo-essential' ),
				'type' => Controls_Manager::CHOOSE,
				'options' => [
					'left' => [
						'title' => esc_html__( 'Left', 'helo-essential' ),
						'icon' => 'eicon-text-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'helo-essential' ),
						'icon' => 'eicon-text-align-center',
					],
					'right' => [
						'title' => esc_html__( 'Right', 'helo-essential' ),
						'icon' => 'eicon-text-align-right',
					],
				],
				'toggle' => true,
				'selectors' => [
					'{{WRAPPER}} .hero-pagination' => 'text-align: {{VALUE}};',
				],
				'condition' => [ 'pagi_pos' => 'relative' ],
			]
		);

		$this->add_control(
			'pagi_hr_pos',
			[
				'label' => esc_html__( 'Horizontal Orientation', 'helo-essential' ),
				'type' => Controls_Manager::CHOOSE,
				'prefix_class' => 'helo-hero-slider-pg-hr-',
				'options' => [
					'left' => [
						'title' => esc_html__( 'Left', 'helo-essential' ),
						'icon' => 'eicon-h-align-left',
					],
					'right' => [
						'title' => esc_html__( 'Right', 'helo-essential' ),
						'icon' => 'eicon-h-align-right',
					],
				],
				'condition' => [ 'pagi_pos' => 'absolute' ],
			]
		);

		$this->add_responsive_control(
			'pagi_hr_offset',
			[
				'label'      => esc_html__( 'Offset', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'range'      => [
					'px' => [
						'min' => -1000,
						'max' => 1000,
					],
					'%'  => [
						'min' => -100,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .swiper-pagination' => '--pg-hr-offset: {{SIZE}}{{UNIT}};',
				],
				'condition' => [ 'pagi_pos' => 'absolute' ],
			]
		);

		$this->add_control(
			'pagi_vr_pos',
			[
				'label' => esc_html__( 'Vertical Orientation', 'helo-essential' ),
				'type' => Controls_Manager::CHOOSE,
				'prefix_class' => 'helo-hero-slider-pg-vr-',
				'options' => [
					'top' => [
						'title' => esc_html__( 'Top', 'helo-essential' ),
						'icon' => 'eicon-v-align-top',
					],
					'bottom' => [
						'title' => esc_html__( 'Bottom', 'helo-essential' ),
						'icon' => 'eicon-v-align-bottom',
					],
				],
				'condition' => [ 'pagi_pos' => 'absolute' ],
			]
		);

		$this->add_responsive_control(
			'pagi_vr_offset',
			[
				'label'      => esc_html__( 'Offset', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'range'      => [
					'px' => [
						'min' => -1000,
						'max' => 1000,
					],
					'%'  => [
						'min' => -100,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .swiper-pagination' => '--pg-vr-offset: {{SIZE}}{{UNIT}};',
				],
				'condition' => [ 'pagi_pos' => 'absolute' ],
			]
		);

		$this->add_responsive_control(
			'pagi_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .swiper-pagination' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
                'separator' => 'before',
			]
		);

		// Pagination Active
		$this->add_control(
			'pagi_a_heading',
			[
				'label'     => esc_html__( 'Active', 'helo-essential' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'pagi_a_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .swiper-pagination-bullet-active' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} .swiper-pagination-current, {{WRAPPER}} .dash' => 'color: {{VALUE}}; fill: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'fraction_a_typo',
				'selector' => '{{WRAPPER}} .swiper-pagination-current, {{WRAPPER}} .dash',
				'condition' => [ 'pagination_type' => 'fraction' ],
			]
		);

		$this->add_responsive_control(
			'bullet_a_width',
			[
				'label'      => esc_html__( 'Width', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .swiper-pagination-bullet-active' => 'width: {{SIZE}}{{UNIT}};',
				],
				'condition' => [ 'pagination_type' => 'bullets' ],
			]
		);

		$this->add_responsive_control(
			'bullet_a_height',
			[
				'label'      => esc_html__( 'Height', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 20,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .swiper-pagination-bullet-active' => 'height: {{SIZE}}{{UNIT}};',
				],
				'condition' => [ 'pagination_type' => 'bullets' ],
			]
		);

		$this->end_controls_section();
	}

	/**
	 * Render the widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function render() {
		$settings = $this->get_settings_for_display();

		if ( empty( $settings['hero_slides'] ) ) {
			return;
		}

		//slider settings
		$show_dots   = ( in_array( $settings['navigation'], [ 'dots', 'both' ] ) );
		$show_arrows = ( in_array( $settings['navigation'], [ 'arrows', 'both' ] ) );

		$slider_settings = $this->get_slider_settings( $settings );

		$this->add_render_attribute(
			'wrapper',
			[
				'class'         => [ 'helo_hero_wrapper helo_hero_slider-' . $settings['element_list'] ],
				'data-settings' => json_encode( $slider_settings ), //phpcs:ignore
			]
		);

		$this->add_render_attribute(
			'carousel-wrapper',
			[
				'class' => 'helo_hero_slider swiper',
				'style' => 'position: static',
				'dir'   => 'ltr',
			]
		);

		?>

        <div <?php $this->print_render_attribute_string( 'wrapper' ); ?>>
            <div <?php $this->print_render_attribute_string( 'carousel-wrapper' ); ?>>
                <div class="swiper-wrapper">
					<?php foreach ( $settings['hero_slides'] as $index => $item ) { ?>
                        <div class="swiper-slide">
							<?php
							if ( '2' === $settings['element_list'] ) {
								$this->render_hero_slider_two( $settings, $item, $index );
							} else {
								$this->render_hero_slider_one( $settings, $item, $index );
							}
							?>
                        </div>
					<?php } ?>
                </div>
            </div>
            <!-- navigation and pagination -->
			<?php if ( 1 < count( $settings['hero_slides'] ) ) : ?>
                <div class="controls-wrap">
					<?php if ( $show_arrows ) : ?>
                        <div class="hero-navigation">
                            <div class="wcf-arrow wcf-arrow-prev" role="button" tabindex="0">
								<?php $this->render_swiper_button( 'previous' ); ?>
                            </div>
                            <div class="wcf-arrow wcf-arrow-next" role="button" tabindex="0">
								<?php $this->render_swiper_button( 'next' ); ?>
                            </div>
                        </div>
					<?php endif; ?>

					<?php if ( $show_dots ) : ?>
                        <div class="hero-pagination">
                            <div class="swiper-pagination"></div>
                        </div>
					<?php endif; ?>
                </div>
			<?php endif; ?>
        </div>
		<?php
	}

	private function get_slider_settings( $settings ) {

		$show_dots   = ( in_array( $settings['navigation'], [ 'dots', 'both' ] ) );
		$show_arrows = ( in_array( $settings['navigation'], [ 'arrows', 'both' ] ) );

		$slider_settings = [
			'loop'           => 'true' === $settings['loop'],
			'speed'          => $settings['speed'],
			'allowTouchMove' => $settings['allow_touch_move'],
			'slidesPerView'  => $settings['slides_to_show'],
			'spaceBetween'   => $settings['space_between'],
			'effect'         => $settings['slide_effect'],
		];

		if ( 'yes' === $settings['autoplay'] ) {
			$slider_settings['autoplay'] = [
				'delay'                => $settings['autoplay_delay'],
				'disableOnInteraction' => $settings['autoplay_interaction'],
			];
		}

		if ( $show_arrows ) {
			$slider_settings['navigation'] = [
				'nextEl' => '.elementor-element-' . $this->get_id() . ' .wcf-arrow-next',
				'prevEl' => '.elementor-element-' . $this->get_id() . ' .wcf-arrow-prev',
			];
		}

		if ( $show_dots ) {
			$slider_settings['pagination'] = [
				'el'   => '.elementor-element-' . $this->get_id() . ' .swiper-pagination',
				'type' => $settings['pagination_type'],
                'clickable' => true,
			];
		}

		//slider breakpoints
		$active_breakpoints = Plugin::$instance->breakpoints->get_active_breakpoints();

		foreach ( $active_breakpoints as $breakpoint_name => $breakpoint ) {
			$slides_to_show = ! empty( $settings[ 'slides_to_show_' . $breakpoint_name ] ) ? $settings[ 'slides_to_show_' . $breakpoint_name ] : $settings['slides_to_show'];

			$slider_settings['breakpoints'][ $breakpoint->get_value() ]['slidesPerView'] = $slides_to_show;
		}

		return $slider_settings;
	}

	/**
	 * Render swiper button.
	 *
	 * @since 1.0.0
	 *
	 * @access private
	 */
	private function render_swiper_button( $type ) {
		$direction     = 'next' === $type ? 'right' : 'left';
		$icon_settings = $this->get_settings_for_display( 'navigation_' . $type . '_icon' );

		if ( empty( $icon_settings['value'] ) ) {
			$icon_settings = [
				'library' => 'eicons',
				'value'   => 'eicon-chevron-' . $direction,
			];
		}

		Icons_Manager::render_icon( $icon_settings, [ 'aria-hidden' => 'true' ] );
	}

	protected function render_hero_slider_one( $settings, $item, $index ) {
		$btn_id = 'btn_link_' . $index;
		if ( ! empty( $item['btn_link']['url'] ) ) {
			$this->add_link_attributes( $btn_id, $item['btn_link'] );
		}
		?>

        <div class="slide" style="background-image: url('<?php echo esc_url( $item['image']['url'] ); ?>');">
            <div class="content">
				<?php
				$title = $item['title'];
				preg_match_all( '/\[([^\]]*)\]/', $title, $matches );
				foreach ( $matches[0] as $key => $value ) {
					$title = str_replace( $value, '<span class="highlight">' . $matches[1][ $key ] . '</span>', $title, );
				}
				?>
                <<?php Utils::print_validated_html_tag( $settings['title_tag'] ); ?> class="title">
				<?php echo wp_kses_post( $title ); ?>
            </<?php Utils::print_validated_html_tag( $settings['title_tag'] ); ?>>

            <div class="helo-btn-wrapper style-<?php echo esc_html( $settings['btn_style'] ); ?>">
				<?php if ( '2' === $settings['btn_style'] ) { ?>
                    <a <?php $this->print_render_attribute_string( $btn_id ); ?> class="helo--btn btn-border-divide">
                        <span class="text"><?php echo esc_html( $item['btn_text'] ); ?></span>
                        <span class="icon">
                            <?php Icons_Manager::render_icon( $item['btn_icon'], [ 'aria-hidden' => 'true' ] ); ?>
                            <?php Icons_Manager::render_icon( $item['btn_icon'], [ 'aria-hidden' => 'true' ] ); ?>
                        </span>
                    </a>
				<?php } else { ?>
                    <a <?php $this->print_render_attribute_string( $btn_id ); ?> class="helo--btn">
	                    <?php echo esc_html( $item['btn_text'] ); ?>
                        <span class="icon"><?php Icons_Manager::render_icon( $item['btn_icon'], [ 'aria-hidden' => 'true' ] ); ?> </span>
                    </a>
				<?php } ?>
            </div>
        </div>
        <span class="shape-2">
                <img src="<?php echo $settings['shape_2']['url']; ?>" alt="">
            </span>
        <span class="shape-1">
                <img src="<?php echo $settings['shape_1']['url']; ?>" alt="">
            </span>
        </div>
		<?php
	}

	protected function render_hero_slider_two( $settings, $item, $index ) {
		$btn_id = 'btn_link_' . $index;
		if ( ! empty( $item['btn_link']['url'] ) ) {
			$this->add_link_attributes( $btn_id, $item['btn_link'] );
		}
		?>
        <div class="slide">
            <div class="content">
                <?php
                $title = $item['title'];
                preg_match_all( '/\[([^\]]*)\]/', $title, $matches );
                foreach ( $matches[0] as $key => $value ) {
                    $title = str_replace( $value, '<span class="highlight">' . $matches[1][ $key ] . '</span>', $title, );
                }
                ?>
                <<?php Utils::print_validated_html_tag( $settings['title_tag'] ); ?> class="title">
                <?php echo wp_kses_post( $title ); ?>
                </<?php Utils::print_validated_html_tag( $settings['title_tag'] ); ?>>

                <div class="helo-btn-wrapper style-<?php echo esc_html( $settings['btn_style'] ); ?>">
                    <?php if ( '2' === $settings['btn_style'] ) { ?>
                        <a <?php $this->print_render_attribute_string( $btn_id ); ?> class="helo--btn btn-border-divide">
                            <span class="text"><?php echo esc_html( $item['btn_text'] ); ?></span>
                            <span class="icon">
                                    <?php Icons_Manager::render_icon( $item['btn_icon'], [ 'aria-hidden' => 'true' ] ); ?>
                                    <?php Icons_Manager::render_icon( $item['btn_icon'], [ 'aria-hidden' => 'true' ] ); ?>
                                </span>
                        </a>
                    <?php } else { ?>
                        <a <?php $this->print_render_attribute_string( $btn_id ); ?> class="helo--btn">
                            <?php echo esc_html( $item['btn_text'] ); ?>
                            <span class="icon"><?php Icons_Manager::render_icon( $item['btn_icon'], [ 'aria-hidden' => 'true' ] ); ?> </span>
                        </a>
                    <?php } ?>
                </div>
            </div>
            <?php if ( $item['image']['url'] ) { ?>
                <?php $img_alt = Control_Media::get_image_alt( $item['image'] ); ?>

                <div class="thumb">
                    <img src="<?php echo $item['image']['url']; ?>" alt="<?php echo esc_attr( $img_alt ); ?>">
                </div>
            <?php } ?>
        </div>
		<?php
	}

}
