<?php

namespace HeloEssentialApp\Widgets;

use Elementor\Control_Media;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Image_Size;
use Elementor\Group_Control_Text_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Icons_Manager;
use Elementor\Plugin;
use Elementor\Repeater;
use Elementor\Utils;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Testimonial
 *
 * Elementor widget for testimonial.
 *
 * @since 1.0.0
 */
class Helo_Image_Accordion extends Widget_Base {

	/**
	 * Retrieve the widget name.
	 *
	 * @return string Widget name.
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function get_name() {
		return 'helo--image-accordion';
	}

	/**
	 * Retrieve the widget title.
	 *
	 * @return string Widget title.
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function get_title() {
		return esc_html__( 'Helo Image Accordion', 'helo-essential' );
	}

	/**
	 * Retrieve the widget icon.
	 *
	 * @return string Widget icon.
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function get_icon() {
		return 'wcf eicon-testimonial';
	}

	/**
	 * Retrieve the list of categories the widget belongs to.
	 *
	 * Used to determine where to display the widget in the editor.
	 *
	 * Note that currently Elementor supports only one category.
	 * When multiple categories passed, Elementor uses the first one.
	 *
	 * @return array Widget categories.
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function get_categories() {
		return [ 'weal-coder-addon' ];
	}

	public function get_style_depends() {
		wp_register_style( 'helo-image-accordion', HELO_ESSENTIAL_ASSETS_URL . 'css/builder-image-accordion.css' );
		return [ 'helo-image-accordion' ];
	}

	public function get_script_depends() {
		wp_register_script( 'helo-image-accordion', HELO_ESSENTIAL_ASSETS_URL . '/js/widgets/builder-image-accordion.js', [ 'jquery' ], false, true );
		return [ 'swiper', 'helo-image-accordion' ];
	}

	/**
	 * Register the widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function register_controls() {

		$this->start_controls_section(
			'section_content',
			[
				'label' => esc_html__( 'Image Accordion', 'helo-essential' ),
			]
		);

        $this->add_control(
			'name',
			[
				'label' => esc_html__( 'Name', 'helo-essential' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Name ', 'helo-essential' ),
			]
		);

        $this->add_control(
			'value',
			[
				'label' => esc_html__( 'Value', 'helo-essential' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Value ', 'helo-essential' ),
			]
		);

        $this->add_control(
			'type',
			[
				'label' => esc_html__( 'Type', 'helo-essential' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Type ', 'helo-essential' ),
			]
		);

		$repeater = new Repeater();

		$repeater->add_control(
			'name',
			[
				'label' => esc_html__( 'Name', 'helo-essential' ),
				'type' => Controls_Manager::TEXT,
				'default' => esc_html__( 'Federa Chair ', 'helo-essential' ),
			]
		);

        $repeater->add_control(
            'value',
            [
                'label' => esc_html__( 'Value', 'helo-essential' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Wood, Carpet ', 'helo-essential' ),
            ]
        );

        $repeater->add_control(
            'type',
            [
                'label' => esc_html__( 'Type', 'helo-essential' ),
                'type' => Controls_Manager::TEXT,
                'default' => esc_html__( 'Furniture ', 'helo-essential' ),
            ]
        );

        $repeater->add_control(
            'gallery',
            [
                'label' => esc_html__( 'Add Images', 'helo-essential' ),
                'type' => Controls_Manager::GALLERY,
                'show_label' => false,
                'default' => [],
            ]
        );

		$this->add_control(
			'image-accordion',
			[
				'label'   => esc_html__( 'Accordions', 'helo-essential' ),
				'type'    => Controls_Manager::REPEATER,
                'separator' => 'before',
				'fields'  => $repeater->get_controls(),
				'default' => [ [] ],
			]
		);

		$this->add_control(
			'selected_icon',
			[
				'label'       => esc_html__( 'Icon', 'helo-essential' ),
				'type'        => Controls_Manager::ICONS,
				'separator'   => 'before',
				'default'     => [
					'value'   => 'fas fa-plus',
					'library' => 'fa-solid',
				],
				'recommended' => [
					'fa-solid'   => [
						'chevron-down',
						'angle-down',
						'angle-double-down',
						'caret-down',
						'caret-square-down',
					],
					'fa-regular' => [
						'caret-square-down',
					],
				],
				'skin'        => 'inline',
				'label_block' => false,
			]
		);

		$this->add_control(
			'selected_active_icon',
			[
				'label'       => esc_html__( 'Active Icon', 'helo-essential' ),
				'type'        => Controls_Manager::ICONS,
				'default'     => [
					'value'   => 'fas fa-minus',
					'library' => 'fa-solid',
				],
				'recommended' => [
					'fa-solid'   => [
						'chevron-up',
						'angle-up',
						'angle-double-up',
						'caret-up',
						'caret-square-up',
					],
					'fa-regular' => [
						'caret-square-up',
					],
				],
				'skin'        => 'inline',
				'label_block' => false,
				'condition'   => [
					'selected_icon[value]!' => '',
				],
			]
		);

		$this->end_controls_section();

		//layout style
		$this->register_heading_style_controls();

		$this->register_body_style_controls();

		$this->start_controls_section(
			'section_toggle_style_icon',
			[
				'label' => esc_html__( 'Icon', 'helo-essential' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_responsive_control(
			'icon_size',
			[
				'label'      => esc_html__( 'Size', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'selectors'  => [
					'{{WRAPPER}} .accordion-icon' => 'font-size: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'icon_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .accordion-icon i'   => 'color: {{VALUE}};',
					'{{WRAPPER}} .accordion-icon svg' => 'fill: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'icon_active_color',
			[
				'label'     => esc_html__( 'Active Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .accordion-icon.accordion-icon-opened i'   => 'color: {{VALUE}};',
					'{{WRAPPER}} .accordion-icon.accordion-icon-opened svg' => 'fill: {{VALUE}};',
				],
			]
		);

		$this->end_controls_section();
	}

	protected function register_heading_style_controls() {

		$this->start_controls_section(
			'section_header_style',
			[
				'label' => esc_html__( 'Header', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'heading_color',
			[
				'label'     => esc_html__( 'Text Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .heading-title' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'heading_typography',
				'selector' => '{{WRAPPER}} .heading-title',
			]
		);

		$this->add_responsive_control(
			'header_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .header' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();
	}

	protected function register_body_style_controls() {
		$this->start_controls_section(
			'section_body_style',
			[
				'label' => esc_html__( 'Item', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'item_color',
			[
				'label'     => esc_html__( 'Text Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .title' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'item_typography',
				'selector' => '{{WRAPPER}} .title',
			]
		);

		$this->add_responsive_control(
			'item_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .heading' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'item_border_color',
			[
				'label'     => esc_html__( 'Border Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .heading'             => 'border-color: {{VALUE}}',
					'{{WRAPPER}} .gallary-image-inner' => 'border-color: {{VALUE}}',
				],
			]
		);

		$this->add_responsive_control(
			'item_image_padding',
			[
				'label'      => esc_html__( 'Image Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .gallary-image-inner' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();
	}

	/**
	 * Render the widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function render() {
		$settings = $this->get_settings_for_display();
        ?>
        <div class="helo-image-accordion">
            <div class="header">
                <?php if (!empty($settings['name'])) { ?>
                    <div class="heading-title"><?php echo esc_html($settings['name']) ?></div>
                <?php } ?>

	            <?php if (!empty($settings['value'])) { ?>
                    <div class="heading-title"><?php echo esc_html($settings['value']) ?></div>
	            <?php } ?>

	            <?php if (!empty($settings['type'])) { ?>
                    <div class="heading-title"><?php echo esc_html($settings['type']) ?></div>
	            <?php } ?>
            </div>
            <div class="body">
                <?php
                foreach ($settings['image-accordion'] as $item) {
                    ?>
                    <div class="accordion-item">
                        <div class="heading">
                            <?php if (!empty($settings['name'])) { ?>
                                <div class="title">
                                    <?php echo esc_html($item['name']); ?>
                                </div>
                            <?php } ?>

                            <?php if (!empty($settings['value'])) { ?>
                                <div class="title">
                                    <?php echo esc_html($item['value']); ?>
                                </div>
                            <?php } ?>

                            <?php if (!empty($settings['type'])) { ?>
                                <div class="title">
                                    <?php echo esc_html($item['type']); ?>
                                </div>
                            <?php } ?>

                            <span class="accordion-icon-closed accordion-icon">
		                        <?php Icons_Manager::render_icon( $settings['selected_icon'], [ 'aria-hidden' => 'true' ] ); ?>
                            </span>
                            <span class="accordion-icon-opened accordion-icon">
		                        <?php Icons_Manager::render_icon( $settings['selected_active_icon'], [ 'aria-hidden' => 'true' ] ); ?>
                            </span>
                        </div>
                        <div class="gallary-image">
                            <div class="gallary-image-inner">
	                            <?php foreach ( $item['gallery'] as $image ) { ?>
                                    <div class='image-item'>
                                        <img src="<?php echo esc_url( $image['url'] ); ?>"
                                             alt="<?php echo esc_attr( get_post_meta( $image['id'], '_wp_attachment_image_alt', true ) ); ?>">
                                    </div>
	                            <?php } ?>
                            </div>
                        </div>
                    </div>
                <?php } ?>
            </div>
        </div>
        <?php
	}

}
