<?php

namespace HeloEssentialApp\Widgets;

use Elementor\Controls_Manager;
use Elementor\Control_Media;
use Elementor\Group_Control_Background;
use Elementor\Utils;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Typography;
use Elementor\Plugin;
use Elementor\Widget_Base;
use Elementor\Icons_Manager;
use WP_Query;

if ( ! defined( 'ABSPATH' ) ) {
	exit;   // Exit if accessed directly.
}

class Helo_Posts extends Widget_Base {

	public function get_name() {
		return 'wcf--helo-posts';
	}

	public function get_title() {
		return wcf_elementor_widget_concat_prefix( 'Helo Posts' );
	}

	public function get_icon() {
		return 'wcf eicon-post-list';
	}

	public function get_categories() {
		return [ 'weal-coder-addon' ];
	}

	// Content Control
	public function register_content_controls() {
		$this->start_controls_section(
			'helo_sec_post_layout',
			[
				'label' => __( 'Layout', 'helo-essential' ),
			]
		);

		$this->add_control(
			'helo_rm_text',
			[
				'label'       => esc_html__( 'Read More Text', 'helo-essential' ),
				'type'        => Controls_Manager::TEXT,
				'default'     => esc_html__( 'Read More', 'helo-essential' ),
				'placeholder' => esc_html__( 'Type your text here', 'helo-essential' ),
			]
		);

		$this->add_control(
			'helo_rm_icon',
			[
				'label'       => esc_html__( 'Read More Icon', 'helo-essential' ),
				'type'        => Controls_Manager::ICONS,
				'skin'        => 'inline',
				'label_block' => false,
				'default'     => [
					'value'   => 'fas fa-arrow-right',
					'library' => 'fa-solid',
				],
			]
		);

		$this->add_control(
			'helo_date_text',
			[
				'label'       => esc_html__( 'Date Text', 'helo-essential' ),
				'type'        => Controls_Manager::TEXT,
				'default'     => esc_html__( 'Written by', 'helo-essential' ),
				'placeholder' => esc_html__( 'Type your text here', 'helo-essential' ),
			]
		);

		$this->add_control(
			'helo_link_icon',
			[
				'label'       => esc_html__( 'Link Icon', 'helo-essential' ),
				'type'        => Controls_Manager::ICONS,
				'skin'        => 'inline',
				'label_block' => false,
				'default'     => [
					'value'   => 'fas fa-arrow-right',
					'library' => 'fa-solid',
				],
			]
		);

		$this->add_control(
			'helo_show_title',
			[
				'label'     => esc_html__( 'Show Title', 'helo-essential' ),
				'type'      => Controls_Manager::SWITCHER,
				'default'   => 'yes',
				'label_off' => esc_html__( 'Off', 'helo-essential' ),
				'label_on'  => esc_html__( 'On', 'helo-essential' ),
			]
		);

		$this->add_control(
			'helo_show_excerpt',
			[
				'label'     => esc_html__( 'Show Excerpt', 'helo-essential' ),
				'type'      => Controls_Manager::SWITCHER,
				'default'   => 'yes',
				'label_off' => esc_html__( 'Off', 'helo-essential' ),
				'label_on'  => esc_html__( 'On', 'helo-essential' ),
			]
		);

		$this->add_control(
			'helo_show_thumb',
			[
				'label'     => esc_html__( 'Show Thumbnail', 'helo-essential' ),
				'type'      => Controls_Manager::SWITCHER,
				'default'   => 'no',
				'label_off' => esc_html__( 'Off', 'helo-essential' ),
				'label_on'  => esc_html__( 'On', 'helo-essential' ),
			]
		);

		$this->add_control(
			'helo_show_author',
			[
				'label'     => esc_html__( 'Show Author', 'helo-essential' ),
				'type'      => Controls_Manager::SWITCHER,
				'default'   => 'yes',
				'label_off' => esc_html__( 'Off', 'helo-essential' ),
				'label_on'  => esc_html__( 'On', 'helo-essential' ),
			]
		);

		$this->add_control(
			'helo_show_meta',
			[
				'label'     => esc_html__( 'Show Meta', 'helo-essential' ),
				'type'      => Controls_Manager::SWITCHER,
				'default'   => 'yes',
				'label_off' => esc_html__( 'Off', 'helo-essential' ),
				'label_on'  => esc_html__( 'On', 'helo-essential' ),
			]
		);

		$this->add_control(
			'helo_show_date',
			[
				'label'     => esc_html__( 'Show Date', 'helo-essential' ),
				'type'      => Controls_Manager::SWITCHER,
				'default'   => 'yes',
				'label_off' => esc_html__( 'Off', 'helo-essential' ),
				'label_on'  => esc_html__( 'On', 'helo-essential' ),
				'condition' => [
					'helo_show_meta' => 'yes',
				],
			]
		);

		$this->add_control(
			'helo_show_category',
			[
				'label'     => esc_html__( 'Show Category', 'helo-essential' ),
				'type'      => Controls_Manager::SWITCHER,
				'default'   => 'yes',
				'label_off' => esc_html__( 'Off', 'helo-essential' ),
				'label_on'  => esc_html__( 'On', 'helo-essential' ),
				'condition' => [
					'helo_show_meta' => 'yes',
				],
			]
		);

		$this->add_control(
			'helo_show_comment',
			[
				'label'     => esc_html__( 'Show Comment', 'helo-essential' ),
				'type'      => Controls_Manager::SWITCHER,
				'default'   => 'yes',
				'label_off' => esc_html__( 'Off', 'helo-essential' ),
				'label_on'  => esc_html__( 'On', 'helo-essential' ),
				'condition' => [
					'helo_show_meta' => 'yes',
				],
			]
		);

		$this->add_control(
			'helo_show_rm',
			[
				'label'     => esc_html__( 'Show Read More', 'helo-essential' ),
				'type'      => Controls_Manager::SWITCHER,
				'default'   => 'yes',
				'label_off' => esc_html__( 'Off', 'helo-essential' ),
				'label_on'  => esc_html__( 'On', 'helo-essential' ),
			]
		);

		$this->add_control(
			'helo_show_link',
			[
				'label'     => esc_html__( 'Show Link Icon', 'helo-essential' ),
				'type'      => Controls_Manager::SWITCHER,
				'default'   => 'yes',
				'label_off' => esc_html__( 'Off', 'helo-essential' ),
				'label_on'  => esc_html__( 'On', 'helo-essential' ),
			]
		);

		$this->end_controls_section();


		$this->start_controls_section(
			'helo_sec_post_query',
			[
				'label' => __( 'Query', 'helo-essential' ),
			]
		);

		$this->add_control(
			'helo_post_order_by',
			[
				'label'   => esc_html__( 'Order By', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'default' => 'date',
				'options' => [
					'date'          => esc_html__( 'Date', 'helo-essential' ),
					'title'         => esc_html__( 'Title', 'helo-essential' ),
					'menu_order'    => esc_html__( 'Menu Order', 'helo-essential' ),
					'modified'      => esc_html__( 'Last Modified', 'helo-essential' ),
					'comment_count' => esc_html__( 'Comment Count', 'helo-essential' ),
					'rand'          => esc_html__( 'Random', 'helo-essential' ),
				],
			]
		);

		$this->add_control(
			'helo_post_order',
			[
				'label'   => esc_html__( 'Order', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'default' => 'desc',
				'options' => [
					'asc'  => esc_html__( 'ASC', 'helo-essential' ),
					'desc' => esc_html__( 'DESC', 'helo-essential' ),
				],
			]
		);

		$this->add_control(
			'helo_posts_ppage',
			[
				'label'   => esc_html__( 'Posts Per Page', 'helo-essential' ),
				'type'    => Controls_Manager::NUMBER,
				'default' => 6,
			]
		);


		$this->end_controls_section();
	}

	// Style Control
	public function register_style_controls() {

		$this->start_controls_section(
			'helo_style_layout',
			[
				'label' => esc_html__( 'Layout', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_responsive_control(
			'blog_item_gap',
			[
				'label'      => esc_html__( 'Gap', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 200,
					],
					'%'  => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .default-blog__style-2' => 'gap: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name'     => 'blog_item_border',
				'selector' => '{{WRAPPER}} .default-blog__style-2',
			]
		);

		$this->add_responsive_control(
			'blog_item_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .default-blog__style-2' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();

		//  Title
		$this->start_controls_section(
			'helo_style_title',
			[
				'label'     => esc_html__( 'Title', 'helo-essential' ),
				'tab'       => Controls_Manager::TAB_STYLE,
				'condition' => [
					'helo_show_title' => 'yes',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'helo_blog_title_typo',
				'selector' => '{{WRAPPER}} .default-blog__style-2 .title',
			]
		);

		$this->start_controls_tabs(
			'helo_bt_style_tabs'
		);

		$this->start_controls_tab(
			'helo_bt_normal_tab',
			[
				'label' => esc_html__( 'Normal', 'helo-essential' ),
			]
		);

		$this->add_control(
			'helo_blog_title_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .title' => 'color: {{VALUE}}',
				],
			]
		);

		$this->end_controls_tab();

		$this->start_controls_tab(
			'helo_bt_hover_tab',
			[
				'label' => esc_html__( 'Hover', 'helo-essential' ),
			]
		);

		$this->add_control(
			'helo_bt_h_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .title a:hover' => 'color: {{VALUE}}',
				],
			]
		);

		$this->end_controls_tab();

		$this->end_controls_tabs();

		$this->add_responsive_control(
			'helo_bt_margin',
			[
				'label'      => esc_html__( 'Spacing', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .title' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();


		// Excerpt
		$this->start_controls_section(
			'helo_style_excerpt',
			[
				'label'     => esc_html__( 'Excerpt', 'helo-essential' ),
				'tab'       => Controls_Manager::TAB_STYLE,
				'condition' => [
					'helo_show_excerpt' => 'yes',
				],
			]
		);

		$this->add_control(
			'helo_bex_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .cf_text p' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'helo_bex_typo',
				'selector' => '{{WRAPPER}} .cf_text p',
			]
		);

		$this->add_responsive_control(
			'helo_bex_margin',
			[
				'label'      => esc_html__( 'Spacing', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .cf_text' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();


		// Thumbnail
		$this->start_controls_section(
			'helo_style_thumb',
			[
				'label'     => esc_html__( 'Thumbnail', 'helo-essential' ),
				'tab'       => Controls_Manager::TAB_STYLE,
				'condition' => [
					'helo_show_thumb' => 'yes',
				],
			]
		);

		$this->add_responsive_control(
			'helo_bthumb_width',
			[
				'label'      => esc_html__( 'Width', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 1000,
						'step' => 5,
					],
					'%'  => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .thumb img' => 'width: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'helo_bthumb_margin',
			[
				'label'      => esc_html__( 'Spacing', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .thumb img' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();


		// Author
		$this->start_controls_section(
			'helo_style_author',
			[
				'label'     => esc_html__( 'Author', 'helo-essential' ),
				'tab'       => Controls_Manager::TAB_STYLE,
				'condition' => [
					'helo_show_author' => 'yes',
				],
			]
		);

		// Image
		$this->add_control(
			'blog_author_img_heading',
			[
				'label'     => esc_html__( 'Image', 'helo-essential' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_responsive_control(
			'blog_author_img_size',
			[
				'label'      => esc_html__( 'Size', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
						'step' => 1,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .author-img img' => 'width: {{SIZE}}{{UNIT}}; height: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'blog_author_img_radius',
			[
				'label'      => esc_html__( 'Border Radius', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 100,
					],
					'%'  => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .author-img img' => 'border-radius: {{SIZE}}{{UNIT}};',
				],
			]
		);

		// Text
		$this->add_control(
			'blog_author_text_heading',
			[
				'label'     => esc_html__( 'Text', 'helo-essential' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'blog_author_text_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .author h4' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'blog_author_text_typo',
				'selector' => '{{WRAPPER}} .author h4',
			]
		);

		$this->add_responsive_control(
			'blog_author_text_margin',
			[
				'label'      => esc_html__( 'Spacing', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .author h4' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'blog_author_name_heading',
			[
				'label'     => esc_html__( 'Name', 'helo-essential' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'blog_author_name_typo',
				'selector' => '{{WRAPPER}} .author-bio a',
			]
		);

		$this->start_controls_tabs(
			'blog_author_name_tabs'
		);

		$this->start_controls_tab(
			'blog_author_name_normal',
			[
				'label' => esc_html__( 'Normal', 'helo-essential' ),
			]
		);

		$this->add_control(
			'blog_author_name_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .author-bio a' => 'color: {{VALUE}}',
				],
			]
		);

		$this->end_controls_tab();

		$this->start_controls_tab(
			'blog_author_name_hover',
			[
				'label' => esc_html__( 'Hover', 'helo-essential' ),
			]
		);

		$this->add_control(
			'blog_author_name_h_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .author-bio a:hover' => 'color: {{VALUE}}',
				],
			]
		);

		$this->end_controls_tab();

		$this->end_controls_tabs();

		$this->end_controls_section();

		// Meta
		$this->start_controls_section(
			'helo_style_meta',
			[
				'label'     => esc_html__( 'Meta', 'helo-essential' ),
				'tab'       => Controls_Manager::TAB_STYLE,
				'condition' => [
					'helo_show_meta' => 'yes',
				],
			]
		);

		$this->add_control(
			'blog_meta_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .default-blog__item-meta'      => 'color: {{VALUE}}',
					'{{WRAPPER}} .default-blog__item-meta span' => 'background-color: {{VALUE}}',
				],
			]
		);

		$this->add_control(
			'blog_meta_cat_h_color',
			[
				'label'     => esc_html__( 'Category Hover Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .jpost-cat:hover' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'blog_meta_typo',
				'selector' => '{{WRAPPER}} .default-blog__item-meta',
			]
		);

		$this->add_responsive_control(
			'blog_meta_margin',
			[
				'label'      => esc_html__( 'Spacing', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .default-blog__item-meta' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();

		// Category
		$this->start_controls_section(
			'helo_style_category',
			[
				'label'     => esc_html__( 'Category', 'helo-essential' ),
				'tab'       => Controls_Manager::TAB_STYLE,
				'condition' => [
					'helo_show_category' => 'yes',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'blog_cat_typo',
				'selector' => '{{WRAPPER}} .jpost-cat',
			]
		);

		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name'     => 'cat_bg_color',
				'types'    => [ 'classic', 'gradient' ],
				'exclude'  => [ 'image' ],
				'selector' => '{{WRAPPER}} .jpost-cat',
			]
		);

		$this->add_responsive_control(
			'cat_b_radius',
			[
				'label'      => esc_html__( 'Border Radius', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .jpost-cat' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'blog_cat_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .jpost-cat' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->start_controls_tabs(
			'cat_style_tabs'
		);

		// Normal Tab
		$this->start_controls_tab(
			'cat_normal_tab',
			[
				'label' => esc_html__( 'Normal', 'helo-essential' ),
			]
		);

		$this->add_control(
			'blog_cat_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .jpost-cat' => 'color: {{VALUE}}',
				],
			]
		);

		$this->end_controls_tab();

		// Hover Tab
		$this->start_controls_tab(
			'cat_hover_tab',
			[
				'label' => esc_html__( 'Hover', 'helo-essential' ),
			]
		);

		$this->add_control(
			'blog_cat_h_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .jpost-cat:hover' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name'     => 'cat_h_bg_color',
				'types'    => [ 'classic', 'gradient' ],
				'exclude'  => [ 'image' ],
				'selector' => '{{WRAPPER}} .jpost-cat:hover',
			]
		);

		$this->end_controls_tab();

		$this->end_controls_tabs();

		$this->end_controls_section();

		// Read More
		$this->start_controls_section(
			'helo_style_rm',
			[
				'label'     => esc_html__( 'Read More', 'helo-essential' ),
				'tab'       => Controls_Manager::TAB_STYLE,
				'condition' => [
					'helo_show_rm' => 'yes',
				],
			]
		);

		$this->add_control(
			'rm_text_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .read-more' => 'color: {{VALUE}}; fill: {{VALUE}}',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'rm_text_typo',
				'selector' => '{{WRAPPER}} .read-more',
			]
		);

		$this->add_control(
			'rm_text_h_color',
			[
				'label'     => esc_html__( 'Hover Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .read-more:hover'  => 'color: {{VALUE}}; fill: {{VALUE}}',
					'{{WRAPPER}} .read-more::after' => 'background-color: {{VALUE}};',
				],
			]
		);


		$this->end_controls_section();

		// Link Icon
		$this->start_controls_section(
			'helo_style_icon',
			[
				'label'     => esc_html__( 'Link Icon', 'helo-essential' ),
				'tab'       => Controls_Manager::TAB_STYLE,
				'condition' => [
					'helo_show_link' => 'yes',
				],
			]
		);

		$this->add_control(
			'blog_icon_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .cf_btn .link' => 'color: {{VALUE}}; fill: {{VALUE}}',
				],
			]
		);

		$this->add_responsive_control(
			'blog_icon_size',
			[
				'label'      => esc_html__( 'Size', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .cf_btn .link' => 'font-size: {{SIZE}}{{UNIT}}; min-width: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();
	}

	protected function register_controls() {
		$this->register_content_controls();
		$this->register_style_controls();
		$this->register_pagination_section_controls();
	}

	public function register_pagination_section_controls() {
		$this->start_controls_section(
			'section_pagination',
			[
				'label' => esc_html__( 'Pagination', 'helo-essential' ),
			]
		);

		$this->add_control(
			'pagination_type',
			[
				'label'   => esc_html__( 'Pagination', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'default' => '',
				'options' => [
					''                      => esc_html__( 'None', 'helo-essential' ),
					'numbers_and_prev_next' => esc_html__( 'Numbers', 'helo-essential' ) . ' + ' . esc_html__( 'Previous/Next', 'helo-essential' ),
				],
			]
		);

		$this->add_control(
			'pagination_page_limit',
			[
				'label'     => esc_html__( 'Page Limit', 'helo-essential' ),
				'default'   => '5',
				'condition' => [
					'pagination_type' => 'numbers_and_prev_next',
				],
			]
		);

		$this->add_control(
			'pagination_numbers_shorten',
			[
				'label'     => esc_html__( 'Shorten', 'helo-essential' ),
				'type'      => Controls_Manager::SWITCHER,
				'default'   => '',
				'condition' => [
					'pagination_type' => 'numbers_and_prev_next',
				],
			]
		);

		$this->add_control(
			'navigation_previous_icon',
			[
				'label'            => esc_html__( 'Previous Arrow Icon', 'helo-essential' ),
				'type'             => Controls_Manager::ICONS,
				'fa4compatibility' => 'icon',
				'skin'             => 'inline',
				'label_block'      => false,
				'skin_settings'    => [
					'inline' => [
						'none' => [
							'label' => 'Default',
							'icon'  => 'eicon-chevron-left',
						],
						'icon' => [
							'icon' => 'eicon-star',
						],
					],
				],
				'recommended'      => [
					'fa-regular' => [
						'arrow-alt-circle-left',
						'caret-square-left',
					],
					'fa-solid'   => [
						'angle-double-left',
						'angle-left',
						'arrow-alt-circle-left',
						'arrow-circle-left',
						'arrow-left',
						'caret-left',
						'caret-square-left',
						'chevron-circle-left',
						'chevron-left',
						'long-arrow-alt-left',
					],
				],
				'condition'        => [
					'pagination_type' => [
						'numbers',
						'numbers_and_prev_next',
					],
				],
			]
		);

		$this->add_control(
			'navigation_next_icon',
			[
				'label'            => esc_html__( 'Next Arrow Icon', 'helo-essential' ),
				'type'             => Controls_Manager::ICONS,
				'fa4compatibility' => 'icon',
				'skin'             => 'inline',
				'label_block'      => false,
				'skin_settings'    => [
					'inline' => [
						'none' => [
							'label' => 'Default',
							'icon'  => 'eicon-chevron-right',
						],
						'icon' => [
							'icon' => 'eicon-star',
						],
					],
				],
				'recommended'      => [
					'fa-regular' => [
						'arrow-alt-circle-right',
						'caret-square-right',
					],
					'fa-solid'   => [
						'angle-double-right',
						'angle-right',
						'arrow-alt-circle-right',
						'arrow-circle-right',
						'arrow-right',
						'caret-right',
						'caret-square-right',
						'chevron-circle-right',
						'chevron-right',
						'long-arrow-alt-right',
					],
				],
				'condition'        => [
					'pagination_type' => [
						'numbers',
						'numbers_and_prev_next',
					],
				],
			]
		);

		$this->add_control(
			'pagination_align',
			[
				'label'     => esc_html__( 'Alignment', 'helo-essential' ),
				'type'      => Controls_Manager::CHOOSE,
				'options'   => [
					'left'   => [
						'title' => esc_html__( 'Left', 'helo-essential' ),
						'icon'  => 'eicon-text-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'helo-essential' ),
						'icon'  => 'eicon-text-align-center',
					],
					'right'  => [
						'title' => esc_html__( 'Right', 'helo-essential' ),
						'icon'  => 'eicon-text-align-right',
					],
				],
				'default'   => 'center',
				'selectors' => [
					'{{WRAPPER}} .post-pagination' => 'justify-content: {{VALUE}};',
				],
				'condition' => [
					'pagination_type!' => '',
				],
			]
		);

		$this->add_responsive_control(
			'pagination_spacing_top',
			[
				'label'      => esc_html__( 'Spacing', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', 'em', 'rem', 'custom' ],
				'default'    => [
					'size' => 70,
				],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .post-pagination' => 'margin-top: {{SIZE}}{{UNIT}}',
				],
				'condition'  => [
					'pagination_type!' => '',
				],
			]
		);

		$this->end_controls_section();

		// Pagination style controls for prev/next and numbers pagination.
		$this->start_controls_section(
			'section_pagination_style',
			[
				'label'     => esc_html__( 'Pagination', 'helo-essential' ),
				'tab'       => Controls_Manager::TAB_STYLE,
				'condition' => [
					'pagination_type' => 'numbers_and_prev_next',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'pagination_typography',
				'selector' => '{{WRAPPER}} .post-pagination .page-numbers',
			]
		);

		$this->add_control(
			'pagination_color_heading',
			[
				'label'     => esc_html__( 'Colors', 'helo-essential' ),
				'type'      => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->start_controls_tabs( 'pagination_colors' );

		$this->start_controls_tab(
			'pagination_color_normal',
			[
				'label' => esc_html__( 'Normal', 'helo-essential' ),
			]
		);

		$this->add_control(
			'pagination_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .post-pagination .page-numbers' => 'color: {{VALUE}}; fill: {{VALUE}};',
				],
			]
		);

		$this->end_controls_tab();

		$this->start_controls_tab(
			'pagination_color_hover',
			[
				'label' => esc_html__( 'Hover', 'helo-essential' ),
			]
		);

		$this->add_control(
			'pagination_hover_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .post-pagination .page-numbers:not(.dots):hover, {{WRAPPER}} .post-pagination .page-numbers.current' => 'color: {{VALUE}}; fill: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'pagination_hover_bg_color',
			[
				'label'     => esc_html__( 'Background Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .post-pagination .page-numbers.current, {{WRAPPER}} .post-pagination .page-numbers:not(.prev, .next, .dots):hover' => 'background-color: {{VALUE}};',
				],
				'condition' => [
					'pagination_list' => '2',
				],
			]
		);

		$this->end_controls_tab();

		$this->end_controls_tabs();

		$this->add_responsive_control(
			'pagination_spacing',
			[
				'label'      => esc_html__( 'Space Between', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', 'em', 'rem', 'custom' ],
				'separator'  => 'before',
				'default'    => [
					'size' => 10,
				],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .post-pagination' => 'gap: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();
	}

	public function get_current_page() {
		if ( '' === $this->get_settings_for_display( 'pagination_type' ) ) {
			return 1;
		}

		return max( 1, get_query_var( 'paged' ), get_query_var( 'page' ) );
	}

	protected function render() {
		$settings = $this->get_settings_for_display();

		$helo_query_args = [
			'post_type'      => 'post',
			'posts_per_page' => $settings['helo_posts_ppage'],
			'order'          => $settings['helo_post_order'],
			'orderby'        => $settings['helo_post_order_by'],
		];

		$helo_query = new WP_Query( $helo_query_args );

		?>
        <div class="helo-post__list">
			<?php
			while ( $helo_query->have_posts() ) {
				$helo_query->the_post();
				?>

                <article <?php post_class( 'default-blog__style-2' ); ?>>
					<?php
					if ( 'yes' === $settings['helo_show_author'] ) {
						?>
                        <div class="author">
                            <div class="author-img">
								<?php echo wp_kses_post( get_avatar( get_the_author_meta( 'ID' ), 60 ) ); ?>
                            </div>
                            <div class="author-bio">
                                <h4><?php if ( ! empty( $settings['helo_date_text'] ) ) {
										echo esc_html( $settings['helo_date_text'] );
									} ?></h4>
                                <a href="<?php echo esc_url( get_author_posts_url( get_the_author_meta( 'ID' ) ) ); ?>"><?php the_author(); ?></a>
                            </div>
                        </div>
						<?php
					}
					?>

                    <div class="content">
						<?php
						if ( 'yes' === $settings['helo_show_thumb'] ) {
							?>
                            <div class="thumb">
								<?php if ( has_post_thumbnail() ): ?>
                                    <a href="<?php the_permalink(); ?>"><img
                                                src="<?php echo get_the_post_thumbnail_url(); ?>"
                                                alt="<?php the_title_attribute(); ?>"></a>
								<?php endif; ?>
                            </div>
							<?php
						}
						?>

						<?php
						$post_meta = [];
						$category  = helo_get_random_category();

						if ( 'yes' === $settings['helo_show_category'] ) {
							$post_meta[] = $category;
						}

						if ( 'yes' === $settings['helo_show_date'] ) {
							$post_meta[] = get_the_date( get_option( 'date_format' ) );
						}

						if ( 'yes' === $settings['helo_show_comment'] ) {
							$comments_number = get_comments_number();
							$post_meta[]     = $comments_number > 1 ? sprintf( esc_html__( '%s comments', 'helo' ), $comments_number ) : sprintf( esc_html__( '%s comment', 'helo' ), $comments_number );
						}
						?>

						<?php if ( 'yes' === $settings['helo_show_meta'] ) { ?>
                            <i class="default-blog__item-meta">
								<?php
								echo helo_kses( implode( '<span></span>', $post_meta ) );
								?>
                            </i>
						<?php } ?>

						<?php
						if ( 'yes' === $settings['helo_show_title'] ) {
							?>
                            <h2 class="title"><a href="<?php the_permalink(); ?>"><?php the_title(); ?></a></h2>
							<?php
						}

						if ( 'yes' === $settings['helo_show_excerpt'] ) {
							?>
                            <div class="cf_text">
								<?php helo_excerpt( 40, null ); ?>
                            </div>
							<?php
						}

						if ( 'yes' === $settings['helo_show_rm'] ) {
							?>
                            <div class="read_more_btn">
                                <a href="<?php the_permalink(); ?>" class="read-more">
									<?php
									echo esc_html( $settings['helo_rm_text'] );
									Icons_Manager::render_icon( $settings['helo_rm_icon'], [ 'aria-hidden' => 'true' ] );
									?>
                                </a>
                            </div>
							<?php
						}
						?>
                    </div>

					<?php
					if ( 'yes' === $settings['helo_show_link'] ) {
						?>
                        <div class="cf_btn">
                            <a href="<?php the_permalink(); ?>" class="link">
								<?php Icons_Manager::render_icon( $settings['helo_rm_icon'], [ 'aria-hidden' => 'true' ] ); ?>
                            </a>
                        </div>
						<?php
					}
					?>
                </article>
				<?php
			}
			?>
        </div>
		<?php
		$this->render_pagination( $helo_query );
		wp_reset_postdata();
	}

	protected function render_pagination( $query ) {

		$settings = $this->get_settings_for_display();

		if ( '' === $settings['pagination_type'] ) {
			return;
		}

		$page_limit = $query->max_num_pages;

		// Page limit control should not effect in load more mode.
		if ( '' !== $settings['pagination_page_limit'] && 'load_more' !== $settings['pagination_type'] ) {
			$page_limit = min( $settings['pagination_page_limit'], $page_limit );
		}

		if ( 2 > $page_limit ) {
			return;
		}

		//number and prev next
		if ( 'numbers_and_prev_next' === $settings['pagination_type'] ) {
			$paginate_args = [
				'current'            => $this->get_current_page(),
				'total'              => $page_limit,
				'prev_next'          => true,
				'prev_text'          => sprintf( '%1$s', $this->render_next_prev_button( 'prev' ) ),
				'next_text'          => sprintf( '%1$s', $this->render_next_prev_button( 'next' ) ),
				'show_all'           => 'yes' !== $settings['pagination_numbers_shorten'],
				'before_page_number' => '<span class="elementor-screen-only">' . esc_html__( 'Page', 'helo-essential' ) . '</span>',
			];

			//pagination class
			$this->add_render_attribute( 'pagination', 'class', 'post-pagination' );
			?>
            <nav <?php $this->print_render_attribute_string( 'pagination' ); ?>
                    aria-label="<?php esc_attr_e( 'Pagination', 'helo-essential' ); ?>" style="display: flex">
				<?php echo wp_kses_post( paginate_links( $paginate_args ) ); ?>
            </nav>
			<?php
		}
	}

	private function render_next_prev_button( $type ) {
		$direction     = 'next' === $type ? 'right' : 'left';
		$icon_settings = $this->get_settings( 'navigation_' . $type . '_icon' );

		if ( empty( $icon_settings['value'] ) ) {
			$icon_settings = [
				'library' => 'eicons',
				'value'   => 'eicon-chevron-' . $direction,
			];
		}

		if ( 'next' === $type ) {
			return esc_html( $type ) . ' ' . Icons_Manager::try_get_icon_html( $icon_settings, [ 'aria-hidden' => 'true' ] );
		} else {
			return Icons_Manager::try_get_icon_html( $icon_settings, [ 'aria-hidden' => 'true' ] ) . ' ' . esc_html( $type );
		}
	}
}
