<?php

namespace HeloEssentialApp\Widgets;

use Elementor\Controls_Manager;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Typography;
use Elementor\Icons_Manager;
use Elementor\Plugin;
use Elementor\Repeater;
use Elementor\Utils;
use Elementor\Widget_Base;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

/**
 * Elementor tabs widget.
 *
 * Elementor widget that displays vertical or horizontal tabs with different
 * pieces of content.
 *
 * @since 1.0.0
 */
class Helo_work_process extends Widget_Base {

	/**
	 * Get widget name.
	 *
	 * Retrieve tabs widget name.
	 *
	 * @return string Widget name.
	 * @since 1.0.0
	 * @access public
	 */
	public function get_name() {
		return 'helo--work-workprocess';
	}

	/**
	 * Get widget title.
	 *
	 * Retrieve tabs widget title.
	 *
	 * @return string Widget title.
	 * @since 1.0.0
	 * @access public
	 */
	public function get_title() {
		return esc_html__( 'Helo Work Process', 'helo-essential' );
	}

	/**
	 * Get widget icon.
	 *
	 * Retrieve tabs widget icon.
	 *
	 * @return string Widget icon.
	 * @since 1.0.0
	 * @access public
	 */
	public function get_icon() {
		return 'wcf eicon-tabs';
	}

	/**
	 * Retrieve the list of categories the widget belongs to.
	 *
	 * Used to determine where to display the widget in the editor.
	 *
	 * Note that currently Elementor supports only one category.
	 * When multiple categories passed, Elementor uses the first one.
	 *
	 * @return array Widget categories.
	 * @since 1.0.0
	 *
	 * @access public
	 */
	public function get_categories() {
		return [ 'weal-coder-addon' ];
	}

	// Style Dependency
	public function get_style_depends() {
		wp_register_style( 'helo-tabs', HELO_ESSENTIAL_ASSETS_URL . 'css/helo-tabs.css' );
		return [ 'helo-tabs' ];
	}

    // Script Dependency
	public function get_script_depends() {
		wp_register_script( 'helo-work-process', HELO_ESSENTIAL_ASSETS_URL . '/js/widgets/helo-workprocess.js', [ 'jquery' ], false, true );

		return [ 'helo-work-process' ];
	}

	/**
	 * Register tabs widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 3.1.0
	 * @access protected
	 */
	protected function register_controls() {
		$start = is_rtl() ? 'right' : 'left';
		$end   = is_rtl() ? 'left' : 'right';

		$this->start_controls_section(
			'section_tabs',
			[
				'label' => esc_html__( 'Tabs', 'helo-essential' ),
			]
		);

		$repeater = new Repeater();

		$repeater->add_control(
			'number',
			[
				'label'       => esc_html__( 'Number', 'helo-essential' ),
				'type'        => Controls_Manager::TEXT,
				'default'     => esc_html__( 'Number', 'helo-essential' ),
				'placeholder' => esc_html__( 'Number', 'helo-essential' ),
				'label_block' => true,
				'dynamic'     => [
					'active' => true,
				],
			]
		);

		$repeater->add_control(
			'tab_title',
			[
				'label'       => esc_html__( 'Title', 'helo-essential' ),
				'type'        => Controls_Manager::TEXT,
				'default'     => esc_html__( 'Tab Title', 'helo-essential' ),
				'placeholder' => esc_html__( 'Tab Title', 'helo-essential' ),
				'label_block' => true,
				'dynamic'     => [
					'active' => true,
				],
			]
		);

		$repeater->add_control(
			'tab_content',
			[
				'label'       => esc_html__( 'Discription', 'helo-essential' ),
				'default'     => esc_html__( 'Title Discription', 'helo-essential' ),
				'placeholder' => esc_html__( 'Title Discription', 'helo-essential' ),
				'type'        => Controls_Manager::TEXTAREA,
			]
		);

		$repeater->add_control(
			'content_image',
			[
				'label'   => esc_html__( 'Content Image', 'helo-essential' ),
				'type'    => Controls_Manager::MEDIA,
				'dynamic' => [
					'active' => true,
				],
				'default' => [
					'url' => Utils::get_placeholder_image_src(),
				],
			]
		);

		$repeater->add_control(
			'elementor_templates',
			[
				'label'       => esc_html__( 'Save Template', 'helo-essential' ),
				'type'        => Controls_Manager::SELECT2,
				'label_block' => false,
				'multiple'    => false,
				'options'     => wcf_addons_get_saved_template_list(),
				'condition'   => [
					'tabs_content_type' => 'template',
				],
			]
		);

		$this->add_control(
			'tabs',
			[
				'label'       => esc_html__( 'Tabs Items', 'helo-essential' ),
				'type'        => Controls_Manager::REPEATER,
				'fields'      => $repeater->get_controls(),
				'default'     => [
					[
						'tab_title'   => esc_html__( 'Tab Title 1', 'helo-essential' ),
						'tab_content' => esc_html__( 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.', 'helo-essential' ),
					],
					[
						'tab_title'   => esc_html__( 'Tab Title 2', 'helo-essential' ),
						'tab_content' => esc_html__( 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.', 'helo-essential' ),
					],
					[
						'tab_title'   => esc_html__( 'Tab Title 3', 'helo-essential' ),
						'tab_content' => esc_html__( 'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Ut elit tellus, luctus nec ullamcorper mattis, pulvinar dapibus leo.', 'helo-essential' ),
					],
				],
				'title_field' => '{{{ tab_title }}}',
			]
		);

		$this->add_control(
			'view',
			[
				'label'   => esc_html__( 'View', 'helo-essential' ),
				'type'    => Controls_Manager::HIDDEN,
				'default' => 'traditional',
			]
		);

		$this->add_responsive_control( 'tabs_direction',
			[
				'label'        => esc_html__( 'Wrapper Direction', 'helo-essential' ),
				'type'         => Controls_Manager::CHOOSE,
				'options'      => [
					'column'         => [
						'title' => esc_html__( 'Above', 'helo-essential' ),
						'icon'  => 'eicon-v-align-top',
					],
					'column-reverse' => [
						'title' => esc_html__( 'Below', 'helo-essential' ),
						'icon'  => 'eicon-v-align-bottom',
					],
					'row-reverse'    => [
						'title' => esc_html__( 'After', 'helo-essential' ),
						'icon'  => 'eicon-h-align-' . $end,
					],
					'row'            => [
						'title' => esc_html__( 'Before', 'helo-essential' ),
						'icon'  => 'eicon-h-align-' . $start,
					],
				],
				'separator'    => 'before',
				'selectors'    => [
					'{{WRAPPER}} .wcf--tabs' => 'flex-direction: {{VALUE}}',
				],
				'prefix_class' => 'wcf-tabs-direction-',
			]
		);
		$this->add_responsive_control( 'tab_direction',
			[
				'label'        => esc_html__( 'Tab Direction', 'helo-essential' ),
				'type'         => Controls_Manager::CHOOSE,
				'options'      => [
					'column'         => [
						'title' => esc_html__( 'Above', 'helo-essential' ),
						'icon'  => 'eicon-v-align-top',
					],
					'column-reverse' => [
						'title' => esc_html__( 'Below', 'helo-essential' ),
						'icon'  => 'eicon-v-align-bottom',
					],
					'row-reverse'    => [
						'title' => esc_html__( 'After', 'helo-essential' ),
						'icon'  => 'eicon-h-align-' . $end,
					],
					'row'            => [
						'title' => esc_html__( 'Before', 'helo-essential' ),
						'icon'  => 'eicon-h-align-' . $start,
					],
				],
				'separator'    => 'before',
				'selectors'    => [
					'{{WRAPPER}} .tabs-wrapper' => 'flex-direction: {{VALUE}}',
				],
				'prefix_class' => 'wcf-tabs-direction-',
			]
		);

		$this->add_control(
			'tabs_align',
			[
				'label'        => esc_html__( 'Alignment', 'helo-essential' ),
				'type'         => Controls_Manager::SELECT,
				'options'      => [
					''        => esc_html__( 'Start', 'helo-essential' ),
					'center'  => esc_html__( 'Center', 'helo-essential' ),
					'end'     => esc_html__( 'End', 'helo-essential' ),
					'stretch' => esc_html__( 'Stretch', 'helo-essential' ),
				],
				'selectors'    => [
					'{{WRAPPER}} .tabs-wrapper' => 'justify-content: {{VALUE}}',
				],
				'prefix_class' => 'tabs-alignment-',
			]
		);

		$this->end_controls_section();

		//settings
		$this->start_controls_section( 'section_tabs_setting', [
			'label' => esc_html__( 'Settings', 'helo-essential' ),
		] );

		$this->add_responsive_control(
			'tab_tittle_align',
			[
				'label'     => esc_html__( 'Title Alignment', 'helo-essential' ),
				'type'      => Controls_Manager::CHOOSE,
				'options'   => [
					'start'  => [
						'title' => esc_html__( 'Left', 'helo-essential' ),
						'icon'  => 'eicon-text-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'helo-essential' ),
						'icon'  => 'eicon-text-align-center',
					],
					'end'    => [
						'title' => esc_html__( 'Right', 'helo-essential' ),
						'icon'  => 'eicon-text-align-right',
					],
				],
				'selectors' => [
					'{{WRAPPER}} .tab-title' => 'text-align: {{VALUE}}; justify-content:{{VALUE}};',
				],
			]
		);

		$dropdown_options     = [
			'none' => esc_html__( 'None', 'helo-essential' ),
		];
		$excluded_breakpoints = [
			'laptop',
			'tablet_extra',
			'widescreen',
		];

		foreach ( Plugin::$instance->breakpoints->get_active_breakpoints() as $breakpoint_key => $breakpoint_instance ) {
			// Exclude the larger breakpoints from the dropdown selector.
			if ( in_array( $breakpoint_key, $excluded_breakpoints, true ) ) {
				continue;
			}

			$dropdown_options[ $breakpoint_key ] = sprintf(
			/* translators: 1: Breakpoint label, 2: `>` character, 3: Breakpoint value. */
				esc_html__( '%1$s (%2$s %3$dpx)', 'helo-essential' ),
				$breakpoint_instance->get_label(),
				'>',
				$breakpoint_instance->get_value()
			);
		}

		$this->add_control(
			'breakpoint_selector',
			[
				'label'        => esc_html__( 'Breakpoint', 'helo-essential' ),
				'type'         => Controls_Manager::SELECT,
				'description'  => esc_html__( 'Note: Choose at which breakpoint tabs will automatically switch to a vertical (“accordion”) layout.', 'helo-essential' ),
				'options'      => $dropdown_options,
				'default'      => 'mobile',
				'prefix_class' => 'wcf-tabs-',
			]
		);

		$this->end_controls_section();

		//style tabs title
		$this->start_controls_section(
			'section_tabs_style',
			[
				'label' => esc_html__( 'Tabs', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'navigation_width',
			[
				'label'     => esc_html__( 'Navigation Width', 'helo-essential' ),
				'type'      => Controls_Manager::SLIDER,
				'default'   => [
					'unit' => '%',
				],
				'range'     => [
					'%' => [
						'min' => 10,
						'max' => 50,
					],
				],
				'selectors' => [
					'{{WRAPPER}} .tabs-wrapper' => 'width: {{SIZE}}{{UNIT}}',
				],
				'condition' => [
					'tabs_direction' => [ 'row-reverse', 'row' ],
				],
			]
		);

		$this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name'      => 'tab_border',
				'selector'  => '{{WRAPPER}} .helo--tabs',
				'separator' => 'before',
			]
		);

		$this->add_responsive_control(
			'tab_border_radius',
			[
				'label'      => esc_html__( 'Border Radius', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .helo--tabs' => 'border-radius: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();

		// Title Style
		$this->start_controls_section(
			'sec_style_title', [
				'label' => esc_html__( 'Title', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'title_color',
			[
				'label' => esc_html__( 'Color', 'helo-essential' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .title' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'title_typo',
				'selector' => '{{WRAPPER}} .title',
			]
		);

		$this->add_control(
			'title_b_color',
			[
				'label' => esc_html__( 'Border Color', 'helo-essential' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .tab-title' => 'border-bottom-color: {{VALUE}};',
					'{{WRAPPER}} .tabs-wrapper' => 'border-right-color: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'title_border',
				'selector' => '{{WRAPPER}} .tab-title',
			]
		);

		$this->add_responsive_control(
			'title_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'vw', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .tab-title' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'title_margin',
			[
				'label'      => esc_html__( 'Margin', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'vw', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .tabs-wrapper' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		// Number

		$this->add_control(
			'number_highlight',
			[
				'label' => esc_html__( 'Number', 'helo-essential' ),
				'type' => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'number_color',
			[
				'label' => esc_html__( 'Color', 'helo-essential' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .number' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'number_typo',
				'selector' => '{{WRAPPER}} .number',
			]
		);


		$this->end_controls_section();

		// Content Box
		$this->start_controls_section( 'section_box_style', [
			'label' => esc_html__( 'Content', 'helo-essential' ),
			'tab'   => Controls_Manager::TAB_STYLE,
		] );

		$this->add_control(
			'box_color',
			[
				'label' => esc_html__( 'Color', 'helo-essential' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .tab-content' => 'color: {{VALUE}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'box_typo',
				'selector' => '{{WRAPPER}} .tab-content',
			]
		);

		$this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'box_border',
				'selector' => '{{WRAPPER}} .tabs-content-wrapper',
			]
		);

		$this->add_responsive_control(
			'box_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'em', 'rem', 'vw', 'custom' ],
				'selectors'  => [
					'{{WRAPPER}} .tab-content, {{WRAPPER}} .tab-item' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();

		// Hover/Active Style
		$this->start_controls_section(
			'sec_style_active', [
				'label' => esc_html__( 'Hover/Active', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'title_h_color',
			[
				'label' => esc_html__( 'Title Color', 'helo-essential' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .tab-title.active, {{WRAPPER}} .tab-title:hover' => 'color: {{VALUE}};',
				],
			]
		);

		$this->end_controls_section();
	}

	/**
	 * Render tabs widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 * @access protected
	 */
	protected function render() {
		$settings = $this->get_settings_for_display();
		$tabs     = $this->get_settings_for_display( 'tabs' );
		$id_int   = substr( $this->get_id_int(), 0, 3 );

		$this->add_render_attribute( 'wrapper', 'class', 'wcf--tabs helo--tabs' );

		?>

        <div <?php $this->print_render_attribute_string( 'wrapper' ); ?>>
            <div class="tabs-wrapper" role="tablist">
			<?php
				foreach ( $tabs as $index => $item ) :
					$tab_count             = $index + 1;
					$tab_title_setting_key = $this->get_repeater_setting_key( 'tab_title', 'tabs', $index );

					$this->add_render_attribute( $tab_title_setting_key, [
						'id'            => 'tab-title-' . $id_int . $tab_count,
						'class'         => [ 'tab-title', 'tab-desktop-title' ],
						'aria-selected' => 1 === $tab_count ? 'true' : 'false',
						'data-tab'      => $tab_count,
						'role'          => 'tab',
						'tabindex'      => 1 === $tab_count ? '0' : '-1',
						'aria-controls' => 'tab-content-' . $id_int . $tab_count,
					] );
					?>
                    <div <?php $this->print_render_attribute_string( $tab_title_setting_key ); ?>>
                        <div class="content">
	                        <?php
	                        $title = $item['tab_title'];
	                        $dis = $item['tab_content'];
	                        $number = $item['number'];

	                        preg_match_all( '/\[([^\]]*)\]/', $title, $matches );
	                        foreach ( $matches[0] as $key => $value ) {
		                        $title = str_replace( $value, '<span class="highlight">' . $matches[1][ $key ] . '</span>', $title, );
	                        }
	                        ?>
                            <div class="title">
								<span class="number"><?php echo wp_kses_post( $number ); ?></span>
								<span class="title-text"><?php echo wp_kses_post( $title ); ?></span>
                            </div>
							<p class="disc"><?php echo wp_kses_post( $dis ); ?></p>
                        </div>
                    </div>
				<?php endforeach; ?>
            </div>
            <div class="tabs-content-wrapper" role="tablist">
			<?php
				foreach ( $tabs as $index => $item ) :
					$tab_count = $index + 1;
					$hidden = 1 === $tab_count ? 'false' : 'hidden';
					$tab_content_setting_key = $this->get_repeater_setting_key( 'tab_content', 'tabs', $index );
					$this->add_render_attribute( $tab_content_setting_key, [
						'id'              => 'tab-content-' . $id_int . $tab_count,
						'class'           => 'tab-item',
						'data-tab'        => $tab_count,
						'role'            => 'tabpanel',
						'aria-labelledby' => 'tab-title-' . $id_int . $tab_count,
						'tabindex'        => '0',
					] );
					?>
                    <div <?php $this->print_render_attribute_string( $tab_content_setting_key ); ?>>
						<img src="<?php echo $item['content_image']['url'];?>" />
                    </div>
				<?php endforeach; ?>
            </div>
        </div>
		<?php
	}
}
