<?php

namespace HeloEssentialApp\Widgets;

use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Background;
use Elementor\Icons_Manager;
use Elementor\Utils;
use Elementor\Repeater;
use Elementor\Widget_Base;
use Elementor\Controls_Manager;

if ( ! defined( 'ABSPATH' ) ) {
	exit;
} // Exit if accessed directly

/**
 * Image
 *
 * Elementor widget for image.
 *
 * @since 1.0.0
 */
class Side_Offcanvas extends Widget_Base {

	/**
	 * Widget base constructor.
	 *
	 * Initializing the widget base class.
	 *
	 * @param array $data Widget data. Default is an empty array.
	 * @param array|null $args Optional. Widget default arguments. Default is null.
	 *
	 * @throws \Exception If arguments are missing when initializing a full widget
	 *                   instance.
	 *
	 * @since 1.0.0
	 * @access public
	 *
	 */
	public function __construct( $data = [], $args = null ) {
		parent::__construct( $data, $args );
	}

	/**
	 * Retrieve the widget name.
	 *
	 * @return string Widget name.
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 */
	public function get_name() {
		return 'wcf--side-offcanvas';
	}

	/**
	 * Retrieve the widget title.
	 *
	 * @return string Widget title.
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 */
	public function get_title() {
		return wcf_elementor_widget_concat_prefix( 'Side Offcanvas' );
	}

	/**
	 * Retrieve the widget icon.
	 *
	 * @return string Widget icon.
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 */
	public function get_icon() {
		return 'wcf eicon-header';
	}

	/**
	 * Retrieve the list of categories the widget belongs to.
	 *
	 * Used to determine where to display the widget in the editor.
	 *
	 * Note that currently Elementor supports only one category.
	 * When multiple categories passed, Elementor uses the first one.
	 *
	 * @return array Widget categories.
	 * @since 1.0.0
	 *
	 * @access public
	 *
	 */
	public function get_categories() {
		return [ 'weal-coder-addon' ];
	}

	public function get_style_depends() {
		return [ 'wcf-side-offcanvas' ];
	}

	public function get_script_depends() {
		wp_register_script( 'wcf-side-offcanvas', HELO_ESSENTIAL_ASSETS_URL . '/js/widgets/side-offcanvas.js', [ 'jquery' ], false, true );

		return [ 'wcf-side-offcanvas' ];
	}

	/**
	 * Register the widget controls.
	 *
	 * Adds different input fields to allow the user to change and customize the widget settings.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function register_controls() {
		$this->start_controls_section(
			'section_offcanvas',
			[
				'label' => __( 'Offcanvas', 'helo-essential' ),
			]
		);

		$this->add_control(
			'off_open_icon',
			[
				'label'       => esc_html__( 'Offcanvas Open Icon', 'helo-essential' ),
				'type'        => Controls_Manager::ICONS,
				'skin'        => 'inline',
				'label_block' => false,
				'default'     => [
					'value'   => 'fas fa-circle',
					'library' => 'fa-solid',
				],
			]
		);

		$this->add_control(
			'off_close_icon',
			[
				'label'   => esc_html__( 'Offcanvas Close Icon', 'helo-essential' ),
				'type'    => Controls_Manager::ICONS,
				'skin'        => 'inline',
				'label_block' => false,
				'default' => [
					'value'   => 'fas fa-circle',
					'library' => 'fa-solid',
				],
			]
		);

		$this->add_responsive_control(
			'off_position',
			[
				'label'   => esc_html__( 'Offcanvas Position', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'default' => 'right',
				'options' => [
					'right' => esc_html__( 'Right', 'helo-essential' ),
					'left'  => esc_html__( 'Left', 'helo-essential' ),
				],
				'separator' => 'before',
			]
		);

		$this->add_control(
			'text_align',
			[
				'label'     => esc_html__( 'Alignment', 'helo-essential' ),
				'type'      => Controls_Manager::CHOOSE,
				'options'   => [
					'start'  => [
						'title' => esc_html__( 'Left', 'helo-essential' ),
						'icon'  => 'eicon-text-align-left',
					],
					'center' => [
						'title' => esc_html__( 'Center', 'helo-essential' ),
						'icon'  => 'eicon-text-align-center',
					],
					'end'    => [
						'title' => esc_html__( 'Right', 'helo-essential' ),
						'icon'  => 'eicon-text-align-right',
					],
				],
				'toggle'    => true,
				'selectors' => [
					'.offcanvas-{{ID}}.side--offcanvas'     => 'text-align: {{VALUE}};',
					'.offcanvas-{{ID}} .side--off-media ul' => 'justify-content: {{VALUE}};',
				],
			]
		);

		$this->end_controls_section();

		// Logo
		$this->start_controls_section(
			'section_logo',
			[
				'label' => __( 'Logo', 'helo-essential' ),
			]
		);

		$this->add_responsive_control(
			'side_off_logo',
			[
				'label'   => esc_html__( 'Logo', 'helo-essential' ),
				'type'    => Controls_Manager::SELECT,
				'default' => 'site',
				'options' => [
					'site'   => esc_html__( 'Site Logo', 'helo-essential' ),
					'custom' => esc_html__( 'Custom', 'helo-essential' ),
				],
			]
		);

		$this->add_control(
			'custom_logo',
			[
				'label'     => esc_html__( 'Logo', 'helo-essential' ),
				'type'      => Controls_Manager::MEDIA,
				'default'   => [
					'url' => Utils::get_placeholder_image_src(),
				],
				'condition' => [ 'side_off_logo' => 'custom' ]
			]
		);

		$this->end_controls_section();

		$this->start_controls_section(
			'section_content',
			[
				'label' => esc_html__( 'Content', 'helo-essential' ),
			]
		);

		$this->add_control(
			'desc',
			[
				'label'       => esc_html__( 'Description', 'helo-essential' ),
				'type'        => Controls_Manager::TEXTAREA,
				'rows'        => 7,
				'default'     => esc_html__( 'Default description', 'helo-essential' ),
				'placeholder' => esc_html__( 'Type your description here', 'helo-essential' ),
			]
		);

		$this->end_controls_section();


		// Gallery
		$this->start_controls_section(
			'section_gallery',
			[
				'label' => esc_html__( 'Gallery', 'helo-essential' ),
			]
		);

		$this->add_control(
			'gallery_title',
			[
				'label'       => esc_html__( 'Gallery Title', 'helo-essential' ),
				'type'        => Controls_Manager::TEXT,
				'default'     => esc_html__( 'Gallery', 'helo-essential' ),
				'placeholder' => esc_html__( 'Type your cart text here', 'helo-essential' ),
			]
		);

		$repeater = new \Elementor\Repeater();

		$repeater->add_control(
			'image',
			[
				'label' => esc_html__( 'Image', 'helo-essential' ),
				'type' =>Controls_Manager::MEDIA,
				'default' => [
					'url' => Utils::get_placeholder_image_src(),
				],
			]
		);

		$repeater->add_control(
			'gallery_link',
			[
				'label' => esc_html__( 'Link', 'helo-essential' ),
				'type' => Controls_Manager::URL,
				'options' => [ 'url', 'is_external', 'nofollow' ],
				'default' => [
					'url' => '#',
					'is_external' => true,
					'nofollow' => true,
				],
				'label_block' => true,
			]
		);

		$this->add_control(
			'gallery_list',
			[
				'label'  => esc_html__( 'Gallery Items', 'helo-essential' ),
				'type'   => Controls_Manager::REPEATER,
				'fields' => $repeater->get_controls(),
			]
		);

		$this->add_control(
			'gallery_icon',
			[
				'label' => esc_html__( 'Icon', 'helo-essential' ),
				'type' => Controls_Manager::ICONS,
				'skin' => 'inline',
				'label_block' => false,
				'default' => [
					'value' => 'fab fa-instagram',
					'library' => 'fa-solid',
				],
			]
		);

		$this->end_controls_section();


		// Social Media
		$this->start_controls_section(
			'section_media',
			[
				'label' => esc_html__( 'Social Media', 'helo-essential' ),
			]
		);

		$this->add_control(
			'social_title',
			[
				'label'       => esc_html__( 'Social Title', 'helo-essential' ),
				'type'        => Controls_Manager::TEXT,
				'default'     => esc_html__( 'Social Media', 'helo-essential' ),
				'placeholder' => esc_html__( 'Type your social text here', 'helo-essential' ),
			]
		);

		$repeater = new \Elementor\Repeater();

		$repeater->add_control(
			'website_link',
			[
				'label'       => esc_html__( 'Link', 'helo-essential' ),
				'type'        => Controls_Manager::URL,
				'options'     => [ 'url', 'is_external', 'nofollow' ],
				'default'     => [
					'url'         => '#',
					'is_external' => true,
					'nofollow'    => true,
				],
				'label_block' => true,
			]
		);

		$repeater->add_control(
			'icon',
			[
				'label'   => esc_html__( 'Icon', 'helo-essential' ),
				'type'    => Controls_Manager::ICONS,
				'default' => [
					'value'   => 'fab fa-facebook-f',
					'library' => 'fa-solid',
				],
			]
		);

		$this->add_control(
			'social_list',
			[
				'label'  => esc_html__( 'Social Items', 'helo-essential' ),
				'type'   => Controls_Manager::REPEATER,
				'fields' => $repeater->get_controls(),
			]
		);

		$this->end_controls_section();


		// Layout Style
		$this->start_controls_section(
			'style_offcanvas',
			[
				'label' => esc_html__( 'Side Offcanvas', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_responsive_control(
			'offcanvas_width',
			[
				'label'      => esc_html__( 'Width', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 600,
					],
				],
				'selectors'  => [
					'.offcanvas-{{ID}}.side--offcanvas' => '--width: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Background::get_type(),
			[
				'name'     => 'offcanvas_bg',
				'types'    => [ 'classic', 'gradient' ],
				'selector' => '.offcanvas-{{ID}}.side--offcanvas',
			]
		);

		$this->add_responsive_control(
			'offcanvas_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px' ],
				'selectors'  => [
					'.offcanvas-{{ID}} .side--off-inner' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();


		// Icons Style
		$this->start_controls_section(
			'style_icons',
			[
				'label' => esc_html__( 'Open & Close Icon', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'open_icon_color',
			[
				'label' => esc_html__( 'Open Icon Color', 'helo-essential' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .side--off-open' => 'color: {{VALUE}}; fill: {{VALUE}}',
				],
			]
		);

		$this->add_responsive_control(
			'open_icon_size',
			[
				'label'      => esc_html__( 'Open Icon Size', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 300,
					],
				],
				'selectors'  => [
					'{{WRAPPER}} .side--off-open' => 'font-size: {{SIZE}}{{UNIT}};',
				],
			]
		);



        // Close Icon
		$this->add_control(
			'close_icon_heading',
			[
				'label' => esc_html__( 'Close Icon', 'helo-essential' ),
				'type' => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'close_icon_color',
			[
				'label' => esc_html__( 'Color', 'helo-essential' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'.offcanvas-{{ID}} .side--off-close' => 'color: {{VALUE}}; fill: {{VALUE}}',
				],
			]
		);

		$this->add_responsive_control(
			'close_icon_size',
			[
				'label'      => esc_html__( 'Icon Size', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 300,
						'step' => 1,
					],
				],
				'selectors'  => [
					'.offcanvas-{{ID}} .side--off-close' => 'font-size: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'close_icon_top',
			[
				'label'      => esc_html__( 'Position Top', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 100,
					],
				],
				'selectors'  => [
					'.offcanvas-{{ID}} .side--off-close' => 'top: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'close_icon_right',
			[
				'label'      => esc_html__( 'Position Right', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px', '%' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 600,
					],
				],
				'selectors'  => [
					'.offcanvas-{{ID}} .side--off-close' => 'right: {{SIZE}}{{UNIT}};',
				],
			]
		);

		// Hover
		$this->add_control(
			'icon_hover_heading',
			[
				'label' => esc_html__( 'Hover', 'helo-essential' ),
				'type' => Controls_Manager::HEADING,
				'separator' => 'before',
			]
		);

		$this->add_control(
			'open_icon_h_color',
			[
				'label' => esc_html__( 'Open Icon Color', 'helo-essential' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .side--off-open:hover' => 'color: {{VALUE}}; fill: {{VALUE}}',
				],
			]
		);

		$this->add_control(
			'close_icon_h_color',
			[
				'label' => esc_html__( 'Close Icon Color', 'helo-essential' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'.offcanvas-{{ID}} .side--off-close:hover' => 'color: {{VALUE}}; fill: {{VALUE}}',
				],
			]
		);

		$this->end_controls_section();


		// Logo Style
		$this->start_controls_section(
			'style_logo',
			[
				'label' => esc_html__( 'Logo', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_responsive_control(
			'logo_width',
			[
				'label'      => esc_html__( 'Width', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 300,
						'step' => 1,
					],
				],
				'selectors'  => [
					'.offcanvas-{{ID}} .side--off-logo img' => 'width: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'logo_height',
			[
				'label'      => esc_html__( 'Height', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min'  => 0,
						'max'  => 300,
						'step' => 1,
					],
				],
				'selectors'  => [
					'.offcanvas-{{ID}} .side--off-logo img' => 'height: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();


		// Title
		$this->start_controls_section(
			'style_title',
			[
				'label' => esc_html__( 'Title', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'title_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'.offcanvas-{{ID}} .side--off-title' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'title_typo',
				'selector' => '.offcanvas-{{ID}} .side--off-title',
			]
		);

		$this->add_responsive_control(
			'title_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px' ],
				'selectors'  => [
					'.offcanvas-{{ID}} .side--off-title' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();


		// Content
		$this->start_controls_section(
			'style_desc',
			[
				'label' => esc_html__( 'Description', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'desc_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'.offcanvas-{{ID}} .side--off-desc' => 'color: {{VALUE}}',
				],
			]
		);

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name'     => 'desc_typo',
				'selector' => '.offcanvas-{{ID}} .side--off-desc',
			]
		);

		$this->add_responsive_control(
			'desc_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px' ],
				'selectors'  => [
					'.offcanvas-{{ID}} .side--off-desc' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();


		// Gallery
		$this->start_controls_section(
			'style_gallery',
			[
				'label' => esc_html__( 'Gallery', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_responsive_control(
			'gallery_width',
			[
				'label'      => esc_html__( 'Width', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'.offcanvas-{{ID}} .items .item' => 'width: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'gallery_height',
			[
				'label'      => esc_html__( 'Height', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 100,
					],
				],
				'selectors'  => [
					'.offcanvas-{{ID}} .items .item' => 'height: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'gallery_row_gap',
			[
				'label'      => esc_html__( 'Row Gap', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 50,
					],
				],
				'selectors'  => [
					'.offcanvas-{{ID}} .items' => 'row-gap: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'gallery_col_gap',
			[
				'label'      => esc_html__( 'Column Gap', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 50,
					],
				],
				'selectors'  => [
					'.offcanvas-{{ID}} .items' => 'column-gap: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'gallery_icon_color',
			[
				'label' => esc_html__( 'Icon Color', 'helo-essential' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'.offcanvas-{{ID}} .side--off-gallery .item span' => 'fill: {{VALUE}}; color: {{VALUE}}',
				],
			]
		);

		$this->add_responsive_control(
			'gallery_icon_size',
			[
				'label'      => esc_html__( 'Icon Size', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 50,
					],
				],
				'selectors'  => [
					'.offcanvas-{{ID}} .side--off-gallery .item span' => 'font-size: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_control(
			'gallery_overlay',
			[
				'label' => esc_html__( 'Overlay Color', 'helo-essential' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'.offcanvas-{{ID}} .side--off-gallery .item span' => 'background: {{VALUE}}',
				],
			]
		);

		$this->add_responsive_control(
			'gallery_padding',
			[
				'label'      => esc_html__( 'Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px' ],
				'selectors'  => [
					'.offcanvas-{{ID}} .side--off-gallery' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();

		// Social Media
		$this->start_controls_section(
			'style_media',
			[
				'label' => esc_html__( 'Social Media', 'helo-essential' ),
				'tab'   => Controls_Manager::TAB_STYLE,
			]
		);

		$this->add_control(
			'media_color',
			[
				'label'     => esc_html__( 'Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'.offcanvas-{{ID}} .side--off-media li a' => 'color: {{VALUE}}; fill: {{VALUE}};',
				],
			]
		);

		$this->add_control(
			'media_h_color',
			[
				'label'     => esc_html__( 'Hover Color', 'helo-essential' ),
				'type'      => Controls_Manager::COLOR,
				'selectors' => [
					'.offcanvas-{{ID}} .side--off-media li a:hover' => 'color: {{VALUE}}; fill: {{VALUE}};',
				],
			]
		);

		$this->add_responsive_control(
			'media_size',
			[
				'label'      => esc_html__( 'Icon Size', 'helo-essential' ),
				'type'       => Controls_Manager::SLIDER,
				'size_units' => [ 'px' ],
				'range'      => [
					'px' => [
						'min' => 0,
						'max' => 50,
					],
				],
				'selectors'  => [
					'.offcanvas-{{ID}} .side--off-media li a' => 'font-size: {{SIZE}}{{UNIT}};',
				],
			]
		);

		$this->add_responsive_control(
			'media_padding',
			[
				'label'      => esc_html__( 'Icon Padding', 'helo-essential' ),
				'type'       => Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px' ],
				'selectors'  => [
					'.offcanvas-{{ID}} .side--off-media li a' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};',
				],
			]
		);

		$this->end_controls_section();
	}

	/**
	 * Render the widget output on the frontend.
	 *
	 * Written in PHP and used to generate the final HTML.
	 *
	 * @since 1.0.0
	 *
	 * @access protected
	 */
	protected function render() {
		$settings = $this->get_settings_for_display();

		$offcanvas_gallery = $settings['gallery_list'];
		$offcanvas_social  = $settings['social_list'];

		// Wrapper
		$this->add_render_attribute( 'wrapper', [
			'class'   => [ 'side--offcanvas offcanvas-' . $this->get_id() . ' offcanvas-' . $settings['off_position'] ],
			'data-id' => $this->get_id()
		] );
		?>
        <div class="side--off-btn">
            <button class="side--off-open" data-id="<?php echo $this->get_id(); ?>">
	            <?php Icons_Manager::render_icon( $settings['off_open_icon'], [ 'aria-hidden' => 'true' ] ); ?>
            </button>
        </div>

        <div <?php $this->print_render_attribute_string( 'wrapper' ); ?>>
            <div class="side--off-inner">
                <button class="side--off-close">
	                <?php Icons_Manager::render_icon( $settings['off_close_icon'], [ 'aria-hidden' => 'true' ] ); ?>
                </button>

                <div class="side--off-logo">
					<?php
					if ( 'site' === $settings['side_off_logo'] ) {
						the_custom_logo();
					} else {
						if ( $settings['custom_logo']['url'] != '' ) {
							?>
                            <a href="<?php echo esc_url( home_url( '/' ) ); ?>">
                                <img src="<?php echo esc_url( $settings['custom_logo']['url'] ); ?>"
                                     alt="<?php echo esc_attr__( 'Site Logo', 'helo-essential' ) ?>">
                            </a>
							<?php
						}
					}
					?>
                </div>

				<?php
				if ( $settings['desc'] ) {
					?>
                    <div class="side--off-desc">
						<?php echo wp_kses_post( wpautop( $settings['desc'] ) ); ?>
                    </div>
					<?php
				}
				?>

				<?php if ( ! empty( $offcanvas_gallery ) ) { ?>
                    <div class="side--off-gallery">
                        <h2 class="side--off-title"><?php echo esc_html( $settings['gallery_title'] ); ?></h2>
                        <div class="items">
							<?php foreach ( $offcanvas_gallery as $g_index => $gallery_item ) {
							    $g_link = 'g_link_' . $g_index;

								if ( ! empty( $gallery_item['gallery_link']['url'] ) ) {
									$this->add_link_attributes( $g_link, $gallery_item['gallery_link'] );
								}
							    ?>
                                <div class="item">
                                    <a <?php $this->print_render_attribute_string( $g_link ); ?>>
                                        <img
                                                src="<?php echo esc_url( $gallery_item['image']['url'] ); ?>"
                                                alt="<?php echo esc_attr( $settings['gallery_title'] ); ?>">
                                        <span><?php Icons_Manager::render_icon( $settings['gallery_icon'], [ 'aria-hidden' => 'true' ] ); ?></span></a>
                                </div>
							<?php } ?>
                        </div>
                    </div>
				<?php } ?>

				<?php if ( ! empty( $offcanvas_social ) ) { ?>
                    <div class="side--off-media">
                        <h2 class="side--off-title"><?php echo esc_html( $settings['social_title'] ); ?></h2>
                        <ul>
							<?php foreach ( $offcanvas_social as $social_item ) { ?>
                                <li><a href="<?php echo $social_item['website_link']['url']; ?>">
										<?php Icons_Manager::render_icon( $social_item['icon'], [ 'aria-hidden' => 'true' ] ); ?>
                                    </a></li>
							<?php } ?>
                        </ul>
                    </div>
				<?php } ?>
            </div>
        </div>
		<?php
	}
}
