<?php

namespace helo\Core;

class Theme_Setup
{
    public $theme_data_key = 'helo_theme_data';
    public $theme_slug = 'helo';

    public $api_params = [];    
   
    /**
     * register default hooks and actions for WordPress
     * @return
     */
    public function register()
    {
        add_action( 'admin_menu', [ $this,'register_theme_admin_menu' ] );
        add_action( 'after_setup_theme' , array( $this, 'setup' ) );
        add_action( 'admin_init' , [ $this , 'theme_activated_options' ]);       
        add_action( 'after_switch_theme' , [ $this , 'theme_activated' ]);       
        // add extra html tags for smooth
        add_action( 'wp_ajax_wcf_user_guide_ls_checker' , [ $this,'wcf_user_guide'] );     
        add_action( 'wp_ajax_wcf_user_guide_ls_remove' , [ $this,'license_deactivate'] );        
        
    }
    
    function register_theme_admin_menu() {		
		
		if(!defined('HELO_ESSENTIAL')) {
            add_menu_page(
                esc_html__( 'Helo Theme', 'helo' ),
                esc_html__( 'Helo Theme','helo'),
                'manage_options',
                'wcf-helo-theme-parent',
                [$this,'_render_dashboard'],
                null,
                6
            );	
        }
		
	}
    
    public function theme_activated(){
       
        if( isset( $_GET['activated'] ) ) {
            wp_safe_redirect( admin_url('admin.php?page=wcf-helo-theme-parent') );
            exit;
        }
	}
	
	public function _render_dashboard(){
		echo '<div id="wcf-user-guider-dashboard" class="wcf-user-guider-dashboard"></div>';
	}
    
    public function wcf_user_guide(){   
    
        if ( !wp_verify_nonce( $_REQUEST['nonce'] , "wcf_user_guider_helo_secure" ) ) {
            exit("No naughty business please");
        } 
        
        $code           = isset($_REQUEST['code']) ? sanitize_text_field( $_REQUEST['code'] ) : get_option('helo_lic_Key');
        $user_submitted = isset($_REQUEST['user_submitted']) ? sanitize_text_field( $_REQUEST['user_submitted'] ) : false;
        $email          = isset($_REQUEST['email']) ? sanitize_email( $_REQUEST['email'] ) : get_option('admin_email');
        
        if($user_submitted == 'no'){
            wp_send_json_success( [ 'msg' => get_option($this->theme_slug . '_license_key_status') , 'code' => get_option('helo_lic_Key'), 'email' => get_option('helo_lic_email') !='' ? get_option('helo_lic_email'): get_option('admin_email')] );
        }
        $url = add_query_arg( array(
            'edd_action' => 'check_license',
            'item_id' => HELO_PRODUCT_ID,
            'license' =>  $code,
            'url'         => home_url(),
		    'environment' => function_exists( 'wp_get_environment_type' ) ? wp_get_environment_type() : 'production',
        ), HELO_PRODUCT_DOMAIN );
	
		$args = [
			'sslverify'   => false,
			'timeout'     => 120,
			'redirection' => 5,
			'cookies'     => array(),
			'headers'     => array(
				'Accept' => 'application/json',
			)
		];

		$response = wp_remote_get( $url, $args );

		if ( ( ! is_wp_error( $response ) ) && ( 200 === wp_remote_retrieve_response_code( $response ) ) ) {
			$responseBody = json_decode( $response['body'] );
			if ( json_last_error() === JSON_ERROR_NONE ) {				
				if(isset($responseBody->success)){
				    if($user_submitted == 'yes'){
                        update_option( $this->theme_slug . '_license_key_status', $responseBody->license);
                        update_option( $this->theme_slug . '_license_key',$code );
                        update_option( 'helo_lic_Key',$code );    
                        $this->update_readme($code, $email, $responseBody->license);
                        if(isset($_REQUEST['email'])){                        
                            update_option( 'helo_lic_email',$email );  
                        }
                                             
                    }
					wp_send_json_success( [ 
					'msg' => $responseBody->license , 
					'code' => $code, 
					'email' => get_option('admin_email')
					] );
				} else {
					wp_send_json_error( [ 'msg' => esc_html__( 'Contact Author', 'helo' ) ] );
				}

			}
		}
		wp_send_json_error( [ 'msg' => esc_html__( 'Unknown error', 'helo' ) ] );
        wp_die();
    }
    
    public function update_readme($licenseCode, $licenseEmail, $status){
        $return = [];
        try{
            $return['lic'] = $licenseCode;
            $return['helo_lic_Key'] = $licenseCode;
            $return['helo_lic_email'] = $licenseEmail;
            $return['helo_license_key_status'] = $status;
            delete_user_meta( 1 , $this->theme_data_key );
            update_user_meta( 1 , $this->theme_data_key , $return );
        }catch(\Exception $e){            
        }
       
    }
    
    public function license_deactivate(){
    
        if ( !wp_verify_nonce( $_REQUEST['nonce'] , "wcf_user_guider_helo_secure" ) ) {
            exit("No naughty business please");
        }
        
        $return['path'] = admin_url( 'admin.php?page=wcf-helo-theme-parent');
        if(isset($_REQUEST['status']) && $_REQUEST['status'] == 'request'){
        
            $return['status']    = 'deactivated';
            $return['next_step'] = 'deactivated';
            $return['license']   = get_option($this->theme_slug.'_lic_Key');
            $return['email']     = get_option('helo_lic_email');
            
            if( get_option($this->theme_slug.'_lic_Key') && $this->theme_slug . '_license_key_status') {
                
                $this->api_params = array(
                    'edd_action'  => 'deactivate_license',
                    'license'     => get_option( $this->theme_slug.'_lic_Key' ),                    
                    'item_id'     => HELO_PRODUCT_ID,
                    'url'         => home_url(),
                    'environment' => function_exists( 'wp_get_environment_type' ) ? wp_get_environment_type() : 'production',	
                );
                
                // Call the custom API.			
                $response = wp_remote_post( HELO_PRODUCT_DOMAIN , array( 'body' => $this->api_params, 'timeout' => 60 , 'sslverify' => false ) );		
                if ( is_wp_error( $response ) ) {                	
                  $return['status'] = wp_remote_retrieve_response_code($response);                 
                }	
                
                $license_data = json_decode( wp_remote_retrieve_body( $response ) , true );    
                
                if(isset($license_data['license'])){		                  
                    $return['next_step']    = 'done';
                    $return['status']       = 'done';                    
                    update_option( 'helo_lic_Key', '' );	                                
                    delete_transient( 'helo_theme_service_pass_cache' );	                                
                    delete_user_meta( 1 ,'helo_theme_data' );	                                
                    update_option( $this->theme_slug . '_license_key_status', $license_data['license'] );
                }else{
                    $return['msg']       = $license_data['license'];   
                    $return['next_step']    = 'done';
                }						   
                
            }else{
                $return['msg']       = 'removed';
                $return['next_step'] = 'done';
            }
        }
		
        wp_send_json($return);
        wp_die();
    }
    
    function theme_activated_options() {
        $is_admin      = current_user_can( 'manage_options' );       
       
        if ( (current_user_can( 'administrator' ) || $is_admin) && isset($_GET['page']) && $_GET['page'] === 'wcf-helo-theme-parent') {        
            wp_register_script( 'helo-configure', HELO_JS . '/user-configure.js', array( 'jquery' ), time(), true );  
            $params = array(
                'ajaxurl'     => admin_url('admin-ajax.php'),
                'ajax_nonce'  => wp_create_nonce('wcf_user_guider_helo_secure'),
                'update_path' => admin_url('admin.php?page=helo-theme#tab=theme-update'),
                'license_domain' => HELO_PRODUCT_DOMAIN,
                'download_id' => HELO_PRODUCT_ID, 
                'home_url'      => home_url(),
                'demo_active' => function_exists('helo_option') ? helo_option('theme_demo_activate', true) : true
            );
            wp_localize_script( 'helo-configure', 'ajax_object', $params );
            wp_enqueue_script( 'helo-configure' );    
        }
    }    

    public function setup(){
        /*
        * You can activate this if you're planning to build a multilingual theme
        */        
        load_theme_textdomain( 'helo', get_template_directory() . '/languages' );
        add_theme_support( 'automatic-feed-links' );
        add_theme_support( 'title-tag' );
        add_theme_support( 'post-thumbnails' );
        add_theme_support( 'post-formats' , [
           'standard', 'image', 'video', 'audio'
        ]);
        
        //Thumbnail size 1200 x 780
        set_post_thumbnail_size(1200, 780, ['center', 'center']);

  
        add_theme_support( 'html5', array(
              'search-form',
              'comment-form',
              'comment-list',
              'gallery',
              'caption',
        ) );
        
        remove_theme_support( 'widgets-block-editor' );
        /*
        Register all your menus here
        */
        register_nav_menus( array(        
            'primary'     => esc_html__( 'Primary', 'helo' )    
        ) );
        
    }

    public function is_elementor_builder(){

	    if ( isset( $_GET['preview'] ) && $_GET['preview'] == true ) {
		    return false;
	    }

        if( ( isset($_GET['wcf-edit']) && $_GET['wcf-edit'] == '1' )) {
            return true;
        }
    
        return false;
    }
     
    
}
